@@ -22,7 +22,6 @@ CChan::CChan(const CString& sName, CUser* pUser, bool bInConfig) {
 
 	m_bInConfig = bInConfig;
 	m_Nick.SetUser(pUser);
-	m_bAutoCycle = m_pUser->AutoCycle();
 	m_bDetached = false;
 	m_uBufferCount = m_pUser->GetBufferCount();
 	m_bKeepBuffer = m_pUser->KeepBuffer();
@@ -46,6 +45,7 @@ void CChan::Reset() {
 	m_sTopicOwner = "";
 	m_ulTopicDate = 0;
 	m_ulCreationDate = 0;
+	m_Nick.Reset();
 	ClearNicks();
 	ResetJoinTries();
 }
@@ -60,11 +60,9 @@ bool CChan::WriteConfig(CFile& File) {
 	if (m_pUser->GetBufferCount() != GetBufferCount())
 		File.Write("\t\tBuffer     = " + CString(GetBufferCount()) + "\n");
 	if (m_pUser->KeepBuffer() != KeepBuffer())
-		File.Write("\t\tKeepBuffer = " + CString((KeepBuffer()) ? "true" : "false") + "\n");
+		File.Write("\t\tKeepBuffer = " + CString(KeepBuffer()) + "\n");
 	if (IsDetached())
 		File.Write("\t\tDetached   = true\n");
-	if (m_pUser->AutoCycle() != AutoCycle())
-		File.Write("\t\tAutoCycle  = " + CString((AutoCycle()) ? "true" : "false") + "\n");
 	if (!GetKey().empty()) { File.Write("\t\tKey        = " + GetKey() + "\n"); }
 	if (!GetDefaultModes().empty()) { File.Write("\t\tModes      = " + GetDefaultModes() + "\n"); }
 
@@ -76,7 +74,6 @@ void CChan::Clone(CChan& chan) {
 	// We assume that m_sName and m_pUser are equal
 	SetBufferCount(chan.GetBufferCount());
 	SetKeepBuffer(chan.KeepBuffer());
-	SetAutoCycle(chan.AutoCycle());
 	SetKey(chan.GetKey());
 	SetDefaultModes(chan.GetDefaultModes());
 
@@ -195,9 +192,9 @@ CString CChan::GetModeForNames() const {
 
 	for (map<unsigned char, CString>::const_iterator it = m_musModes.begin(); it != m_musModes.end(); it++) {
 		if (it->first == 's') {
-		    sMode = "@";
+			sMode = "@";
 		} else if ((it->first == 'p') && sMode.empty()){
-		    sMode = "*";
+			sMode = "*";
 		}
 	}
 
@@ -207,7 +204,6 @@ CString CChan::GetModeForNames() const {
 void CChan::SetModes(const CString& sModes) {
 	m_musModes.clear();
 	m_uLimit = 0;
-	m_sCurKey = "";
 	ModeChange(sModes);
 }
 
@@ -274,7 +270,7 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 							IncPermCount(uPerm);
 						}
 
-						if (pNick->GetNick().CaseCmp(m_pUser->GetCurNick()) == 0) {
+						if (pNick->GetNick().Equals(m_pUser->GetCurNick())) {
 							AddPerm(uPerm);
 						}
 					} else {
@@ -282,7 +278,7 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 							DecPermCount(uPerm);
 						}
 
-						if (pNick->GetNick().CaseCmp(m_pUser->GetCurNick()) == 0) {
+						if (pNick->GetNick().Equals(m_pUser->GetCurNick())) {
 							RemPerm(uPerm);
 						}
 					}
@@ -331,9 +327,17 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 					break;
 			}
 
-			if (uMode == M_Key) {
-				m_sCurKey = (bAdd) ? sArg : "";
+#ifdef _MODULES
+			bool bNoChange;
+			if (bList) {
+				bNoChange = false;
+			} else if (bAdd) {
+				bNoChange = HasMode(uMode) && GetModeArg(uMode) == sArg;
+			} else {
+				bNoChange = !HasMode(uMode);
 			}
+			MODULECALL(OnMode(*pOpNick, *this, uMode, sArg, bAdd, bNoChange), m_pUser, NULL, );
+#endif
 
 			if (!bList) {
 				(bAdd) ? AddMode(uMode, sArg) : RemMode(uMode, sArg);
@@ -349,10 +353,6 @@ CString CChan::GetOptions() const {
 		sRet += (sRet.empty()) ? "Detached" : ", Detached";
 	}
 
-	if (AutoCycle()) {
-		sRet += (sRet.empty()) ? "AutoCycle" : ", AutoCycle";
-	}
-
 	if (KeepBuffer()) {
 		sRet += (sRet.empty()) ? "KeepBuffer" : ", KeepBuffer";
 	}
@@ -377,10 +377,6 @@ bool CChan::HasMode(unsigned char uMode) const {
 }
 
 bool CChan::AddMode(unsigned char uMode, const CString& sArg) {
-	if (HasMode(uMode)) {
-		return false;
-	}
-
 	m_musModes[uMode] = sArg;
 	return true;
 }
@@ -409,10 +405,6 @@ void CChan::ClearNicks() {
 }
 
 int CChan::AddNicks(const CString& sNicks) {
-	if (IsOn()) {
-		return 0;
-	}
-
 	int iRet = 0;
 	CString sCurNick;
 
@@ -475,7 +467,7 @@ bool CChan::AddNick(const CString& sNick) {
 		}
 	}
 
-	if (pNick->GetNick().CaseCmp(m_pUser->GetCurNick()) == 0) {
+	if (pNick->GetNick().Equals(m_pUser->GetCurNick())) {
 		for (CString::size_type i = 0; i < sPrefix.length(); i++) {
 			AddPerm(sPrefix[i]);
 		}
@@ -486,8 +478,8 @@ bool CChan::AddNick(const CString& sNick) {
 	return true;
 }
 
-unsigned int CChan::GetPermCount(unsigned char uPerm) {
-	map<unsigned char, unsigned int>::iterator it = m_muuPermCount.find(uPerm);
+unsigned int CChan::GetPermCount(unsigned char uPerm) const {
+	map<unsigned char, unsigned int>::const_iterator it = m_muuPermCount.find(uPerm);
 	return (it == m_muuPermCount.end()) ? 0 : it->second;
 }
 
@@ -530,13 +522,6 @@ bool CChan::RemNick(const CString& sNick) {
 
 	delete it->second;
 	m_msNicks.erase(it);
-	CNick* pNick = m_msNicks.begin()->second;
-
-	if ((m_msNicks.size() == 1) && (!pNick->HasPerm(Op)) && (pNick->GetNick().CaseCmp(m_pUser->GetCurNick()) == 0)) {
-		if (AutoCycle()) {
-			Cycle();
-		}
-	}
 
 	return true;
 }
@@ -586,7 +571,7 @@ void CChan::SendBuffer(CClient* pClient) {
 		const vector<CString>& vsBuffer = GetBuffer();
 
 		if (vsBuffer.size()) {
-			m_pUser->PutUser(":***!znc@znc.com PRIVMSG " + GetName() + " :Buffer Playback...", pClient);
+			m_pUser->PutUser(":***!znc@znc.in PRIVMSG " + GetName() + " :Buffer Playback...", pClient);
 
 			for (unsigned int a = 0; a < vsBuffer.size(); a++) {
 				m_pUser->PutUser(vsBuffer[a], pClient);
@@ -596,7 +581,7 @@ void CChan::SendBuffer(CClient* pClient) {
 				ClearBuffer();
 			}
 
-			m_pUser->PutUser(":***!znc@znc.com PRIVMSG " + GetName() + " :Playback Complete.", pClient);
+			m_pUser->PutUser(":***!znc@znc.in PRIVMSG " + GetName() + " :Playback Complete.", pClient);
 		}
 	}
 }
