@@ -11,7 +11,7 @@
 
 #include "FileUtils.h"
 #include "Nick.h"
-#include "String.h"
+#include "ZNCString.h"
 #include <map>
 #include <set>
 #include <vector>
@@ -112,7 +112,6 @@ public:
 	void SetDefaultModes(const CString& s) { m_sDefaultModes = s; }
 	void SetBufferCount(unsigned int u) { m_uBufferCount = u; }
 	void SetKeepBuffer(bool b) { m_bKeepBuffer = b; }
-	void SetAutoCycle(bool b) { m_bAutoCycle = b; }
 	void SetWhoDone(bool b = true) { m_bWhoDone = b; }
 	void SetDetached(bool b = true) { m_bDetached = b; }
 	void SetInConfig(bool b) { m_bInConfig = b; }
@@ -127,11 +126,11 @@ public:
 	bool HasMode(unsigned char uMode) const;
 	CString GetOptions() const;
 	CString GetModeArg(unsigned char uMode) const;
-	unsigned int GetPermCount(unsigned char uPerm);
+	unsigned int GetPermCount(unsigned char uPerm) const;
 	bool IsOn() const { return m_bIsOn; }
 	const CString& GetName() const { return m_sName; }
 	const map<unsigned char, CString>& GetModes() const { return m_musModes; }
-	const CString& GetKey() const { return (!m_sCurKey.empty()) ? m_sCurKey : m_sKey; }
+	const CString& GetKey() const { return m_sKey; }
 	unsigned int GetLimit() const { return m_uLimit; }
 	const CString& GetTopic() const { return m_sTopic; }
 	const CString& GetTopicOwner() const { return m_sTopicOwner; }
@@ -143,12 +142,11 @@ public:
 	unsigned int GetNickCount() const { return m_msNicks.size(); }
 	unsigned int GetBufferCount() const { return m_uBufferCount; }
 	bool KeepBuffer() const { return m_bKeepBuffer; }
-	bool AutoCycle() const { return m_bAutoCycle; }
 	bool IsDetached() const { return m_bDetached; }
 	bool InConfig() const { return m_bInConfig; }
 	unsigned long GetCreationDate() const { return m_ulCreationDate; }
 	bool IsDisabled() const { return m_bDisabled; }
-	unsigned int GetJoinTries() { return m_uJoinTries; }
+	unsigned int GetJoinTries() const { return m_uJoinTries; }
 	// !Getters
 private:
 protected:
@@ -156,12 +154,10 @@ protected:
 	bool				m_bIsOn;
 	bool				m_bWhoDone;
 	bool				m_bKeepBuffer;
-	bool				m_bAutoCycle;
 	bool				m_bInConfig;
 	bool				m_bDisabled;
 	CString				m_sName;
 	CString				m_sKey;
-	CString				m_sCurKey;
 	CString				m_sTopic;
 	CString				m_sTopicOwner;
 	unsigned long		m_ulTopicDate;
