@@ -65,17 +65,7 @@ protected:
 
 class CClient : public Csock {
 public:
-	CClient() : Csock() {
-		InitClient();
-	}
-
 	CClient(const CString& sHostname, unsigned short uPort, int iTimeout = 60) : Csock(sHostname, uPort, iTimeout) {
-		InitClient();
-	}
-
-	virtual ~CClient();
-
-	void InitClient() {
 		m_pUser = NULL;
 		m_pTimeout = NULL;
 		m_pIRCSock = NULL;
@@ -84,10 +74,16 @@ public:
 		m_bGotUser = false;
 		m_bNamesx = false;
 		m_bUHNames = false;
-		m_uKeepNickCounter = 0;
 		EnableReadLine();
+		// RFC says a line can have 512 chars max, but we are
+		// a little more gentle ;)
+		SetMaxBufferThreshold(1024);
+
+		StartLoginTimeout();
 	}
 
+	virtual ~CClient();
+
 	void AcceptLogin(CUser& User);
 	void RefuseLogin(const CString& sReason);
 	void StartLoginTimeout();
@@ -98,7 +94,6 @@ public:
 	bool HasNamesx() const { return m_bNamesx; }
 	bool HasUHNames() const { return m_bUHNames; }
 
-	bool DecKeepNickCounter();
 	void UserCommand(const CString& sCommand);
 	void StatusCTCP(const CString& sCommand);
 	void IRCConnected(CIRCSock* pIRCSock);
@@ -108,6 +103,7 @@ public:
 
 	void PutIRC(const CString& sLine);
 	void PutClient(const CString& sLine);
+	unsigned int PutStatus(const CTable& table);
 	void PutStatus(const CString& sLine);
 	void PutStatusNotice(const CString& sLine);
 	void PutModule(const CString& sModule, const CString& sLine);
@@ -120,8 +116,7 @@ public:
 	virtual void Connected();
 	virtual void Disconnected();
 	virtual void ConnectionRefused();
-	virtual bool ConnectionFrom(const CString& sHost, unsigned short uPort);
-	virtual Csock* GetSockObj(const CString& sHost, unsigned short uPort);
+	virtual void ReachedMaxBuffer();
 
 	void SetNick(const CString& s);
 	CUser* GetUser() const { return m_pUser; }
@@ -138,7 +133,6 @@ protected:
 	CString		m_sPass;
 	CString		m_sUser;
 	CIRCSock*	m_pIRCSock;
-	unsigned int	m_uKeepNickCounter;
 	CSmartPtr<CAuthBase>	m_spAuth;
 	CClientTimeout*	m_pTimeout;
 };
