@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.191 $
+* $Revision: 1.199 $
 */
 
 // note to compile with win32 need to link to winsock2, using gcc its -lws2_32
@@ -56,6 +56,8 @@
 #define ECONNREFUSED WSAECONNREFUSED
 #define EINPROGRESS WSAEINPROGRESS
 #define ETIMEDOUT WSAETIMEDOUT
+#define EADDRNOTAVAIL WSAEADDRNOTAVAIL
+#define ECONNABORTED WSAECONNABORTED
 
 #endif /* _WIN32 */
 
@@ -292,6 +294,11 @@ inline void ShutdownWin32()
 {
 	WSACleanup();
 }
+#define InitCsocket InitWin32
+#define ShutdownCsocket ShutdownWin32
+#else
+#define InitCsocket (void)0
+#define ShutdownCsocket (void)0
 #endif /* _WIN32 */
 
 //! wrappers for FD_SET and such to work in templates.
@@ -375,6 +382,9 @@ public:
 	//! this is the method you should override
 	virtual void RunJob();
 
+protected:
+	bool		m_bRunOnNextCall; //!< if set to true, RunJob() gets called on next invocation of run() despite the timeout
+
 private:
 	time_t		m_iTime;
 	bool		m_bActive, m_bPause;
@@ -636,18 +646,18 @@ public:
 
 	//! sets the max buffered threshold when EnableReadLine() is enabled
 	void SetMaxBufferThreshold( u_int iThreshold );
-	u_int GetMaxBufferThreshold();
+	u_int GetMaxBufferThreshold() const;
 
 	//! Returns the connection type from enum eConnType
-	int GetType();
+	int GetType() const;
 	void SetType( int iType );
 
 	//! Returns a reference to the socket name
-	const CS_STRING & GetSockName();
+	const CS_STRING & GetSockName() const;
 	void SetSockName( const CS_STRING & sName );
 
 	//! Returns a reference to the host name
-	const CS_STRING & GetHostName();
+	const CS_STRING & GetHostName() const;
 	void SetHostName( const CS_STRING & sHostname );
 
 
@@ -1589,9 +1599,11 @@ public:
 	//! returns a pointer to the FIRST sock found by name or NULL on no match
 	virtual T * FindSockByName( const CS_STRING & sName )
 	{
-		for( unsigned int i = 0; i < this->size(); i++ )
-			if ( (*this)[i]->GetSockName() == sName )
-				return( (*this)[i] );
+		typename std::vector<T *>::iterator it;
+		typename std::vector<T *>::iterator it_end = this->end();
+		for( it = this->begin(); it != it_end; it++ )
+			if ( (*it)->GetSockName() == sName )
+				return( *it );
 
 		return( NULL );
 	}
@@ -2062,6 +2074,14 @@ private:
 						} else
 							CS_Delete( NewpcSock );
 					}
+#ifdef _WIN32
+					else if( GetSockError() != WSAEWOULDBLOCK )
+#else /* _WIN32 */
+					else if( GetSockError() != EAGAIN )
+#endif /* _WIN32 */
+					{
+						pcSock->SockError( GetSockError() );
+					}
 				}
 			}
 		}
@@ -2071,15 +2091,21 @@ private:
 	{
 		time_t iNextRunTime = iNow + iMaxResolution;
 		time_t iMinTimeout = iMaxResolution;
-		for( u_long a = 0; a < this->size() && iMinTimeout; a++ )
+		typename std::vector<T *>::const_iterator it;
+		// This is safe, because we don't modify the vector.
+		typename std::vector<T *>::const_iterator it_end = this->end();
+
+		for (it = this->begin(); it != it_end; it++)
 		{
-			if( (*this)[a]->GetConState() != T::CST_OK )
+			T* pSock = *it;
+
+			if( pSock->GetConState() != T::CST_OK )
 				iMinTimeout = 0; // this is in a nebulous state, need to let it proceed like normal
 
-			time_t iTimeoutInSeconds = (*this)[a]->GetTimeout();
+			time_t iTimeoutInSeconds = pSock->GetTimeout();
 			if( iTimeoutInSeconds > 0 )
 			{
-				time_t iLastTimeData = (*this)[a]->GetLastCheckTimeout();
+				time_t iLastTimeData = pSock->GetLastCheckTimeout();
 				time_t iDiff = iNow - iLastTimeData;
 				if( iDiff > iTimeoutInSeconds )
 					iTimeoutInSeconds = 0;
@@ -2089,12 +2115,16 @@ private:
 				iMinTimeout = std::min( iMinTimeout, iTimeoutInSeconds );
 			}
 
-			const std::vector<CCron *> & vCrons = (*this)[a]->GetCrons();
-			for( u_long b = 0; b < vCrons.size(); b++ )
-				iNextRunTime = std::min( iNextRunTime, vCrons[b]->GetNextRun() );
+			const std::vector<CCron *> & vCrons = pSock->GetCrons();
+			std::vector<CCron *>::const_iterator cit;
+			std::vector<CCron *>::const_iterator cit_end = vCrons.end();
+			for (cit = vCrons.begin(); cit != cit_end; it++)
+				iNextRunTime = std::min( iNextRunTime, (*cit)->GetNextRun() );
 		}
-		for( u_long a = 0; a < m_vcCrons.size(); a++ )
-			iNextRunTime = std::min( iNextRunTime, m_vcCrons[a]->GetNextRun() );
+		std::vector<CCron *>::const_iterator cit;
+		std::vector<CCron *>::const_iterator cit_end = m_vcCrons.end();
+		for (cit = m_vcCrons.begin(); cit != cit_end; cit++)
+			iNextRunTime = std::min( iNextRunTime, (*cit)->GetNextRun() );
 
 		if( iNextRunTime < iNow )
 			return( 0 ); // smallest unit possible
