@@ -43,16 +43,12 @@ public:
 		m_sRemoteIP = sRemoteIP;
 		m_bIsRemote = false;
 
+		SetMaxBufferThreshold(10240);
 		if (bIsChat) {
 			EnableReadLine();
 		}
 	}
-	virtual ~CDCCBounce() {
-		if (m_pPeer) {
-			m_pPeer->Shutdown();
-			m_pPeer = NULL;
-		}
-	}
+	virtual ~CDCCBounce();
 
 	static unsigned short DCCRequest(const CString& sNick, unsigned long uLongIP, unsigned short uPort, const CString& sFileName, bool bIsChat, CUser* pUser, const CString& sLocalIP, const CString& sRemoteIP);
 
@@ -61,6 +57,7 @@ public:
 	virtual void ReadPaused();
 	virtual void Timeout();
 	virtual void ConnectionRefused();
+	virtual void ReachedMaxBuffer();
 	virtual void SockError(int iErrno);
 	virtual void Connected();
 	virtual void Disconnected();
