@@ -10,7 +10,7 @@
 #define _FILEUTILS_H
 
 #include "Csocket.h"
-#include "String.h"
+#include "ZNCString.h"
 #include <dirent.h>
 #include <map>
 #include <signal.h>
@@ -246,7 +246,7 @@ public:
 		return uRet;
 	}
 
-	static unsigned int Delete(mode_t mode, const CString& sWildcard, const CString& sDir = ".") {
+	static unsigned int Delete(const CString& sWildcard, const CString& sDir = ".") {
 		CDir cDir;
 		cDir.FillByWildcard(sDir, sWildcard);
 		return cDir.Delete();
