@@ -11,18 +11,18 @@
 #ifndef _HTTPSOCK_H
 #define _HTTPSOCK_H
 
-#include "Csocket.h"
-#include "main.h"
+#include "Modules.h"
 
-class CHTTPSock : public Csock {
+class CHTTPSock : public CSocket {
 public:
-	CHTTPSock();
-	CHTTPSock(const CString& sHostname, unsigned short uPort, int iTimeout = 60);
+	CHTTPSock(CModule *pMod);
+	CHTTPSock(CModule *pMod, const CString& sHostname, unsigned short uPort, int iTimeout = 60);
 	virtual ~CHTTPSock();
 
 	// Csocket derived members
 	virtual void ReadData(const char* data, int len);
 	virtual void ReadLine(const CString& sData);
+	virtual void ReachedMaxBuffer();
 	virtual void SockError(int iErrno);
 	virtual void Timeout();
 	virtual void Connected();
@@ -71,6 +71,8 @@ public:
 	// !Getters
 private:
 protected:
+	void Init();
+
 	bool					m_bSentHeader;
 	bool					m_bGotHeader;
 	bool					m_bLoggedIn;
@@ -85,6 +87,8 @@ protected:
 	CString					m_sDocRoot;
 	map<CString, VCString>	m_msvsParams;
 	MCString				m_msHeaders;
+	bool					m_bHTTP10Client;
+	CString					m_sIfNoneMatch;
 };
 
 #endif // !_HTTPSOCK_H
