@@ -16,9 +16,7 @@
 CIRCSock::CIRCSock(CUser* pUser) : Csock() {
 	m_pUser = pUser;
 	m_bISpoofReleased = false;
-	m_bKeepNick = true;
 	m_bAuthed = false;
-	m_bOrigNickPending = false;
 	m_bNamesx = false;
 	m_bUHNames = false;
 	EnableReadLine();
@@ -38,6 +36,9 @@ CIRCSock::CIRCSock(CUser* pUser) : Csock() {
 	m_mueChanModes['t'] = NoArg;
 	m_mueChanModes['i'] = NoArg;
 	m_mueChanModes['n'] = NoArg;
+
+	// RFC says a line can have 512 chars max, but we don't care ;)
+	SetMaxBufferThreshold(1024);
 }
 
 CIRCSock::~CIRCSock() {
@@ -78,9 +79,9 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 	MODULECALL(OnRaw(sLine), m_pUser, NULL, return);
 
-	if (strncasecmp(sLine.c_str(), "PING ", 5) == 0) {
+	if (sLine.Equals("PING ", false, 5)) {
 		PutIRC("PONG " + sLine.substr(5));
-	} else if (strncasecmp(sLine.c_str(), "ERROR ", 6) == 0) {
+	} else if (sLine.Equals("ERROR ", false, 6)) {
 		//ERROR :Closing Link: nick[24.24.24.24] (Excess Flood)
 		CString sError(sLine.substr(7));
 
@@ -113,7 +114,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 						CClient* pClient = vClients[a];
 						CString sClientNick = pClient->GetNick(false);
 
-						if (sClientNick.CaseCmp(sNick) != 0) {
+						if (!sClientNick.Equals(sNick)) {
 							// If they connected with a nick that doesn't match the one we got on irc, then we need to update them
 							pClient->PutClient(":" + sClientNick + "!" + m_Nick.GetIdent() + "@" + m_Nick.GetHost() + " NICK :" + sNick);
 						}
@@ -175,62 +176,12 @@ void CIRCSock::ReadLine(const CString& sData) {
 					if (m_pUser->IsChan(sRest.Token(0)) || sNick != "*")
 						break;
 				case 433: {
-					unsigned int uMax = GetMaxNickLen();
 					CString sBadNick = sRest.Token(0);
-					CString sConfNick = m_pUser->GetNick().Left(uMax);
+					CString sConfNick = m_pUser->GetNick().Left(GetMaxNickLen());
 
 					if (sNick == "*") {
-						CString sAltNick = m_pUser->GetAltNick().Left(uMax);
-
-						if (sBadNick.CaseCmp(sConfNick) == 0) {
-							if ((!sAltNick.empty()) && (sConfNick.CaseCmp(sAltNick) != 0)) {
-								PutIRC("NICK " + sAltNick);
-							} else {
-								PutIRC("NICK " + sConfNick.Left(uMax -1) + "-");
-							}
-						} else if (sBadNick.CaseCmp(sAltNick) == 0) {
-							PutIRC("NICK " + sConfNick.Left(uMax -1) + "-");
-						} else if (sBadNick.CaseCmp(CString(sConfNick.Left(uMax -1) + "-")) == 0) {
-							PutIRC("NICK " + sConfNick.Left(uMax -1) + "|");
-						} else if (sBadNick.CaseCmp(CString(sConfNick.Left(uMax -1) + "|")) == 0) {
-							PutIRC("NICK " + sConfNick.Left(uMax -1) + "^");
-						} else if (sBadNick.CaseCmp(CString(sConfNick.Left(uMax -1) + "^")) == 0) {
-							PutIRC("NICK " + sConfNick.Left(uMax -1) + "a");
-						} else {
-							char cLetter = 0;
-							if (sBadNick.empty()) {
-								Quit();
-								return;
-							}
-
-							cLetter = sBadNick.Right(1)[0];
-
-							if (cLetter == 'z') {
-								Quit();
-								return;
-							}
-
-							CString sSend = "NICK " + sConfNick.Left(uMax -1) + ++cLetter;
-							PutIRC(sSend);
-						}
-
+						SendAltNick(sBadNick);
 						return;
-					} else {
-						// :irc.server.net 433 mynick badnick :Nickname is already in use.
-						if ((m_bKeepNick) && (m_pUser->GetKeepNick())) {
-							if (sBadNick.CaseCmp(sConfNick) == 0) {
-								vector<CClient*>& vClients = m_pUser->GetClients();
-
-								for (unsigned int a = 0; a < vClients.size(); a++) {
-									CClient* pClient = vClients[a];
-
-									if (!pClient || !pClient->DecKeepNickCounter()) {
-										SetOrigNickPending(false);
-										return;
-									}
-								}
-							}
-						}
 					}
 					break;
 				}
@@ -289,7 +240,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 					sServer.LeftChomp();
 
-					if (sNick.CaseCmp(GetNick()) == 0) {
+					if (sNick.Equals(GetNick())) {
 						m_Nick.SetIdent(sIdent);
 						m_Nick.SetHost(sHost);
 					}
@@ -328,62 +279,20 @@ void CIRCSock::ReadLine(const CString& sData) {
 					sRest.Trim();
 					// Todo: allow for non @+= server msgs
 					CChan* pChan = m_pUser->FindChan(sRest.Token(1));
-					if (!pChan) {
-						// If we don't know that channel, a client might have
-						// sent a /names on it's own -> forward it.
-						m_pUser->PutUser(sLine);
-						return;
-					}
-
-					CString sNicks = sRest.Token(2, true);
-					if (sNicks.Left(1) == ":") {
-						sNicks.LeftChomp();
-					}
-
-					pChan->AddNicks(sNicks);
-
-					// Get everything except the actual user list
-					CString sTmp = sLine.Token(0, false, " :") + " :";
-					vector<CClient*>& vClients = m_pUser->GetClients();
-
-					for (unsigned int a = 0; a < vClients.size(); a++) {
-						if ((!m_bNamesx || vClients[a]->HasNamesx())
-								&& (!m_bUHNames || vClients[a]->HasUHNames())) {
-							m_pUser->PutUser(sLine, vClients[a]);
-						} else {
-							unsigned int i = 0;
-							// This loop runs once for
-							// every nick on the channel
-							for (;;) {
-								sNick = sNicks.Token(i).Trim_n(" ");
-								if (sNick.empty())
-									break;
-
-								if (m_bNamesx && !vClients[a]->HasNamesx()
-									&& IsPermChar(sNick[0])) {
-									// Server has, client hasnt NAMESX,
-									// so we just use the first perm char
-									while (sNick.length() > 2
-											&& IsPermChar(sNick[1])) {
-										sNick = sNick[0] + sNick.substr(2);
-									}
-								}
-
-								// Server has, client hasnt UHNAMES,
-								// so we strip away ident and host.
-								if (m_bUHNames && !vClients[a]->HasUHNames()) {
-									sNick = sNick.Token(0, false, "!");
-								}
-
-								sTmp += sNick + " ";
-								i++;
-							}
-							// Strip away the spaces we inserted at the end
-							sTmp.Trim(" ");
-							m_pUser->PutUser(sTmp, vClients[a]);
+					// If we don't know that channel, some client might have
+					// requested a /names for it and we really should forward this.
+					if (pChan) {
+						CString sNicks = sRest.Token(2, true);
+						if (sNicks.Left(1) == ":") {
+							sNicks.LeftChomp();
 						}
+
+						pChan->AddNicks(sNicks);
 					}
-					// We forwarded it in the for loop above already
+
+					ForwardRaw353(sLine);
+
+					// We forwarded it already, so return
 					return;
 				}
 				case 366: {	// end of names list
@@ -393,10 +302,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					CChan* pChan = m_pUser->FindChan(sRest.Token(0));
 
 					if (pChan) {
-						if (!pChan->IsOn()) {
-							pChan->SetIsOn(true);
-							PutIRC("MODE " + pChan->GetName());
-
+						if (pChan->IsOn()) {
 							// If we are the only one in the chan, set our default modes
 							if (pChan->GetNickCount() == 1) {
 								CString sModes = pChan->GetDefaultModes();
@@ -420,7 +326,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 			sCmd = sLine.Token(1);
 			CString sRest = sLine.Token(2, true);
 
-			if (sCmd.CaseCmp("NICK") == 0) {
+			if (sCmd.Equals("NICK")) {
 				CString sNewNick = sRest;
 				bool bIsVisible = false;
 
@@ -444,12 +350,10 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 
 				// Todo: use nick compare function here
-				if (Nick.GetNick().CaseCmp(GetNick()) == 0) {
+				if (Nick.GetNick().Equals(GetNick())) {
 					// We are changing our own nick, the clients always must see this!
 					bIsVisible = true;
 					SetNick(sNewNick);
-				} else if (Nick.GetNick().CaseCmp(m_pUser->GetNick()) == 0) {
-					KeepNick(true);
 				}
 
 				MODULECALL(OnNick(Nick, sNewNick, vFoundChans), m_pUser, NULL, );
@@ -457,7 +361,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				if (!bIsVisible) {
 					return;
 				}
-			} else if (sCmd.CaseCmp("QUIT") == 0) {
+			} else if (sCmd.Equals("QUIT")) {
 				CString sMessage = sRest;
 				bool bIsVisible = false;
 
@@ -467,7 +371,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 				// :nick!ident@host.com QUIT :message
 
-				if (Nick.GetNick().CaseCmp(GetNick()) == 0) {
+				if (Nick.GetNick().Equals(GetNick())) {
 					m_pUser->PutStatus("You quit [" + sMessage + "]");
 				}
 
@@ -486,16 +390,12 @@ void CIRCSock::ReadLine(const CString& sData) {
 					}
 				}
 
-				if (Nick.GetNick().CaseCmp(m_pUser->GetNick()) == 0) {
-					KeepNick(true);
-				}
-
 				MODULECALL(OnQuit(Nick, sMessage, vFoundChans), m_pUser, NULL, );
 
 				if (!bIsVisible) {
 					return;
 				}
-			} else if (sCmd.CaseCmp("JOIN") == 0) {
+			} else if (sCmd.Equals("JOIN")) {
 				CString sChan = sRest.Token(0);
 				if (sChan.Left(1) == ":") {
 					sChan.LeftChomp();
@@ -504,12 +404,15 @@ void CIRCSock::ReadLine(const CString& sData) {
 				CChan* pChan;
 
 				// Todo: use nick compare function
-				if (Nick.GetNick().CaseCmp(GetNick()) == 0) {
+				if (Nick.GetNick().Equals(GetNick())) {
 					m_pUser->AddChan(sChan, false);
 					pChan = m_pUser->FindChan(sChan);
 					if (pChan) {
 						pChan->ResetJoinTries();
 						pChan->Enable();
+						pChan->SetIsOn(true);
+						PutIRC("MODE " + pChan->GetName());
+
 					}
 				} else {
 					pChan = m_pUser->FindChan(sChan);
@@ -523,7 +426,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 						return;
 					}
 				}
-			} else if (sCmd.CaseCmp("PART") == 0) {
+			} else if (sCmd.Equals("PART")) {
 				CString sChan = sRest.Token(0);
 				if (sChan.Left(1) == ":") {
 					sChan.LeftChomp();
@@ -540,7 +443,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 
 				// Todo: use nick compare function
-				if (Nick.GetNick().CaseCmp(GetNick()) == 0) {
+				if (Nick.GetNick().Equals(GetNick())) {
 					m_pUser->DelChan(sChan);
 				}
 
@@ -553,7 +456,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				if (bDetached) {
 					return;
 				}
-			} else if (sCmd.CaseCmp("MODE") == 0) {
+			} else if (sCmd.Equals("MODE")) {
 				CString sTarget = sRest.Token(0);
 				CString sModes = sRest.Token(1, true);
 				if (sModes.Left(1) == ":")
@@ -589,7 +492,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 						}
 					}
 				}
-			} else if (sCmd.CaseCmp("KICK") == 0) {
+			} else if (sCmd.Equals("KICK")) {
 				// :opnick!ident@host.com KICK #chan nick :msg
 				CString sChan = sRest.Token(0);
 				CString sKickedNick = sRest.Token(1);
@@ -603,7 +506,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					MODULECALL(OnKick(Nick.GetNickMask(), sKickedNick, *pChan, sMsg), m_pUser, NULL, );
 				}
 
-				if (GetNick().CaseCmp(sKickedNick) == 0 && pChan) {
+				if (GetNick().Equals(sKickedNick) && pChan) {
 					pChan->SetIsOn(false);
 
 					// Don't try to rejoin!
@@ -613,7 +516,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				if ((pChan) && (pChan->IsDetached())) {
 					return;
 				}
-			} else if (sCmd.CaseCmp("NOTICE") == 0) {
+			} else if (sCmd.Equals("NOTICE")) {
 				// :nick!ident@host.com NOTICE #chan :Message
 				CString sTarget = sRest.Token(0);
 				CString sMsg = sRest.Token(1, true);
@@ -623,7 +526,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					sMsg.LeftChomp();
 					sMsg.RightChomp();
 
-					if (sTarget.CaseCmp(GetNick()) == 0) {
+					if (sTarget.Equals(GetNick())) {
 						if (OnCTCPReply(Nick, sMsg)) {
 							return;
 						}
@@ -632,7 +535,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					m_pUser->PutUser(":" + Nick.GetNickMask() + " NOTICE " + sTarget + " :\001" + sMsg + "\001");
 					return;
 				} else {
-					if (sTarget.CaseCmp(GetNick()) == 0) {
+					if (sTarget.Equals(GetNick())) {
 						if (OnPrivNotice(Nick, sMsg)) {
 							return;
 						}
@@ -643,14 +546,14 @@ void CIRCSock::ReadLine(const CString& sData) {
 					}
 				}
 
-				if (Nick.GetNick().CaseCmp(m_pUser->GetIRCServer()) == 0) {
+				if (Nick.GetNick().Equals(m_pUser->GetIRCServer())) {
 					m_pUser->PutUser(":" + Nick.GetNick() + " NOTICE " + sTarget + " :" + sMsg);
 				} else {
 					m_pUser->PutUser(":" + Nick.GetNickMask() + " NOTICE " + sTarget + " :" + sMsg);
 				}
 
 				return;
-			} else if (sCmd.CaseCmp("TOPIC") == 0) {
+			} else if (sCmd.Equals("TOPIC")) {
 				// :nick!ident@host.com TOPIC #chan :This is a topic
 				CChan* pChan = m_pUser->FindChan(sLine.Token(2));
 
@@ -670,7 +573,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 					sLine = ":" + Nick.GetNickMask() + " TOPIC " + pChan->GetName() + " :" + sTopic;
 				}
-			} else if (sCmd.CaseCmp("PRIVMSG") == 0) {
+			} else if (sCmd.Equals("PRIVMSG")) {
 				// :nick!ident@host.com PRIVMSG #chan :Message
 				CString sTarget = sRest.Token(0);
 				CString sMsg = sRest.Token(1, true);
@@ -683,7 +586,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					sMsg.LeftChomp();
 					sMsg.RightChomp();
 
-					if (sTarget.CaseCmp(GetNick()) == 0) {
+					if (sTarget.Equals(GetNick())) {
 						if (OnPrivCTCP(Nick, sMsg)) {
 							return;
 						}
@@ -696,7 +599,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + sTarget + " :\001" + sMsg + "\001");
 					return;
 				} else {
-					if (sTarget.CaseCmp(GetNick()) == 0) {
+					if (sTarget.Equals(GetNick())) {
 						if (OnPrivMsg(Nick, sMsg)) {
 							return;
 						}
@@ -709,7 +612,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + sTarget + " :" + sMsg);
 					return;
 				}
-			} else if (sCmd.CaseCmp("WALLOPS") == 0) {
+			} else if (sCmd.Equals("WALLOPS")) {
 				// :blub!dummy@rox-8DBEFE92 WALLOPS :this is a test
 				CString sMsg = sRest.Token(0, true);
 
@@ -725,15 +628,6 @@ void CIRCSock::ReadLine(const CString& sData) {
 	m_pUser->PutUser(sLine);
 }
 
-void CIRCSock::KeepNick(bool bForce) {
-	const CString& sConfNick = m_pUser->GetNick();
-
-	if (m_bAuthed && m_bKeepNick && (!IsOrigNickPending() || bForce) && m_pUser->GetKeepNick() && GetNick().CaseCmp(sConfNick) != 0) {
-		PutIRC("NICK " + sConfNick);
-		SetOrigNickPending(true);
-	}
-}
-
 bool CIRCSock::OnCTCPReply(CNick& Nick, CString& sMessage) {
 	MODULECALL(OnCTCPReply(Nick, sMessage), m_pUser, NULL, return true);
 
@@ -741,14 +635,13 @@ bool CIRCSock::OnCTCPReply(CNick& Nick, CString& sMessage) {
 }
 
 bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
-	if (sMessage.Left(7).CaseCmp("ACTION ") == 0) {
-		sMessage = sMessage.substr(7);
+	if (sMessage.TrimPrefix("ACTION ")) {
 		MODULECALL(OnPrivAction(Nick, sMessage), m_pUser, NULL, return true);
 		sMessage = "ACTION " + sMessage;
 	}
 	MODULECALL(OnPrivCTCP(Nick, sMessage), m_pUser, NULL, return true);
 
-	if (strncasecmp(sMessage.c_str(), "DCC ", 4) == 0 && m_pUser && m_pUser->BounceDCCs() && m_pUser->IsUserAttached()) {
+	if (sMessage.Equals("DCC ", false, 4) && m_pUser && m_pUser->BounceDCCs() && m_pUser->IsUserAttached()) {
 		// DCC CHAT chat 2453612361 44592
 		CString sType = sMessage.Token(1);
 		CString sFile = sMessage.Token(2);
@@ -756,33 +649,33 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 		unsigned short uPort = strtoul(sMessage.Token(4).c_str(), NULL, 10);
 		unsigned long uFileSize = strtoul(sMessage.Token(5).c_str(), NULL, 10);
 
-		if (sType.CaseCmp("CHAT") == 0) {
+		if (sType.Equals("CHAT")) {
 			CNick FromNick(Nick.GetNickMask());
 			unsigned short uBNCPort = CDCCBounce::DCCRequest(FromNick.GetNick(), uLongIP, uPort, "", true, m_pUser, GetLocalIP(), CUtils::GetIP(uLongIP));
 			if (uBNCPort) {
 				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC CHAT chat " + CString(CUtils::GetLongIP(GetLocalIP())) + " " + CString(uBNCPort) + "\001");
 			}
-		} else if (sType.CaseCmp("SEND") == 0) {
+		} else if (sType.Equals("SEND")) {
 			// DCC SEND readme.txt 403120438 5550 1104
 			unsigned short uBNCPort = CDCCBounce::DCCRequest(Nick.GetNick(), uLongIP, uPort, sFile, false, m_pUser, GetLocalIP(), CUtils::GetIP(uLongIP));
 			if (uBNCPort) {
 				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC SEND " + sFile + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " " + CString(uBNCPort) + " " + CString(uFileSize) + "\001");
 			}
-		} else if (sType.CaseCmp("RESUME") == 0) {
+		} else if (sType.Equals("RESUME")) {
 			// Need to lookup the connection by port, filter the port, and forward to the user
 			CDCCBounce* pSock = (CDCCBounce*) CZNC::Get().GetManager().FindSockByLocalPort(sMessage.Token(3).ToUShort());
 
-			if ((pSock) && (strncasecmp(pSock->GetSockName().c_str(), "DCC::", 5) == 0)) {
+			if (pSock && pSock->GetSockName().Equals("DCC::", false, 5)) {
 				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC " + sType + " " + sFile + " " + CString(pSock->GetUserPort()) + " " + sMessage.Token(4) + "\001");
 			}
-		} else if (sType.CaseCmp("ACCEPT") == 0) {
+		} else if (sType.Equals("ACCEPT")) {
 			// Need to lookup the connection by port, filter the port, and forward to the user
 			CSockManager& Manager = CZNC::Get().GetManager();
 
 			for (unsigned int a = 0; a < Manager.size(); a++) {
 				CDCCBounce* pSock = (CDCCBounce*) Manager[a];
 
-				if ((pSock) && (strncasecmp(pSock->GetSockName().c_str(), "DCC::", 5) == 0)) {
+				if (pSock && pSock->GetSockName().Equals("DCC::", false, 5)) {
 					if (pSock->GetUserPort() == atoi(sMessage.Token(3).c_str())) {
 						m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC " + sType + " " + sFile + " " + CString(pSock->GetLocalPort()) + " " + sMessage.Token(4) + "\001");
 					}
@@ -856,8 +749,7 @@ bool CIRCSock::OnChanCTCP(CNick& Nick, const CString& sChan, CString& sMessage)
 	CChan* pChan = m_pUser->FindChan(sChan);
 	if (pChan) {
 		// Record a /me
-		if (sMessage.Left(7).CaseCmp("ACTION ") == 0) {
-			sMessage = sMessage.substr(7);
+		if (sMessage.TrimPrefix("ACTION ")) {
 			if (pChan->KeepBuffer() || !m_pUser->IsUserAttached()) {
 				pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :\001ACTION " + m_pUser->AddTimestamp(sMessage) + "\001");
 			}
@@ -913,12 +805,19 @@ void CIRCSock::Connected() {
 	DEBUG_ONLY(cout << GetSockName() << " == Connected()" << endl);
 	m_pUser->IRCConnected(this);
 
-	if (!m_sPass.empty()) {
-		PutIRC("PASS " + m_sPass);
+	CString sPass = m_sPass;
+	CString sNick = m_pUser->GetNick();
+	CString sIdent = m_pUser->GetIdent();
+	CString sRealName = m_pUser->GetRealName();
+
+	MODULECALL(OnIRCRegistration(sPass, sNick, sIdent, sRealName), m_pUser, NULL, return);
+
+	if (!sPass.empty()) {
+		PutIRC("PASS " + sPass);
 	}
 
-	PutIRC("NICK " + m_pUser->GetNick());
-	PutIRC("USER " + m_pUser->GetIdent() + " \"" + m_pUser->GetIdent() + "\" \"" + m_pUser->GetIdent() + "\" :" + m_pUser->GetRealName());
+	PutIRC("NICK " + sNick);
+	PutIRC("USER " + sIdent + " \"" + sIdent + "\" \"" + sIdent + "\" :" + sRealName);
 }
 
 void CIRCSock::Disconnected() {
@@ -987,6 +886,12 @@ void CIRCSock::ConnectionRefused() {
 	m_pUser->ClearMotdBuffer();
 }
 
+void CIRCSock::ReachedMaxBuffer() {
+	DEBUG_ONLY(cout << GetSockName() << " == ReachedMaxBuffer()" << endl);
+	m_pUser->PutStatus("Received a too long line from the IRC server!");
+	Quit();
+}
+
 void CIRCSock::ParseISupport(const CString& sLine) {
 	unsigned int i = 0;
 	CString sArg = sLine.Token(i++);
@@ -995,7 +900,7 @@ void CIRCSock::ParseISupport(const CString& sLine) {
 		CString sName = sArg.Token(0, false, "=");
 		CString sValue = sArg.Token(1, true, "=");
 
-		if (sName.CaseCmp("PREFIX") == 0) {
+		if (sName.Equals("PREFIX")) {
 			CString sPrefixes = sValue.Token(1, false, ")");
 			CString sPermModes = sValue.Token(0, false, ")");
 			sPermModes.TrimLeft("(");
@@ -1004,15 +909,15 @@ void CIRCSock::ParseISupport(const CString& sLine) {
 				m_sPerms = sPrefixes;
 				m_sPermModes = sPermModes;
 			}
-		} else if (sName.CaseCmp("CHANTYPES") == 0) {
+		} else if (sName.Equals("CHANTYPES")) {
 			m_pUser->SetChanPrefixes(sValue);
-		} else if (sName.CaseCmp("NICKLEN") == 0) {
+		} else if (sName.Equals("NICKLEN")) {
 			unsigned int uMax = sValue.ToUInt();
 
 			if (uMax) {
 				m_uMaxNickLen = uMax;
 			}
-		} else if (sName.CaseCmp("CHANMODES") == 0) {
+		} else if (sName.Equals("CHANMODES")) {
 			if (!sValue.empty()) {
 				m_mueChanModes.clear();
 
@@ -1024,10 +929,10 @@ void CIRCSock::ParseISupport(const CString& sLine) {
 					}
 				}
 			}
-		} else if (sName.CaseCmp("NAMESX") == 0) {
+		} else if (sName.Equals("NAMESX")) {
 			m_bNamesx = true;
 			PutIRC("PROTOCTL NAMESX");
-		} else if (sName.CaseCmp("UHNAMES") == 0) {
+		} else if (sName.Equals("UHNAMES")) {
 			m_bUHNames = true;
 			PutIRC("PROTOCTL UHNAMES");
 		}
@@ -1036,6 +941,91 @@ void CIRCSock::ParseISupport(const CString& sLine) {
 	}
 }
 
+void CIRCSock::ForwardRaw353(const CString& sLine) const {
+	vector<CClient*>& vClients = m_pUser->GetClients();
+	CString sNicks = sLine.Token(5, true);
+	if (sNicks.Left(1) == ":")
+		sNicks.LeftChomp();
+
+	for (unsigned int a = 0; a < vClients.size(); a++) {
+		if ((!m_bNamesx || vClients[a]->HasNamesx()) && (!m_bUHNames || vClients[a]->HasUHNames())) {
+			// Client and server have both the same UHNames and Namesx stuff enabled
+			m_pUser->PutUser(sLine, vClients[a]);
+		} else {
+			// Get everything except the actual user list
+			CString sTmp = sLine.Token(0, false, " :") + " :";
+
+			unsigned int i = 0;
+			// This loop runs once for every nick on the channel
+			for (;;) {
+				CString sNick = sNicks.Token(i).Trim_n(" ");
+				if (sNick.empty())
+					break;
+
+				if (m_bNamesx && !vClients[a]->HasNamesx() && IsPermChar(sNick[0])) {
+					// Server has, client doesn't have NAMESX, so we just use the first perm char
+					size_t pos = sNick.find_first_not_of(GetPerms());
+					if (pos >= 2 && pos != CString::npos) {
+						sNick = sNick[0] + sNick.substr(pos);
+					}
+				}
+
+				if (m_bUHNames && !vClients[a]->HasUHNames()) {
+					// Server has, client hasnt UHNAMES,
+					// so we strip away ident and host.
+					sNick = sNick.Token(0, false, "!");
+				}
+
+				sTmp += sNick + " ";
+				i++;
+			}
+			// Strip away the spaces we inserted at the end
+			sTmp.TrimRight(" ");
+			m_pUser->PutUser(sTmp, vClients[a]);
+		}
+	}
+}
+
+void CIRCSock::SendAltNick(const CString& sBadNick) {
+	const unsigned int uMax = GetMaxNickLen();
+	const CString& sConfNick = m_pUser->GetNick().Left(uMax);
+	const CString& sAltNick = m_pUser->GetAltNick().Left(uMax);
+
+	if (sBadNick.Equals(sConfNick)) {
+		if ((!sAltNick.empty()) && (!sConfNick.Equals(sAltNick))) {
+			PutIRC("NICK " + sAltNick);
+		} else {
+			PutIRC("NICK " + sConfNick.Left(uMax -1) + "-");
+		}
+	} else if (sBadNick.Equals(sAltNick)) {
+		PutIRC("NICK " + sConfNick.Left(uMax -1) + "-");
+	} else if (sBadNick.Equals(CString(sConfNick.Left(uMax -1) + "-"))) {
+		PutIRC("NICK " + sConfNick.Left(uMax -1) + "|");
+	} else if (sBadNick.Equals(CString(sConfNick.Left(uMax -1) + "|"))) {
+		PutIRC("NICK " + sConfNick.Left(uMax -1) + "^");
+	} else if (sBadNick.Equals(CString(sConfNick.Left(uMax -1) + "^"))) {
+		PutIRC("NICK " + sConfNick.Left(uMax -1) + "a");
+	} else {
+		char cLetter = 0;
+		if (sBadNick.empty()) {
+			m_pUser->PutUser("No free nick available");
+			Quit();
+			return;
+		}
+
+		cLetter = sBadNick.Right(1)[0];
+
+		if (cLetter == 'z') {
+			m_pUser->PutUser("No free nick found");
+			Quit();
+			return;
+		}
+
+		CString sSend = "NICK " + sConfNick.Left(uMax -1) + ++cLetter;
+		PutIRC(sSend);
+	}
+}
+
 unsigned char CIRCSock::GetPermFromMode(unsigned char uMode) const {
 	if (m_sPermModes.size() == m_sPerms.size()) {
 		for (unsigned int a = 0; a < m_sPermModes.size(); a++) {
