@@ -46,21 +46,17 @@ public:
 	virtual void ConnectionRefused();
 	virtual void SockError(int iErrno);
 	virtual void Timeout();
+	virtual void ReachedMaxBuffer();
 
-	void KeepNick(bool bForce = false);
 	void PutIRC(const CString& sLine);
-	void ParseISupport(const CString& sLine);
 	void ResetChans();
 	void Quit();
 
 	// Setters
 	void SetPass(const CString& s) { m_sPass = s; }
-	void SetKeepNick(bool b) { m_bKeepNick = b; }
-	void SetOrigNickPending(bool b) { m_bOrigNickPending = b; }
 	// !Setters
 
 	// Getters
-	bool GetKeepNick() const { return m_bKeepNick; }
 	unsigned int GetMaxNickLen() const { return m_uMaxNickLen; }
 	EChanModeArgs GetModeType(unsigned char uMode) const;
 	unsigned char GetPermFromMode(unsigned char uMode) const;
@@ -72,19 +68,22 @@ public:
 	CString GetNickMask() const { return m_Nick.GetNickMask(); }
 	const CString& GetNick() const { return m_Nick.GetNick(); }
 	const CString& GetPass() const { return m_sPass; }
-	bool IsOrigNickPending() const { return m_bOrigNickPending; }
 	CUser* GetUser() const { return m_pUser; }
 	bool HasNamesx() const { return m_bNamesx; }
 	bool HasUHNames() const { return m_bUHNames; }
 	const set<unsigned char>& GetUserModes() const { return m_scUserModes; }
+	// This is true if we are past raw 001
+	bool IsAuthed() const { return m_bAuthed; }
 	// !Getters
 private:
 	void SetNick(const CString& sNick);
+	void ParseISupport(const CString& sLine);
+	void ForwardRaw353(const CString& sLine) const;
+	// This is called when we connect and the nick we want is already taken
+	void SendAltNick(const CString& sBadNick);
 protected:
 	bool						m_bISpoofReleased;
 	bool						m_bAuthed;
-	bool						m_bKeepNick;
-	bool						m_bOrigNickPending;
 	bool						m_bNamesx;
 	bool						m_bUHNames;
 	CString						m_sPerms;
