@@ -18,76 +18,36 @@
 # warning "your crap box doesnt define RTLD_LOCAL !?"
 #endif
 
-#define MODUNLOADCHK(func)								\
+#define _MODUNLOADCHK(func, type)								\
 	for (unsigned int a = 0; a < size(); a++) {			\
 		try {											\
-			CModule* pMod = (*this)[a];					\
-			if (m_pUser) {								\
+			type* pMod = (type *) (*this)[a];					\
+			pMod->SetClient(m_pClient);					\
+			if (m_pUser) {							\
 				pMod->SetUser(m_pUser);					\
-				pMod->func;								\
+				pMod->func;						\
 				pMod->SetUser(NULL);					\
-			} else {									\
-				pMod->func;								\
-			}											\
+			} else {							\
+				pMod->func;						\
+			}								\
+			pMod->SetClient(NULL);						\
 		} catch (CModule::EModException e) {			\
 			if (e == CModule::UNLOAD) {					\
 				UnloadModule((*this)[a]->GetModName());	\
 			}											\
 		}												\
-	}													\
+	}
 
-#define GLOBALMODCALL(func)								\
-	for (unsigned int a = 0; a < size(); a++) {			\
-		try {											\
-			CGlobalModule* pMod = (CGlobalModule*) (*this)[a];			\
-			if (m_pUser) {								\
-				pMod->SetUser(m_pUser);					\
-				pMod->func;								\
-				pMod->SetUser(NULL);					\
-			} else {									\
-				pMod->func;								\
-			}											\
-		} catch (CModule::EModException e) {			\
-			if (e == CModule::UNLOAD) {					\
-				UnloadModule((*this)[a]->GetModName());	\
-			}											\
-		}												\
-	}													\
+#define MODUNLOADCHK(func)	_MODUNLOADCHK(func, CModule)
+#define GLOBALMODCALL(func)	_MODUNLOADCHK(func, CGlobalModule)
 
-#define GLOBALMODHALTCHK(func)							\
+#define _MODHALTCHK(func, type)							\
 	bool bHaltCore = false;								\
 	for (unsigned int a = 0; a < size(); a++) {			\
 		try {											\
-			CGlobalModule* pMod = (CGlobalModule*) (*this)[a];			\
-			CModule::EModRet e = CModule::CONTINUE;		\
-			if (m_pUser) {								\
-				pMod->SetUser(m_pUser);					\
-				e = pMod->func;							\
-				pMod->SetUser(NULL);					\
-			} else {									\
-				e = pMod->func;							\
-			}											\
-			if (e == CModule::HALTMODS) {				\
-				break;									\
-			} else if (e == CModule::HALTCORE) {		\
-				bHaltCore = true;						\
-			} else if (e == CModule::HALT) {			\
-				bHaltCore = true;						\
-				break;									\
-			}											\
-		} catch (CModule::EModException e) {			\
-			if (e == CModule::UNLOAD) {					\
-				UnloadModule((*this)[a]->GetModName());	\
-			}											\
-		}												\
-	}													\
-	return bHaltCore;
-#define MODHALTCHK(func)								\
-	bool bHaltCore = false;								\
-	for (unsigned int a = 0; a < size(); a++) {			\
-		try {											\
-			CModule* pMod = (*this)[a];					\
+			type* pMod = (type*) (*this)[a];			\
 			CModule::EModRet e = CModule::CONTINUE;		\
+			pMod->SetClient(m_pClient);				\
 			if (m_pUser) {								\
 				pMod->SetUser(m_pUser);					\
 				e = pMod->func;							\
@@ -95,6 +55,7 @@
 			} else {									\
 				e = pMod->func;							\
 			}											\
+			pMod->SetClient(NULL);					\
 			if (e == CModule::HALTMODS) {				\
 				break;									\
 			} else if (e == CModule::HALTCORE) {		\
@@ -111,6 +72,9 @@
 	}													\
 	return bHaltCore;
 
+#define MODHALTCHK(func)	_MODHALTCHK(func, CModule)
+#define GLOBALMODHALTCHK(func)	_MODHALTCHK(func, CGlobalModule)
+
 /////////////////// Timer ///////////////////
 CTimer::CTimer(CModule* pModule, unsigned int uInterval, unsigned int uCycles, const CString& sLabel, const CString& sDescription) : CCron() {
 	SetName(sLabel);
@@ -135,24 +99,30 @@ const CString& CTimer::GetDescription() const { return m_sDescription; }
 /////////////////// !Timer ///////////////////
 
 /////////////////// Socket ///////////////////
-CSocket::CSocket(CModule* pModule, const CString& sLabel) : Csock() {
+CSocket::CSocket(CModule* pModule) : Csock() {
 	m_pModule = pModule;
-	m_sLabel = sLabel;
 	m_pModule->AddSocket(this);
 	EnableReadLine();
+	SetMaxBufferThreshold(10240);
 }
 
-CSocket::CSocket(CModule* pModule, const CString& sLabel, const CString& sHostname, unsigned short uPort, int iTimeout) : Csock(sHostname, uPort, iTimeout) {
+CSocket::CSocket(CModule* pModule, const CString& sHostname, unsigned short uPort, int iTimeout) : Csock(sHostname, uPort, iTimeout) {
 	m_pModule = pModule;
-	m_sLabel = sLabel;
 	m_pModule->AddSocket(this);
 	EnableReadLine();
+	SetMaxBufferThreshold(10240);
 }
 
 CSocket::~CSocket() {
 	m_pModule->UnlinkSocket(this);
 }
 
+void CSocket::ReachedMaxBuffer() {
+	DEBUG_ONLY(cout << GetSockName() << " == ReachedMaxBuffer()" << endl);
+	PutModule("Some socket reached its max buffer limit and was closed!");
+	Close();
+}
+
 bool CSocket::Connect(const CString& sHostname, unsigned short uPort, bool bSSL, unsigned int uTimeout) {
 	CUser* pUser = m_pModule->GetUser();
 	CString sSockName = "MOD::C::" + m_pModule->GetModName();
@@ -163,6 +133,11 @@ bool CSocket::Connect(const CString& sHostname, unsigned short uPort, bool bSSL,
 		sVHost = m_pModule->GetUser()->GetVHost();
 	}
 
+	// Don't overwrite the socket name if one is already set
+	if (!GetSockName().empty()) {
+		sSockName = GetSockName();
+	}
+
 	return m_pModule->GetManager()->Connect(sHostname, uPort, sSockName, uTimeout, bSSL, sVHost, (Csock*) this);
 }
 
@@ -173,6 +148,10 @@ bool CSocket::Listen(unsigned short uPort, bool bSSL, unsigned int uTimeout) {
 	if (pUser) {
 		sSockName += "::" + pUser->GetUserName();
 	}
+	// Don't overwrite the socket name if one is already set
+	if (!GetSockName().empty()) {
+		sSockName = GetSockName();
+	}
 
 	return m_pModule->GetManager()->ListenAll(uPort, sSockName, bSSL, SOMAXCONN, (Csock*) this);
 }
@@ -197,10 +176,7 @@ bool CSocket::PutModNotice(const CString& sLine, const CString& sIdent, const CS
 }
 
 void CSocket::SetModule(CModule* p) { m_pModule = p; }
-void CSocket::SetLabel(const CString& s) { m_sLabel = s; }
-
 CModule* CSocket::GetModule() const { return m_pModule; }
-const CString& CSocket::GetLabel() const { return m_sLabel; }
 /////////////////// !Socket ///////////////////
 
 CModule::CModule(void* pDLL, CUser* pUser, const CString& sModName, const CString& sDataDir) {
@@ -314,7 +290,7 @@ bool CModule::RemTimer(const CString& sLabel) {
 	for (unsigned int a = 0; a < m_vTimers.size(); a++) {
 		CTimer* pTimer = m_vTimers[a];
 
-		if (pTimer->GetName().CaseCmp(sLabel) == 0) {
+		if (pTimer->GetName().Equals(sLabel)) {
 			m_vTimers.erase(m_vTimers.begin() +a);
 			m_pManager->DelCronByAddr(pTimer);
 			return true;
@@ -338,7 +314,7 @@ bool CModule::UnlinkTimer(CTimer* pTimer) {
 CTimer* CModule::FindTimer(const CString& sLabel) {
 	for (unsigned int a = 0; a < m_vTimers.size(); a++) {
 		CTimer* pTimer = m_vTimers[a];
-		if (pTimer->GetName().CaseCmp(sLabel) == 0) {
+		if (pTimer->GetName().Equals(sLabel)) {
 			return pTimer;
 		}
 	}
@@ -369,14 +345,7 @@ void CModule::ListTimers() {
 		Table.SetCell("Description", pTimer->GetDescription());
 	}
 
-	if (Table.size()) {
-		unsigned int uTableIdx = 0;
-		CString sLine;
-
-		while (Table.GetLine(uTableIdx++, sLine)) {
-			PutModule(sLine);
-		}
-	}
+	PutModule(Table);
 }
 
 bool CModule::AddSocket(CSocket* pSocket) {
@@ -400,11 +369,11 @@ bool CModule::RemSocket(CSocket* pSocket) {
 	return false;
 }
 
-bool CModule::RemSocket(const CString& sLabel) {
+bool CModule::RemSocket(const CString& sSockName) {
 	for (unsigned int a = 0; a < m_vSockets.size(); a++) {
 		CSocket* pSocket = m_vSockets[a];
 
-		if (pSocket->GetLabel().CaseCmp(sLabel) == 0) {
+		if (pSocket->GetSockName().Equals(sSockName)) {
 			m_vSockets.erase(m_vSockets.begin() +a);
 			m_pManager->DelSockByAddr(pSocket);
 			return true;
@@ -425,10 +394,10 @@ bool CModule::UnlinkSocket(CSocket* pSocket) {
 	return false;
 }
 
-CSocket* CModule::FindSocket(const CString& sLabel) {
+CSocket* CModule::FindSocket(const CString& sSockName) {
 	for (unsigned int a = 0; a < m_vSockets.size(); a++) {
 		CSocket* pSocket = m_vSockets[a];
-		if (pSocket->GetLabel().CaseCmp(sLabel) == 0) {
+		if (pSocket->GetSockName().Equals(sSockName)) {
 			return pSocket;
 		}
 	}
@@ -454,7 +423,7 @@ void CModule::ListSockets() {
 		CSocket* pSocket = (CSocket*) m_vSockets[a];
 
 		Table.AddRow();
-		Table.SetCell("Name", pSocket->GetLabel());
+		Table.SetCell("Name", pSocket->GetSockName());
 
 		if (pSocket->GetType() == CSocket::LISTENER) {
 			Table.SetCell("State", "Listening");
@@ -468,14 +437,7 @@ void CModule::ListSockets() {
 		Table.SetCell("RemotePort", (pSocket->GetRemotePort()) ? CString(pSocket->GetRemotePort()) : CString(""));
 	}
 
-	if (Table.size()) {
-		unsigned int uTableIdx = 0;
-		CString sLine;
-
-		while (Table.GetLine(uTableIdx++, sLine)) {
-			PutModule(sLine);
-		}
-	}
+	PutModule(Table);
 }
 
 CString CModule::GetModNick() const { return ((m_pUser) ? m_pUser->GetStatusPrefix() : "*") + m_sModName; }
@@ -486,6 +448,7 @@ void CModule::OnPreRehash() {}
 void CModule::OnPostRehash() {}
 void CModule::OnIRCDisconnected() {}
 void CModule::OnIRCConnected() {}
+CModule::EModRet CModule::OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName) { return CONTINUE; }
 CModule::EModRet CModule::OnBroadcast(CString& sMessage) { return CONTINUE; }
 
 CModule::EModRet CModule::OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize) { return CONTINUE; }
@@ -496,10 +459,11 @@ void CModule::OnDeop(const CNick& OpNick, const CNick& Nick, CChan& Channel, boo
 void CModule::OnVoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) {}
 void CModule::OnDevoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) {}
 void CModule::OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs) {}
+void CModule::OnMode(const CNick& OpNick, CChan& Channel, char uMode, const CString& sArg, bool bAdded, bool bNoChange) {}
 
 CModule::EModRet CModule::OnRaw(CString& sLine) { return CONTINUE; }
 
-CModule::EModRet CModule::OnStatusCommand(const CString& sCommand) { return CONTINUE; }
+CModule::EModRet CModule::OnStatusCommand(CString& sCommand) { return CONTINUE; }
 void CModule::OnModCommand(const CString& sCommand) {}
 void CModule::OnModNotice(const CString& sMessage) {}
 void CModule::OnModCTCP(const CString& sMessage) {}
@@ -510,8 +474,8 @@ void CModule::OnKick(const CNick& Nick, const CString& sKickedNick, CChan& Chann
 void CModule::OnJoin(const CNick& Nick, CChan& Channel) {}
 void CModule::OnPart(const CNick& Nick, CChan& Channel) {}
 
-void CModule::OnUserAttached() {}
-void CModule::OnUserDetached() {}
+void CModule::OnClientLogin() {}
+void CModule::OnClientDisconnect() {}
 CModule::EModRet CModule::OnUserRaw(CString& sLine) { return CONTINUE; }
 CModule::EModRet CModule::OnUserCTCPReply(CString& sTarget, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnUserCTCP(CString& sTarget, CString& sMessage) { return CONTINUE; }
@@ -521,6 +485,7 @@ CModule::EModRet CModule::OnUserNotice(CString& sTarget, CString& sMessage) { re
 CModule::EModRet CModule::OnUserJoin(CString& sChannel, CString& sKey) { return CONTINUE; }
 CModule::EModRet CModule::OnUserPart(CString& sChannel, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnUserTopic(CString& sChannel, CString& sTopic) { return CONTINUE; }
+CModule::EModRet CModule::OnUserTopicRequest(CString& sChannel) { return CONTINUE; }
 
 CModule::EModRet CModule::OnCTCPReply(CNick& Nick, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnPrivCTCP(CNick& Nick, CString& sMessage) { return CONTINUE; }
@@ -532,6 +497,7 @@ CModule::EModRet CModule::OnChanMsg(CNick& Nick, CChan& Channel, CString& sMessa
 CModule::EModRet CModule::OnPrivNotice(CNick& Nick, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) { return CONTINUE; }
+CModule::EModRet CModule::OnTimerAutoJoin(CChan& Channel) { return CONTINUE; }
 
 void* CModule::GetDLL() { return m_pDLL; }
 bool CModule::PutIRC(const CString& sLine) {
@@ -543,6 +509,16 @@ bool CModule::PutUser(const CString& sLine) {
 bool CModule::PutStatus(const CString& sLine) {
 	return (m_pUser) ? m_pUser->PutStatus(sLine, m_pClient) : false;
 }
+unsigned int CModule::PutModule(const CTable& table, const CString& sIdent, const CString& sHost) {
+	if (!m_pUser)
+		return 0;
+
+	unsigned int idx = 0;
+	CString sLine;
+	while (table.GetLine(idx++, sLine))
+		PutModule(sLine, sIdent, sHost);
+	return idx - 1;
+}
 bool CModule::PutModule(const CString& sLine, const CString& sIdent, const CString& sHost) {
 	if (!m_pUser)
 		return false;
@@ -565,12 +541,14 @@ bool CModule::PutModNotice(const CString& sLine, const CString& sIdent, const CS
 ///////////////////
 CModule::EModRet CGlobalModule::OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan) { return CONTINUE; }
 CModule::EModRet CGlobalModule::OnDeleteUser(CUser& User) { return CONTINUE; }
+void CGlobalModule::OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort) {}
 CModule::EModRet CGlobalModule::OnLoginAttempt(CSmartPtr<CAuthBase> Auth) { return CONTINUE; }
 void CGlobalModule::OnFailedLogin(const CString& sUsername, const CString& sRemoteIP) {}
 
 
 CModules::CModules() {
 	m_pUser = NULL;
+	m_pClient = NULL;
 }
 
 CModules::~CModules() {
@@ -585,16 +563,16 @@ void CModules::UnloadAll() {
 	}
 }
 
-void CModules::SetClient(CClient* pClient) {
-	for (unsigned int a = 0; a < size(); a++) {
-		(*this)[a]->SetClient(pClient);
-	}
-}
-
 bool CModules::OnBoot() {
 	for (unsigned int a = 0; a < size(); a++) {
-		if (!(*this)[a]->OnBoot()) {
-			return false;
+		try {
+			if (!(*this)[a]->OnBoot()) {
+				return false;
+			}
+		} catch (CModule::EModException e) {
+			if (e == CModule::UNLOAD) {
+				UnloadModule((*this)[a]->GetModName());
+			}
 		}
 	}
 
@@ -604,6 +582,7 @@ bool CModules::OnBoot() {
 bool CModules::OnPreRehash() { MODUNLOADCHK(OnPreRehash()); return false; }
 bool CModules::OnPostRehash() { MODUNLOADCHK(OnPostRehash()); return false; }
 bool CModules::OnIRCConnected() { MODUNLOADCHK(OnIRCConnected()); return false; }
+bool CModules::OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName) { MODHALTCHK(OnIRCRegistration(sPass, sNick, sIdent, sRealName)); }
 bool CModules::OnBroadcast(CString& sMessage) { MODHALTCHK(OnBroadcast(sMessage)); }
 bool CModules::OnIRCDisconnected() { MODUNLOADCHK(OnIRCDisconnected()); return false; }
 bool CModules::OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize) { MODHALTCHK(OnDCCUserSend(RemoteNick, uLongIP, uPort, sFile, uFileSize)); }
@@ -613,10 +592,11 @@ bool CModules::OnDeop(const CNick& OpNick, const CNick& Nick, CChan& Channel, bo
 bool CModules::OnVoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnVoice(OpNick, Nick, Channel, bNoChange)); return false; }
 bool CModules::OnDevoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnDevoice(OpNick, Nick, Channel, bNoChange)); return false; }
 bool CModules::OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs) { MODUNLOADCHK(OnRawMode(OpNick, Channel, sModes, sArgs)); return false; }
+bool CModules::OnMode(const CNick& OpNick, CChan& Channel, char uMode, const CString& sArg, bool bAdded, bool bNoChange) { MODUNLOADCHK(OnMode(OpNick, Channel, uMode, sArg, bAdded, bNoChange)); return false; }
 bool CModules::OnRaw(CString& sLine) { MODHALTCHK(OnRaw(sLine)); }
 
-bool CModules::OnUserAttached() { MODUNLOADCHK(OnUserAttached()); return false; }
-bool CModules::OnUserDetached() { MODUNLOADCHK(OnUserDetached()); return false; }
+bool CModules::OnClientLogin() { MODUNLOADCHK(OnClientLogin()); return false; }
+bool CModules::OnClientDisconnect() { MODUNLOADCHK(OnClientDisconnect()); return false; }
 bool CModules::OnUserRaw(CString& sLine) { MODHALTCHK(OnUserRaw(sLine)); }
 bool CModules::OnUserCTCPReply(CString& sTarget, CString& sMessage) { MODHALTCHK(OnUserCTCPReply(sTarget, sMessage)); }
 bool CModules::OnUserCTCP(CString& sTarget, CString& sMessage) { MODHALTCHK(OnUserCTCP(sTarget, sMessage)); }
@@ -626,6 +606,7 @@ bool CModules::OnUserNotice(CString& sTarget, CString& sMessage) { MODHALTCHK(On
 bool CModules::OnUserJoin(CString& sChannel, CString& sKey) { MODHALTCHK(OnUserJoin(sChannel, sKey)); }
 bool CModules::OnUserPart(CString& sChannel, CString& sMessage) { MODHALTCHK(OnUserPart(sChannel, sMessage)); }
 bool CModules::OnUserTopic(CString& sChannel, CString& sTopic) { MODHALTCHK(OnUserTopic(sChannel, sTopic)); }
+bool CModules::OnUserTopicRequest(CString& sChannel) { MODHALTCHK(OnUserTopicRequest(sChannel)); }
 
 bool CModules::OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans) { MODUNLOADCHK(OnQuit(Nick, sMessage, vChans)); return false; }
 bool CModules::OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans) { MODUNLOADCHK(OnNick(Nick, sNewNick, vChans)); return false; }
@@ -642,7 +623,8 @@ bool CModules::OnChanMsg(CNick& Nick, CChan& Channel, CString& sMessage) { MODHA
 bool CModules::OnPrivNotice(CNick& Nick, CString& sMessage) { MODHALTCHK(OnPrivNotice(Nick, sMessage)); }
 bool CModules::OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage) { MODHALTCHK(OnChanNotice(Nick, Channel, sMessage)); }
 bool CModules::OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) { MODHALTCHK(OnTopic(Nick, Channel, sTopic)); }
-bool CModules::OnStatusCommand(const CString& sCommand) { MODHALTCHK(OnStatusCommand(sCommand)); }
+bool CModules::OnTimerAutoJoin(CChan& Channel) { MODHALTCHK(OnTimerAutoJoin(Channel)); }
+bool CModules::OnStatusCommand(CString& sCommand) { MODHALTCHK(OnStatusCommand(sCommand)); }
 bool CModules::OnModCommand(const CString& sCommand) { MODUNLOADCHK(OnModCommand(sCommand)); return false; }
 bool CModules::OnModNotice(const CString& sMessage) { MODUNLOADCHK(OnModNotice(sMessage)); return false; }
 bool CModules::OnModCTCP(const CString& sMessage) { MODUNLOADCHK(OnModCTCP(sMessage)); return false; }
@@ -658,6 +640,10 @@ bool CGlobalModules::OnDeleteUser(CUser& User) {
 	GLOBALMODHALTCHK(OnDeleteUser(User));
 }
 
+void CGlobalModules::OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort) {
+	GLOBALMODCALL(OnClientConnect(pClient, sHost, uPort));
+}
+
 bool CGlobalModules::OnLoginAttempt(CSmartPtr<CAuthBase> Auth) {
 	GLOBALMODHALTCHK(OnLoginAttempt(Auth));
 }
@@ -668,7 +654,7 @@ void CGlobalModules::OnFailedLogin(const CString& sUsername, const CString& sRem
 
 CModule* CModules::FindModule(const CString& sModule) const {
 	for (unsigned int a = 0; a < size(); a++) {
-		if (sModule.CaseCmp((*this)[a]->GetModName()) == 0) {
+		if (sModule.Equals((*this)[a]->GetModName())) {
 			return (*this)[a];
 		}
 	}
@@ -734,7 +720,6 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 	if (CModule::GetCoreVersion() != Version()) {
 		dlclose(p);
 		sRetMsg = "Version mismatch, recompile this module.";
-		throw CException(CException::EX_BadModVersion);
 		return false;
 	}
 
@@ -796,7 +781,15 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 	pModule->SetDescription(GetDesc());
 	push_back(pModule);
 
-	if (!pModule->OnLoad(sArgs, sRetMsg)) {
+	bool bLoaded;
+	try {
+		bLoaded = pModule->OnLoad(sArgs, sRetMsg);
+	} catch (CModule::EModException e) {
+		bLoaded = false;
+		sRetMsg = "Caught an exception";
+	}
+
+	if (!bLoaded) {
 		UnloadModule(sModule, sModPath);
 		if (!sRetMsg.empty())
 			sRetMsg = "Module [" + sModule + "] aborted: " + sRetMsg;
@@ -886,11 +879,7 @@ bool CModules::ReloadModule(const CString& sModule, const CString& sArgs, CUser*
 		return false;
 	}
 
-	try {
-		if (!LoadModule(sMod, sArgs, pUser, sRetMsg)) {
-			return false;
-		}
-	} catch(...) {
+	if (!LoadModule(sMod, sArgs, pUser, sRetMsg)) {
 		return false;
 	}
 
