@@ -132,34 +132,34 @@ private:
 
 class CSocket : public Csock {
 public:
-	CSocket(CModule* pModule, const CString& sLabel);
-	CSocket(CModule* pModule, const CString& sLabel, const CString& sHostname, unsigned short uPort, int iTimeout = 60);
+	CSocket(CModule* pModule);
+	CSocket(CModule* pModule, const CString& sHostname, unsigned short uPort, int iTimeout = 60);
 	virtual ~CSocket();
 
 	using Csock::Connect;
 	using Csock::Listen;
 
+	// This defaults to closing the socket, feel free to override
+	virtual void ReachedMaxBuffer();
+
 	bool Connect(const CString& sHostname, unsigned short uPort, bool bSSL = false, unsigned int uTimeout = 60);
 	bool Listen(unsigned short uPort, bool bSSL = false, unsigned int uTimeout = 0);
 	virtual bool PutIRC(const CString& sLine);
 	virtual bool PutUser(const CString& sLine);
 	virtual bool PutStatus(const CString& sLine);
-	virtual bool PutModule(const CString& sLine, const CString& sIdent = "", const CString& sHost = "znc.com");
-	virtual bool PutModNotice(const CString& sLine, const CString& sIdent = "", const CString& sHost = "znc.com");
+	virtual bool PutModule(const CString& sLine, const CString& sIdent = "", const CString& sHost = "znc.in");
+	virtual bool PutModNotice(const CString& sLine, const CString& sIdent = "", const CString& sHost = "znc.in");
 
 	// Setters
 	void SetModule(CModule* p);
-	void SetLabel(const CString& s);
 	// !Setters
 
 	// Getters
 	CModule* GetModule() const;
-	const CString& GetLabel() const;
 	// !Getters
 private:
 protected:
 	CModule*	m_pModule;
-	CString		m_sLabel;
 };
 
 class CModInfo {
@@ -229,6 +229,7 @@ public:
 	virtual void OnPostRehash();
 	virtual void OnIRCDisconnected();
 	virtual void OnIRCConnected();
+	virtual EModRet OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName);
 	virtual EModRet OnBroadcast(CString& sMessage);
 
 	virtual EModRet OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize);
@@ -239,10 +240,11 @@ public:
 	virtual void OnVoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
 	virtual void OnDevoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
 	virtual void OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs);
+	virtual void OnMode(const CNick& OpNick, CChan& Channel, char uMode, const CString& sArg, bool bAdded, bool bNoChange);
 
 	virtual EModRet OnRaw(CString& sLine);
 
-	virtual EModRet OnStatusCommand(const CString& sCommand);
+	virtual EModRet OnStatusCommand(CString& sCommand);
 	virtual void OnModCommand(const CString& sCommand);
 	virtual void OnModNotice(const CString& sMessage);
 	virtual void OnModCTCP(const CString& sMessage);
@@ -253,8 +255,10 @@ public:
 	virtual void OnJoin(const CNick& Nick, CChan& Channel);
 	virtual void OnPart(const CNick& Nick, CChan& Channel);
 
-	virtual void OnUserAttached();
-	virtual void OnUserDetached();
+	virtual void OnUserAttached(bool thisFunctionWasRemoved, bool UseOnClientLogin) {}
+	virtual void OnUserDetached(bool thisFunctionWasRemoved, bool UseOnClientDisconnect) {}
+	virtual void OnClientLogin();
+	virtual void OnClientDisconnect();
 	virtual EModRet OnUserRaw(CString& sLine);
 	virtual EModRet OnUserCTCPReply(CString& sTarget, CString& sMessage);
 	virtual EModRet OnUserCTCP(CString& sTarget, CString& sMessage);
@@ -264,6 +268,7 @@ public:
 	virtual EModRet OnUserJoin(CString& sChannel, CString& sKey);
 	virtual EModRet OnUserPart(CString& sChannel, CString& sMessage);
 	virtual EModRet OnUserTopic(CString& sChannel, CString& sTopic);
+	virtual EModRet OnUserTopicRequest(CString& sChannel);
 
 	virtual EModRet OnCTCPReply(CNick& Nick, CString& sMessage);
 	virtual EModRet OnPrivCTCP(CNick& Nick, CString& sMessage);
@@ -275,6 +280,7 @@ public:
 	virtual EModRet OnPrivNotice(CNick& Nick, CString& sMessage);
 	virtual EModRet OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage);
 	virtual EModRet OnTopic(CNick& Nick, CChan& Channel, CString& sTopic);
+	virtual EModRet OnTimerAutoJoin(CChan& Channel);
 
 	void * GetDLL();
 	static double GetCoreVersion() { return VERSION; }
@@ -282,8 +288,9 @@ public:
 	virtual bool PutIRC(const CString& sLine);
 	virtual bool PutUser(const CString& sLine);
 	virtual bool PutStatus(const CString& sLine);
-	virtual bool PutModule(const CString& sLine, const CString& sIdent = "", const CString& sHost = "znc.com");
-	virtual bool PutModNotice(const CString& sLine, const CString& sIdent = "", const CString& sHost = "znc.com");
+	virtual bool PutModule(const CString& sLine, const CString& sIdent = "", const CString& sHost = "znc.in");
+	virtual unsigned int PutModule(const CTable& table, const CString& sIdent = "", const CString& sHost = "znc.in");
+	virtual bool PutModNotice(const CString& sLine, const CString& sIdent = "", const CString& sHost = "znc.in");
 
 	const CString& GetModName() const { return m_sModName; }
 	// This is where constant module data (e.g. webadmin skins) are saved
@@ -302,9 +309,9 @@ public:
 	// Socket stuff
 	bool AddSocket(CSocket* pSocket);
 	bool RemSocket(CSocket* pSocket);
-	bool RemSocket(const CString& sLabel);
+	bool RemSocket(const CString& sSockName);
 	bool UnlinkSocket(CSocket* pSocket);
-	CSocket* FindSocket(const CString& sLabel);
+	CSocket* FindSocket(const CString& sSockName);
 	virtual void ListSockets();
 	// !Socket stuff
 
@@ -358,7 +365,7 @@ public:
 	virtual ~CModules();
 
 	void SetUser(CUser* pUser) { m_pUser = pUser; }
-	void SetClient(CClient* pClient);
+	void SetClient(CClient* pClient) { m_pClient = pClient; }
 
 	void UnloadAll();
 
@@ -367,6 +374,7 @@ public:
 	virtual bool OnPostRehash();
 	virtual bool OnIRCDisconnected();
 	virtual bool OnIRCConnected();
+	virtual bool OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName);
 	virtual bool OnBroadcast(CString& sMessage);
 
 	virtual bool OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize);
@@ -377,10 +385,11 @@ public:
 	virtual bool OnVoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
 	virtual bool OnDevoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange);
 	virtual bool OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs);
+	virtual bool OnMode(const CNick& OpNick, CChan& Channel, char uMode, const CString& sArg, bool bAdded, bool bNoChange);
 
 	virtual bool OnRaw(CString& sLine);
 
-	virtual bool OnStatusCommand(const CString& sCommand);
+	virtual bool OnStatusCommand(CString& sCommand);
 	virtual bool OnModCommand(const CString& sCommand);
 	virtual bool OnModNotice(const CString& sMessage);
 	virtual bool OnModCTCP(const CString& sMessage);
@@ -391,8 +400,8 @@ public:
 	virtual bool OnJoin(const CNick& Nick, CChan& Channel);
 	virtual bool OnPart(const CNick& Nick, CChan& Channel);
 
-	virtual bool OnUserAttached();
-	virtual bool OnUserDetached();
+	virtual bool OnClientLogin();
+	virtual bool OnClientDisconnect();
 	virtual bool OnUserRaw(CString& sLine);
 	virtual bool OnUserCTCPReply(CString& sTarget, CString& sMessage);
 	virtual bool OnUserCTCP(CString& sTarget, CString& sMessage);
@@ -402,6 +411,7 @@ public:
 	virtual bool OnUserJoin(CString& sChannel, CString& sKey);
 	virtual bool OnUserPart(CString& sChannel, CString& sMessage);
 	virtual bool OnUserTopic(CString& sChannel, CString& sTopic);
+	virtual bool OnUserTopicRequest(CString& sChannel);
 
 	virtual bool OnCTCPReply(CNick& Nick, CString& sMessage);
 	virtual bool OnPrivCTCP(CNick& Nick, CString& sMessage);
@@ -413,6 +423,7 @@ public:
 	virtual bool OnPrivNotice(CNick& Nick, CString& sMessage);
 	virtual bool OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage);
 	virtual bool OnTopic(CNick& Nick, CChan& Channel, CString& sTopic);
+	virtual bool OnTimerAutoJoin(CChan& Channel);
 
 	CModule* FindModule(const CString& sModule) const;
 	bool LoadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg, bool bFake = false);
@@ -425,6 +436,7 @@ public:
 
 protected:
 	CUser*		m_pUser;
+	CClient*	m_pClient;
 };
 
 class CGlobalModule : public CModule {
@@ -435,6 +447,7 @@ public:
 
 	virtual EModRet OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan);
 	virtual EModRet OnDeleteUser(CUser& User);
+	virtual void OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort);
 	virtual EModRet OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
 	virtual void OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
 private:
@@ -447,6 +460,7 @@ public:
 
 	virtual bool OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan);
 	virtual bool OnDeleteUser(CUser& User);
+	virtual void OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort);
 	virtual bool OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
 	virtual void OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
 private:
