@@ -24,6 +24,7 @@ CNick::~CNick() {}
 
 void CNick::Reset() {
 	m_cPerm = '\0';
+	m_suChanPerms.clear();
 }
 
 CString CNick::Concat(const CString& sNick, const CString& sSuffix, unsigned int uMaxNickLen) {
@@ -65,7 +66,7 @@ unsigned int CNick::GetCommonChans(vector<CChan*>& vRetChans, CUser* pUser) cons
 		const map<CString,CNick*>& msNicks = pChan->GetNicks();
 
 		for (map<CString,CNick*>::const_iterator it = msNicks.begin(); it != msNicks.end(); it++) {
-			if (it->first.CaseCmp(m_sNick) == 0) {
+			if (it->first.Equals(m_sNick)) {
 				vRetChans.push_back(pChan);
 				continue;
 			}
