@@ -30,12 +30,10 @@ CUser::CUser(const CString& sUserName) {
 #endif
 	m_RawBuffer.SetLineCount(100);		// This should be more than enough raws, especially since we are buffering the MOTD separately
 	m_MotdBuffer.SetLineCount(200);		// This should be more than enough motd lines
-	m_bIRCConnected = false;
 	m_bMultiClients = true;
 	m_bBounceDCCs = true;
 	m_bPassHashed = false;
 	m_bUseClientIP = false;
-	m_bKeepNick = false;
 	m_bDenyLoadMod = false;
 	m_bAdmin= false;
 	m_bDenySetVHost= false;
@@ -43,17 +41,15 @@ CUser::CUser(const CString& sUserName) {
 	m_sChanPrefixes = "";
 	m_uBufferCount = 50;
 	m_uMaxJoinTries = 0;
+	m_uMaxJoins = 5;
 	m_bKeepBuffer = false;
-	m_bAutoCycle = true;
 	m_bBeingDeleted = false;
 	m_sTimestampFormat = "[%H:%M:%S]";
 	m_bAppendTimestamp = false;
 	m_bPrependTimestamp = true;
 	m_bIRCConnectEnabled = true;
-	m_pKeepNickTimer = new CKeepNickTimer(this);
 	m_pJoinTimer = new CJoinTimer(this);
 	m_pMiscTimer = new CMiscTimer(this);
-	CZNC::Get().GetManager().AddCron(m_pKeepNickTimer);
 	CZNC::Get().GetManager().AddCron(m_pJoinTimer);
 	CZNC::Get().GetManager().AddCron(m_pMiscTimer);
 	m_sUserPath = CZNC::Get().GetUserPath() + "/" + sUserName;
@@ -73,7 +69,6 @@ CUser::~CUser() {
 		delete m_vChans[b];
 	}
 
-	CZNC::Get().GetManager().DelCronByAddr(m_pKeepNickTimer);
 	CZNC::Get().GetManager().DelCronByAddr(m_pJoinTimer);
 	CZNC::Get().GetManager().DelCronByAddr(m_pMiscTimer);
 }
@@ -114,16 +109,16 @@ void CUser::IRCConnected(CIRCSock* pIRCSock) {
 }
 
 void CUser::IRCDisconnected() {
-	m_bIRCConnected = false;
-
 	m_pIRCSock = NULL;
 
 	for (unsigned int a = 0; a < m_vClients.size(); a++) {
 		m_vClients[a]->IRCDisconnected();
 	}
 
+	SetIRCServer("");
+
 	// Get the reconnect going
-	CZNC::Get().EnableConnectUser();
+	CheckIRCConnect();
 }
 
 CString CUser::ExpandString(const CString& sStr) const {
@@ -208,7 +203,7 @@ void CUser::UserConnected(CClient* pClient) {
 	m_vClients.push_back(pClient);
 
 	if (m_RawBuffer.IsEmpty()) {
-		pClient->PutClient(":irc.znc.com 001 " + pClient->GetNick() + " :- Welcome to ZNC -");
+		pClient->PutClient(":irc.znc.in 001 " + pClient->GetNick() + " :- Welcome to ZNC -");
 	} else {
 		unsigned int uIdx = 0;
 		CString sLine;
@@ -216,6 +211,9 @@ void CUser::UserConnected(CClient* pClient) {
 		while (m_RawBuffer.GetLine(GetIRCNick().GetNick(), sLine, uIdx++)) {
 			pClient->PutClient(sLine);
 		}
+
+		// The assumption is that the client got this nick from the 001 reply
+		pClient->SetNick(GetIRCNick().GetNick());
 	}
 
 	// Send the cached MOTD
@@ -287,7 +285,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	}
 
 	if (!User.GetPass().empty()) {
-		SetPass(User.GetPass(), User.IsPassHashed());
+		SetPass(User.GetPass(), User.IsPassHashed(), User.GetPassSalt());
 	}
 
 	SetNick(User.GetNick(false));
@@ -300,6 +298,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	SetDefaultChanModes(User.GetDefaultChanModes());
 	SetBufferCount(User.GetBufferCount());
 	SetJoinTries(User.JoinTries());
+	SetMaxJoins(User.MaxJoins());
 
 	// Allowed Hosts
 	m_ssAllowedHosts.clear();
@@ -331,13 +330,9 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 		CModule* pCurMod = vCurMods.FindModule(pNewMod->GetModName());
 
 		if (!pCurMod) {
-			try {
-				vCurMods.LoadModule(pNewMod->GetModName(), pNewMod->GetArgs(), this, sModRet);
-			} catch (...) {}
+			vCurMods.LoadModule(pNewMod->GetModName(), pNewMod->GetArgs(), this, sModRet);
 		} else if (pNewMod->GetArgs() != pCurMod->GetArgs()) {
-			try {
-				vCurMods.ReloadModule(pNewMod->GetModName(), pNewMod->GetArgs(), this, sModRet);
-			} catch (...) {}
+			vCurMods.ReloadModule(pNewMod->GetModName(), pNewMod->GetArgs(), this, sModRet);
 		}
 	}
 
@@ -373,7 +368,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	}
 
 	for (a = 0; a < m_vServers.size(); a++) {
-		if (sServer.CaseCmp(m_vServers[a]->GetName()) == 0) {
+		if (sServer.Equals(m_vServers[a]->GetName())) {
 			m_uServerIdx = a +1;
 			break;
 		}
@@ -414,8 +409,6 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 				pChan->Clone(*pNewChan);
 		}
 	}
-
-	JoinChans();
 	// !Chans
 
 	// CTCP Replies
@@ -428,8 +421,6 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 
 	// Flags
 	SetKeepBuffer(User.KeepBuffer());
-	SetAutoCycle(User.AutoCycle());
-	SetKeepNick(User.GetKeepNick());
 	SetMultiClients(User.MultiClients());
 	SetBounceDCCs(User.BounceDCCs());
 	SetUseClientIP(User.UseClientIP());
@@ -455,7 +446,7 @@ bool CUser::AddAllowedHost(const CString& sHostMask) {
 	return true;
 }
 
-bool CUser::IsHostAllowed(const CString& sHostMask) {
+bool CUser::IsHostAllowed(const CString& sHostMask) const {
 	if (m_ssAllowedHosts.empty()) {
 		return true;
 	}
@@ -522,7 +513,7 @@ bool CUser::AddChan(CChan* pChan) {
 	}
 
 	for (unsigned int a = 0; a < m_vChans.size(); a++) {
-		if (m_vChans[a]->GetName().CaseCmp(pChan->GetName()) == 0) {
+		if (m_vChans[a]->GetName().Equals(pChan->GetName())) {
 			delete pChan;
 			return false;
 		}
@@ -544,7 +535,7 @@ bool CUser::AddChan(const CString& sName, bool bInConfig) {
 
 bool CUser::DelChan(const CString& sName) {
 	for (vector<CChan*>::iterator a = m_vChans.begin(); a != m_vChans.end(); a++) {
-		if (sName.CaseCmp((*a)->GetName()) == 0) {
+		if (sName.Equals((*a)->GetName())) {
 			delete *a;
 			m_vChans.erase(a);
 			return true;
@@ -565,7 +556,15 @@ bool CUser::PrintLine(CFile& File, const CString& sName, const CString& sValue)
 bool CUser::WriteConfig(CFile& File) {
 	File.Write("<User " + GetUserName() + ">\n");
 
-	PrintLine(File, "Pass", GetPass() + ((IsPassHashed()) ? " -" : ""));
+	if (IsPassHashed()) {
+		if (m_sPassSalt.empty()) {
+			PrintLine(File, "Pass", "md5#" + GetPass());
+		} else {
+			PrintLine(File, "Pass", "md5#" + GetPass() + "#" + m_sPassSalt + "#");
+		}
+	} else {
+		PrintLine(File, "Pass", "plain#" + GetPass());
+	}
 	PrintLine(File, "Nick", GetNick());
 	PrintLine(File, "AltNick", GetAltNick());
 	PrintLine(File, "Ident", GetIdent());
@@ -576,20 +575,19 @@ bool CUser::WriteConfig(CFile& File) {
 		PrintLine(File, "StatusPrefix", GetStatusPrefix());
 	PrintLine(File, "ChanModes", GetDefaultChanModes());
 	PrintLine(File, "Buffer", CString(GetBufferCount()));
-	PrintLine(File, "KeepNick", CString((GetKeepNick()) ? "true" : "false"));
-	PrintLine(File, "KeepBuffer", CString((KeepBuffer()) ? "true" : "false"));
-	PrintLine(File, "MultiClients", CString((MultiClients()) ? "true" : "false"));
-	PrintLine(File, "BounceDCCs", CString((BounceDCCs()) ? "true" : "false"));
-	PrintLine(File, "AutoCycle", CString((AutoCycle()) ? "true" : "false"));
-	PrintLine(File, "DenyLoadMod", CString((DenyLoadMod()) ? "true" : "false"));
-	PrintLine(File, "Admin", CString((IsAdmin()) ? "true" : "false"));
-	PrintLine(File, "DenySetVHost", CString((DenySetVHost()) ? "true" : "false"));
+	PrintLine(File, "KeepBuffer", CString(KeepBuffer()));
+	PrintLine(File, "MultiClients", CString(MultiClients()));
+	PrintLine(File, "BounceDCCs", CString(BounceDCCs()));
+	PrintLine(File, "DenyLoadMod", CString(DenyLoadMod()));
+	PrintLine(File, "Admin", CString(IsAdmin()));
+	PrintLine(File, "DenySetVHost", CString(DenySetVHost()));
 	PrintLine(File, "DCCLookupMethod", CString((UseClientIP()) ? "client" : "default"));
 	PrintLine(File, "TimestampFormat", GetTimestampFormat());
-	PrintLine(File, "AppendTimestamp", CString((GetTimestampAppend()) ? "true" : "false"));
-	PrintLine(File, "PrependTimestamp", CString((GetTimestampPrepend()) ? "true" : "false"));
+	PrintLine(File, "AppendTimestamp", CString(GetTimestampAppend()));
+	PrintLine(File, "PrependTimestamp", CString(GetTimestampPrepend()));
 	PrintLine(File, "TimezoneOffset", CString(m_fTimezoneOffset));
 	PrintLine(File, "JoinTries", CString(m_uMaxJoinTries));
+	PrintLine(File, "MaxJoins", CString(m_uMaxJoins));
 	File.Write("\n");
 
 	// Allow Hosts
@@ -653,7 +651,7 @@ bool CUser::WriteConfig(CFile& File) {
 CChan* CUser::FindChan(const CString& sName) const {
 	for (unsigned int a = 0; a < m_vChans.size(); a++) {
 		CChan* pChan = m_vChans[a];
-		if (sName.CaseCmp(pChan->GetName()) == 0) {
+		if (sName.Equals(pChan->GetName())) {
 			return pChan;
 		}
 	}
@@ -662,6 +660,7 @@ CChan* CUser::FindChan(const CString& sName) const {
 }
 
 void CUser::JoinChans() {
+	unsigned int uJoins = m_uMaxJoins;
 	for (unsigned int a = 0; a < m_vChans.size(); a++) {
 		CChan* pChan = m_vChans[a];
 		if (!pChan->IsOn() && !pChan->IsDisabled()) {
@@ -670,16 +669,22 @@ void CUser::JoinChans() {
 				pChan->Disable();
 			} else {
 				pChan->IncJoinTries();
+				MODULECALL(OnTimerAutoJoin(*pChan), this, NULL, continue);
+
 				PutIRC("JOIN " + pChan->GetName() + " " + pChan->GetKey());
+
+				// Limit the number of joins
+				if (uJoins != 0 && --uJoins == 0)
+					return;
 			}
 		}
 	}
 }
 
-CServer* CUser::FindServer(const CString& sName) {
+CServer* CUser::FindServer(const CString& sName) const {
 	for (unsigned int a = 0; a < m_vServers.size(); a++) {
 		CServer* pServer = m_vServers[a];
-		if (sName.CaseCmp(pServer->GetName()) == 0) {
+		if (sName.Equals(pServer->GetName())) {
 			return pServer;
 		}
 	}
@@ -697,7 +702,7 @@ bool CUser::DelServer(const CString& sName) {
 	for (vector<CServer*>::iterator it = m_vServers.begin(); it != m_vServers.end(); it++, a++) {
 		CServer* pServer = *it;
 
-		if (pServer->GetName().CaseCmp(sName) == 0) {
+		if (pServer->GetName().Equals(sName)) {
 			CServer* pCurServer = GetCurrentServer();
 			m_vServers.erase(it);
 
@@ -777,7 +782,7 @@ bool CUser::AddServer(const CString& sName, unsigned short uPort, const CString&
 	return true;
 }
 
-bool CUser::IsLastServer() {
+bool CUser::IsLastServer() const {
 	return (m_uServerIdx >= m_vServers.size());
 }
 
@@ -793,7 +798,7 @@ CServer* CUser::GetNextServer() {
 	return m_vServers[m_uServerIdx++];	// Todo: cycle through these
 }
 
-CServer* CUser::GetCurrentServer() {
+CServer* CUser::GetCurrentServer() const {
 	unsigned int uIdx = (m_uServerIdx) ? m_uServerIdx -1 : 0;
 
 	if (uIdx >= m_vServers.size()) {
@@ -803,12 +808,14 @@ CServer* CUser::GetCurrentServer() {
 	return m_vServers[uIdx];
 }
 
-bool CUser::CheckPass(const CString& sPass) {
+bool CUser::CheckPass(const CString& sPass) const {
 	if (!m_bPassHashed) {
 		return (sPass == m_sPass);
 	}
 
-	return (m_sPass.CaseCmp(sPass.MD5()) == 0);
+	CString sSaltedPass = sPass + m_sPassSalt;
+
+	return (m_sPass.Equals(sSaltedPass.MD5()));
 }
 
 /*CClient* CUser::GetClient() {
@@ -818,7 +825,7 @@ bool CUser::CheckPass(const CString& sPass) {
 
 	for (unsigned int a = 0; a < Manager.size(); a++) {
 		Csock* pSock = Manager[a];
-		if (pSock->GetSockName().CaseCmp(sSockName) == 0) {
+		if (pSock->GetSockName().Equals(sSockName)) {
 			if (!pSock->IsClosed()) {
 				return (CClient*) pSock;
 			}
@@ -828,14 +835,6 @@ bool CUser::CheckPass(const CString& sPass) {
 	return (CClient*) CZNC::Get().GetManager().FindSockByName(sSockName);
 }*/
 
-CIRCSock* CUser::GetIRCSock() {
-	return m_pIRCSock;
-}
-
-const CIRCSock* CUser::GetIRCSock() const {
-	return m_pIRCSock;
-}
-
 CString CUser::GetLocalIP() {
 	CIRCSock* pIRCSock = GetIRCSock();
 
@@ -917,11 +916,11 @@ bool CUser::PutModule(const CString& sModule, const CString& sLine, CClient* pCl
 	return (pClient == NULL);
 }
 
-bool CUser::ResumeFile(const CString& sRemoteNick, unsigned short uPort, unsigned long uFileSize) {
+bool CUser::ResumeFile(unsigned short uPort, unsigned long uFileSize) {
 	CSockManager& Manager = CZNC::Get().GetManager();
 
 	for (unsigned int a = 0; a < Manager.size(); a++) {
-		if (strncasecmp(Manager[a]->GetSockName().c_str(), "DCC::LISTEN::", 13) == 0) {
+		if (Manager[a]->GetSockName().Equals("DCC::LISTEN::", false, 13)) {
 			CDCCSock* pSock = (CDCCSock*) Manager[a];
 
 			if (pSock->GetLocalPort() == uPort) {
@@ -951,8 +950,8 @@ bool CUser::SendFile(const CString& sRemoteNick, const CString& sFileName, const
 
 	unsigned short uPort = CZNC::Get().GetManager().ListenRand("DCC::LISTEN::" + sRemoteNick, GetLocalIP(), false, SOMAXCONN, pSock, 120);
 
-	if (GetNick().CaseCmp(sRemoteNick) == 0) {
-		PutUser(":" + GetStatusPrefix() + "status!znc@znc.com PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
+	if (GetNick().Equals(sRemoteNick)) {
+		PutUser(":" + GetStatusPrefix() + "status!znc@znc.in PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
 				+ CString(uPort) + " " + CString(pFile->GetSize()) + "\001");
 	} else {
 		PutIRC("PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
@@ -1023,20 +1022,22 @@ void CUser::SetAltNick(const CString& s) { m_sAltNick = s; }
 void CUser::SetIdent(const CString& s) { m_sIdent = s; }
 void CUser::SetRealName(const CString& s) { m_sRealName = s; }
 void CUser::SetVHost(const CString& s) { m_sVHost = s; }
-void CUser::SetPass(const CString& s, bool bHashed) { m_sPass = s; m_bPassHashed = bHashed; }
+void CUser::SetPass(const CString& s, bool bHashed, const CString& sSalt) {
+	m_sPass = s;
+	m_bPassHashed = bHashed;
+	m_sPassSalt = sSalt;
+}
 void CUser::SetMultiClients(bool b) { m_bMultiClients = b; }
 void CUser::SetBounceDCCs(bool b) { m_bBounceDCCs = b; }
 void CUser::SetUseClientIP(bool b) { m_bUseClientIP = b; }
-void CUser::SetKeepNick(bool b) { m_bKeepNick = b; }
 void CUser::SetDenyLoadMod(bool b) { m_bDenyLoadMod = b; }
 void CUser::SetAdmin(bool b) { m_bAdmin = b; }
 void CUser::SetDenySetVHost(bool b) { m_bDenySetVHost = b; }
 void CUser::SetDefaultChanModes(const CString& s) { m_sDefaultChanModes = s; }
-void CUser::SetIRCServer(const CString& s) { m_sIRCServer = s; m_bIRCConnected = true; }
+void CUser::SetIRCServer(const CString& s) { m_sIRCServer = s; }
 void CUser::SetQuitMsg(const CString& s) { m_sQuitMsg = s; }
 void CUser::SetBufferCount(unsigned int u) { m_uBufferCount = u; }
 void CUser::SetKeepBuffer(bool b) { m_bKeepBuffer = b; }
-void CUser::SetAutoCycle(bool b) { m_bAutoCycle = b; }
 
 void CUser::CheckIRCConnect()
 {
@@ -1082,6 +1083,7 @@ const CString& CUser::GetRealName() const { return m_sRealName.empty() ? m_sUser
 const CString& CUser::GetVHost() const { return m_sVHost; }
 const CString& CUser::GetPass() const { return m_sPass; }
 bool CUser::IsPassHashed() const { return m_bPassHashed; }
+const CString& CUser::GetPassSalt() const { return m_sPassSalt; }
 
 bool CUser::ConnectPaused() {
 	if (!m_uConnectTime) {
@@ -1098,7 +1100,6 @@ bool CUser::ConnectPaused() {
 }
 
 bool CUser::UseClientIP() const { return m_bUseClientIP; }
-bool CUser::GetKeepNick() const { return m_bKeepNick; }
 bool CUser::DenyLoadMod() const { return m_bDenyLoadMod; }
 bool CUser::IsAdmin() const { return m_bAdmin; }
 bool CUser::DenySetVHost() const { return m_bDenySetVHost; }
@@ -1114,5 +1115,4 @@ CString CUser::GetQuitMsg() const { return (!m_sQuitMsg.empty()) ? m_sQuitMsg :
 const MCString& CUser::GetCTCPReplies() const { return m_mssCTCPReplies; }
 unsigned int CUser::GetBufferCount() const { return m_uBufferCount; }
 bool CUser::KeepBuffer() const { return m_bKeepBuffer; }
-bool CUser::AutoCycle() const { return m_bAutoCycle; }
 // !Getters
