@@ -26,7 +26,6 @@ class CChan;
 class CClient;
 class CIRCSock;
 class CJoinTimer;
-class CKeepNickTimer;
 class CMiscTimer;
 class CServer;
 
@@ -42,19 +41,19 @@ public:
 	bool AddChan(const CString& sName, bool bInConfig);
 	bool DelChan(const CString& sName);
 	void JoinChans();
-	CServer* FindServer(const CString& sName);
+	CServer* FindServer(const CString& sName) const;
 	bool DelServer(const CString& sName);
 	bool AddServer(const CString& sName, bool bIPV6 = false);
 	bool AddServer(const CString& sName, unsigned short uPort, const CString& sPass = "", bool bSSL = false, bool bIPV6 = false);
 	CServer* GetNextServer();
-	CServer* GetCurrentServer();
-	bool CheckPass(const CString& sPass);
+	CServer* GetCurrentServer() const;
+	bool CheckPass(const CString& sPass) const;
 	bool AddAllowedHost(const CString& sHostMask);
-	bool IsHostAllowed(const CString& sHostMask);
+	bool IsHostAllowed(const CString& sHostMask) const;
 	bool IsValid(CString& sErrMsg, bool bSkipPass = false) const;
 	static bool IsValidUserName(const CString& sUserName);
 	static CString MakeCleanUserName(const CString& sUserName);
-	bool IsLastServer();
+	bool IsLastServer() const;
 	bool ConnectPaused();
 
 	void DelClients();
@@ -86,12 +85,12 @@ public:
 	bool PutStatusNotice(const CString& sLine, CClient* pClient = NULL, CClient* pSkipClient = NULL);
 	bool PutModule(const CString& sModule, const CString& sLine, CClient* pClient = NULL, CClient* pSkipClient = NULL);
 
-	bool IsUserAttached() { return (m_vClients.size() > 0); }
+	bool IsUserAttached() const { return (m_vClients.size() > 0); }
 	void UserConnected(CClient* pClient);
 	void UserDisconnected(CClient* pClient);
 
 	CString GetLocalIP();
-	bool IsIRCConnected() { return m_bIRCConnected; }
+	bool IsIRCConnected() const { return GetIRCSock() != NULL; }
 	void IRCConnected(CIRCSock* pIRCSock);
 	void IRCDisconnected();
 	void CheckIRCConnect();
@@ -104,7 +103,7 @@ public:
 
 	bool SendFile(const CString& sRemoteNick, const CString& sFileName, const CString& sModuleName = "");
 	bool GetFile(const CString& sRemoteNick, const CString& sRemoteIP, unsigned short uRemotePort, const CString& sFileName, unsigned long uFileSize, const CString& sModuleName = "");
-	bool ResumeFile(const CString& sRemoteNick, unsigned short uPort, unsigned long uFileSize);
+	bool ResumeFile(unsigned short uPort, unsigned long uFileSize);
 	CString GetCurNick() const;
 	bool Clone(const CUser& User, CString& sErrorRet, bool bCloneChans = true);
 	void BounceAllClients();
@@ -119,11 +118,10 @@ public:
 	void SetIdent(const CString& s);
 	void SetRealName(const CString& s);
 	void SetVHost(const CString& s);
-	void SetPass(const CString& s, bool bHashed);
+	void SetPass(const CString& s, bool bHashed, const CString& sSalt = "");
 	void SetBounceDCCs(bool b);
 	void SetMultiClients(bool b);
 	void SetUseClientIP(bool b);
-	void SetKeepNick(bool b);
 	void SetDenyLoadMod(bool b);
 	void SetAdmin(bool b);
 	void SetDenySetVHost(bool b);
@@ -135,7 +133,6 @@ public:
 	bool AddCTCPReply(const CString& sCTCP, const CString& sReply);
 	void SetBufferCount(unsigned int u);
 	void SetKeepBuffer(bool b);
-	void SetAutoCycle(bool b);
 	void SetChanPrefixes(const CString& s) { m_sChanPrefixes = s; }
 	void SetBeingDeleted(bool b) { m_bBeingDeleted = b; }
 	void SetTimestampFormat(const CString& s) { m_sTimestampFormat = s; }
@@ -143,13 +140,14 @@ public:
 	void SetTimestampPrepend(bool b) { m_bPrependTimestamp = b; }
 	void SetTimezoneOffset(float b) { m_fTimezoneOffset = b; }
 	void SetJoinTries(unsigned int i) { m_uMaxJoinTries = i; }
+	void SetMaxJoins(unsigned int i) { m_uMaxJoins = i; }
 	void SetIRCConnectEnabled(bool b) { m_bIRCConnectEnabled = b; }
 	// !Setters
 
 	// Getters
 	vector<CClient*>& GetClients() { return m_vClients; }
-	CIRCSock* GetIRCSock();
-	const CIRCSock* GetIRCSock() const;
+	CIRCSock* GetIRCSock() { return m_pIRCSock; }
+	const CIRCSock* GetIRCSock() const { return m_pIRCSock; }
 	const CString& GetUserName() const;
 	const CString& GetCleanUserName() const;
 	const CString& GetNick(bool bAllowDefault = true) const;
@@ -159,6 +157,7 @@ public:
 	const CString& GetVHost() const;
 	const CString& GetPass() const;
 	bool IsPassHashed() const;
+	const CString& GetPassSalt() const;
 	const set<CString>& GetAllowedHosts() const;
 	const CString& GetTimestampFormat() const;
 	bool GetTimestampAppend() const;
@@ -172,7 +171,6 @@ public:
 	const CString& GetDLPath() const { if (!CFile::Exists(m_sDLPath)) { CDir::MakeDir(m_sDLPath); } return m_sDLPath; }
 
 	bool UseClientIP() const;
-	bool GetKeepNick() const;
 	bool DenyLoadMod() const;
 	bool IsAdmin() const;
 	bool DenySetVHost() const;
@@ -188,13 +186,13 @@ public:
 	const MCString& GetCTCPReplies() const;
 	unsigned int GetBufferCount() const;
 	bool KeepBuffer() const;
-	bool AutoCycle() const;
 	bool IsBeingDeleted() const { return m_bBeingDeleted; }
 	bool HasServers() const { return m_vServers.size() > 0; }
 	float GetTimezoneOffset() const { return m_fTimezoneOffset; }
 	unsigned long long BytesRead() const { return m_uBytesRead; }
 	unsigned long long BytesWritten() const { return m_uBytesWritten; }
 	unsigned int JoinTries() const { return m_uMaxJoinTries; }
+	unsigned int MaxJoins() const { return m_uMaxJoins; }
 	// !Getters
 private:
 protected:
@@ -207,6 +205,7 @@ protected:
 	CString			m_sRealName;
 	CString			m_sVHost;
 	CString			m_sPass;
+	CString			m_sPassSalt;
 	CString			m_sStatusPrefix;
 	CString			m_sDefaultChanModes;
 	CString			m_sChanPrefixes;
@@ -225,24 +224,20 @@ protected:
 	CBuffer				m_RawBuffer;
 	CBuffer				m_MotdBuffer;
 	CBuffer				m_QueryBuffer;
-	bool				m_bIRCConnected;
 	bool				m_bMultiClients;
 	bool				m_bBounceDCCs;
 	bool				m_bPassHashed;
 	bool				m_bUseClientIP;
-	bool				m_bKeepNick;
 	bool				m_bDenyLoadMod;
 	bool				m_bAdmin;
 	bool				m_bDenySetVHost;
 	bool				m_bKeepBuffer;
-	bool				m_bAutoCycle;
 	bool				m_bBeingDeleted;
 	bool				m_bAppendTimestamp;
 	bool				m_bPrependTimestamp;
 	bool				m_bIRCConnectEnabled;
 	CIRCSock*			m_pIRCSock;
 
-	CKeepNickTimer*		m_pKeepNickTimer;
 	CJoinTimer*			m_pJoinTimer;
 	CMiscTimer*			m_pMiscTimer;
 
@@ -255,6 +250,7 @@ protected:
 	unsigned long long      m_uBytesRead;
 	unsigned long long      m_uBytesWritten;
 	unsigned int		m_uMaxJoinTries;
+	unsigned int		m_uMaxJoins;
 
 #ifdef _MODULES
 	CModules*		m_pModules;
