@@ -8,6 +8,7 @@
 
 #include "Utils.h"
 #include "MD5.h"
+#include "main.h"
 #include <errno.h>
 #ifdef HAVE_LIBSSL
 #include <openssl/ssl.h>
@@ -17,6 +18,10 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+// Required with GCC 4.3+ if openssl is disabled
+#include <cstring>
+#include <cstdlib>
+
 using std::stringstream;
 
 bool CUtils::stdoutIsTTY;
@@ -189,6 +194,54 @@ CString CUtils::GetHashPass() {
 	return "";
 }
 
+CString CUtils::GetSaltedHashPass(CString& sSalt) {
+	sSalt = GetSalt();
+	unsigned int uiSaltLength = sSalt.length();
+	const char *pSalt = sSalt.c_str();
+
+	while (true) {
+		char* pass = CUtils::GetPass("Enter Password");
+		char* pass1 = (char *) malloc(strlen(pass) + 1);
+		// Make a copy of this since it is stored in a static buffer and will be overwritten when we fill pass2 below
+		strcpy(pass1, pass);
+		// null out our pass so it doesn't sit in memory
+		memset(pass, 0, strlen(pass));
+		char* pass2 = CUtils::GetPass("Confirm Password");
+		int iLen = strlen(pass1);
+
+		if (strcmp(pass1, pass2) != 0) {
+			CUtils::PrintError("The supplied passwords did not match");
+		} else if (!iLen) {
+			CUtils::PrintError("You can not use an empty password");
+		} else {
+			// Construct the salted pass
+			char *salted_pass = (char *) malloc(iLen + uiSaltLength + 1);
+			strcpy(salted_pass, pass1);
+			strcpy(salted_pass + iLen, pSalt);
+
+			CString sRet((const char*) CMD5(salted_pass, iLen + uiSaltLength));
+			// null out our pass so it doesn't sit in memory
+			memset(salted_pass, 0, iLen + uiSaltLength);
+			memset(pass1, 0, iLen);
+			memset(pass2, 0, strlen(pass2));
+			free(salted_pass);
+			free(pass1);
+
+			return sRet;
+		}
+
+		memset((char*) pass1, 0, iLen);	// null out our pass so it doesn't sit in memory
+		memset((char*) pass2, 0, strlen(pass2));	// null out our pass so it doesn't sit in memory
+		free(pass1);
+	}
+
+	return "";
+}
+
+CString CUtils::GetSalt() {
+	return CString::RandomString(20);
+}
+
 char* CUtils::GetPass(const CString& sPrompt) {
 	PrintPrompt(sPrompt);
 	return getpass("");
@@ -205,15 +258,15 @@ bool CUtils::GetBoolInput(const CString& sPrompt, bool *pbDefault) {
 		sDefault = (*pbDefault) ? "yes" : "no";
 	}
 
-	GetInput(sPrompt, sRet, sDefault, "yes/no");
+	while (true) {
+		GetInput(sPrompt, sRet, sDefault, "yes/no");
 
-	if (sRet.CaseCmp("yes") == 0) {
-		return true;
-	} else if (sRet.CaseCmp("no") == 0) {
-		return false;
+		if (sRet.Equals("y") || sRet.Equals("yes")) {
+			return true;
+		} else if (sRet.Equals("n") || sRet.Equals("no")) {
+			return false;
+		}
 	}
-
-	return GetBoolInput(sPrompt, pbDefault);
 }
 
 bool CUtils::GetNumInput(const CString& sPrompt, unsigned int& uRet, unsigned int uMin, unsigned int uMax, unsigned int uDefault) {
@@ -348,28 +401,22 @@ void CUtils::PrintStatus(bool bSuccess, const CString& sMessage) {
 	fflush(stdout);
 }
 
-CTable::CTable() {}
-CTable::~CTable() {
-	for (unsigned int a = 0; a < size(); a++) {
-		delete (*this)[a];
-	}
-
-	clear();
-}
-
 bool CTable::AddColumn(const CString& sName) {
 	for (unsigned int a = 0; a < m_vsHeaders.size(); a++) {
-		if (m_vsHeaders[a].CaseCmp(sName) == 0) {
+		if (m_vsHeaders[a].Equals(sName)) {
 			return false;
 		}
 	}
 
 	m_vsHeaders.push_back(sName);
+	m_msuWidths[sName] = sName.size();
+
 	return true;
 }
 
 unsigned int CTable::AddRow() {
-	push_back(new map<CString, CString>);
+	// Add a vector with enough space for each column
+	push_back(vector<CString>(m_vsHeaders.size()));
 	return size() -1;
 }
 
@@ -382,11 +429,15 @@ bool CTable::SetCell(const CString& sColumn, const CString& sValue, unsigned int
 		uRowIdx = size() -1;
 	}
 
-	(*(*this)[uRowIdx])[sColumn] = sValue;
+	(*this)[uRowIdx][GetColumnIndex(sColumn)] = sValue;
+
+	if (m_msuWidths[sColumn] < sValue.size())
+		m_msuWidths[sColumn] = sValue.size();
+
 	return true;
 }
 
-bool CTable::GetLine(unsigned int uIdx, CString& sLine) {
+bool CTable::GetLine(unsigned int uIdx, CString& sLine) const {
 	stringstream ssRet;
 
 	if (!size()) {
@@ -394,7 +445,6 @@ bool CTable::GetLine(unsigned int uIdx, CString& sLine) {
 	}
 
 	if (uIdx == 1) {
-		m_msuWidths.clear();	// Clear out the width cache
 		ssRet.fill(' ');
 		ssRet << "| ";
 
@@ -422,13 +472,13 @@ bool CTable::GetLine(unsigned int uIdx, CString& sLine) {
 		uIdx -= 3;
 
 		if (uIdx < size()) {
-			map<CString, CString>* pRow = (*this)[uIdx];
+			const vector<CString>& mRow = (*this)[uIdx];
 			ssRet.fill(' ');
 			ssRet << "| ";
 
 			for (unsigned int c = 0; c < m_vsHeaders.size(); c++) {
 				ssRet.width(GetColumnWidth(c));
-				ssRet << std::left << (*pRow)[m_vsHeaders[c]];
+				ssRet << std::left << mRow[c];
 				ssRet << ((c == m_vsHeaders.size() -1) ? " |" : " | ");
 			}
 
@@ -467,31 +517,37 @@ bool CTable::Output(std::ostream oOut) {
 }
 */
 
-unsigned int CTable::GetColumnWidth(unsigned int uIdx) {
+unsigned int CTable::GetColumnIndex(const CString& sName) const {
+	for (unsigned int i = 0; i < m_vsHeaders.size(); i++) {
+		if (m_vsHeaders[i] == sName)
+			return i;
+	}
+
+	DEBUG_ONLY(cout << "CTable::GetColumnIndex(" + sName + ") failed" << endl);
+
+	return 0;
+}
+
+unsigned int CTable::GetColumnWidth(unsigned int uIdx) const {
 	if (uIdx >= m_vsHeaders.size()) {
 		return 0;
 	}
 
 	const CString& sColName = m_vsHeaders[uIdx];
-	unsigned int uRet = sColName.size();
-	map<CString, unsigned int>::iterator it = m_msuWidths.find(sColName);
+	map<CString, unsigned int>::const_iterator it = m_msuWidths.find(sColName);
 
-	if (it != m_msuWidths.end()) {
-		return it->second;
+	if (it == m_msuWidths.end()) {
+		// AddColumn() and SetCell() should make sure that we get a value :/
+		return 0;
 	}
-
-	for (unsigned int a = 0; a < size(); a++) {
-		map<CString, CString>* pRow = (*this)[a];
-		unsigned int uTmp = (*pRow)[m_vsHeaders[uIdx]].size();
-
-		if (uTmp > uRet) {
-			uRet = uTmp;
-		}
-	}
-
-	return uRet;
+	return it->second;
 }
 
+void CTable::Clear() {
+	clear();
+	m_vsHeaders.clear();
+	m_msuWidths.clear();
+}
 
 #ifdef HAVE_LIBSSL
 CBlowfish::CBlowfish(const CString & sPassword, int iEncrypt, const CString & sIvec) {
