@@ -9,8 +9,9 @@
 #ifndef _UTILS_H
 #define _UTILS_H
 
-#include "String.h"
+#include "ZNCString.h"
 #include <assert.h>
+#include <cstdio>
 #include <map>
 #include <sys/file.h>
 #include <sys/time.h>
@@ -52,6 +53,8 @@ public:
 	static void PrintAction(const CString& sMessage);
 	static void PrintStatus(bool bSuccess, const CString& sMessage = "");
 	static CString GetHashPass();
+	static CString GetSaltedHashPass(CString& sSalt);
+	static CString GetSalt();
 	static char* GetPass(const CString& sPrompt);
 	static bool GetInput(const CString& sPrompt, CString& sRet, const CString& sDefault = "", const CString& sHint = "");
 	static bool GetBoolInput(const CString& sPrompt, bool bDefault);
@@ -182,7 +185,7 @@ class CException {
 public:
 	typedef enum {
 		EX_Shutdown,
-		EX_BadModVersion
+		EX_Restart
 	} EType;
 
 	CException(EType e) {
@@ -197,18 +200,23 @@ protected:
 };
 
 
-class CTable : public vector<map<CString, CString>* > {
+class CTable : protected vector<vector<CString> > {
 public:
-	CTable();
-	virtual ~CTable();
+	CTable() {}
+	virtual ~CTable() {}
 
 	bool AddColumn(const CString& sName);
 	unsigned int AddRow();
 	bool SetCell(const CString& sColumn, const CString& sValue, unsigned int uRowIdx = ~0);
-	bool GetLine(unsigned int uIdx, CString& sLine);
+	bool GetLine(unsigned int uIdx, CString& sLine) const;
 
-	unsigned int GetColumnWidth(unsigned int uIdx);
+	unsigned int GetColumnWidth(unsigned int uIdx) const;
+
+	void Clear();
+	using vector<vector<CString> >::size;
 private:
+	unsigned int GetColumnIndex(const CString& sName) const;
+
 protected:
 	vector<CString>				m_vsHeaders;
 	map<CString, unsigned int>	m_msuWidths;	// Used to cache the width of a column
