@@ -55,16 +55,13 @@ static void die(int sig) {
 }
 
 static void rehash(int sig) {
+	CUtils::PrintMessage("Caught SIGHUP");
 	CZNC::Get().SetNeedRehash(true);
 }
 
 static bool isRoot() {
-	uid_t u_real, u_effective, u_saved;
-
-	getresuid(&u_real, &u_effective, &u_saved);
-
 	// User root? If one of these were root, we could switch the others to root, too
-	if (u_real == 0 || u_effective == 0 || u_saved == 0)
+	if (geteuid() == 0 || getuid() == 0)
 		return true;
 
 	return false;
@@ -93,7 +90,7 @@ int main(int argc, char** argv) {
 #else
 	while ((iArg = getopt_long(argc, argv, "hvnrcsd:", g_LongOpts, &iOptIndex)) != -1) {
 #endif /* HAVE_LIBSSL */
-	    switch (iArg) {
+		switch (iArg) {
 		case 'h':
 			GenerateHelp(argv[0]);
 			return 0;
@@ -127,7 +124,7 @@ int main(int argc, char** argv) {
 		default:
 			GenerateHelp(argv[0]);
 			return 1;
-	    }
+		}
 	}
 
 	if (optind < argc) {
@@ -136,72 +133,41 @@ int main(int argc, char** argv) {
 		sConfig = "znc.conf";
 	}
 
-	if (bMakeConf) {
-		CZNC& ZNC = CZNC::Get();
-		ZNC.InitDirs("", sDataDir);
-		if (ZNC.WriteNewConfig(sConfig)) {
-			char const* args[5];
-
-			if (argc > 2) {
-				args[0] = argv[0];
-				if (!sDataDir.empty()) {
-					args[1] = "--datadir";
-					args[2] = strdup(sDataDir.c_str());
-					args[3] = argv[optind];
-					args[4] = NULL;
-				} else {
-					args[1] = argv[optind];
-					args[2] = NULL;
-				}
-			} else if (argc > 1) {
-				args[0] = argv[0];
-				if (!sDataDir.empty()) {
-					args[1] = "--datadir";
-					args[2] = strdup(sDataDir.c_str());
-					args[3] = NULL;
-				} else {
-					args[1] = NULL;
-				}
-			} else {
-				CUtils::PrintError("Unable to launch znc [Try manually restarting]");
-				return 1;
-			}
-
-			if ((chdir(ZNC.GetCurPath().c_str()) == -1)
-					|| (execv(*argv, (char *const*)args) == -1)) {
-				CUtils::PrintError("Unable to launch znc [" + CString(strerror(errno)) + "]");
-				return 1;
-			}
-		}
-
-		return 0;
-	}
+	CZNC* pZNC = &CZNC::Get();
+	pZNC->InitDirs(((argc) ? argv[0] : ""), sDataDir);
 
 #ifdef HAVE_LIBSSL
 	if (bMakePem) {
-		CZNC* pZNC = &CZNC::Get();
-		pZNC->InitDirs("", sDataDir);
 		pZNC->WritePemFile(bEncPem);
 
 		delete pZNC;
 		return 0;
 	}
+
 	if (bEncPem && !bMakePem) {
 		CUtils::PrintError("--encrypt-pem should be used along with --makepem.");
+		delete pZNC;
 		return 1;
 	}
-
 #endif /* HAVE_LIBSSL */
-	if (bMakePass) {
-		CString sHash = CUtils::GetHashPass();
-		CUtils::PrintMessage("Use this in the <User> section of your config:");
-		CUtils::PrintMessage("Pass = " + sHash + " -");
 
+	if (bMakePass) {
+		CString sSalt;
+		CString sHash = CUtils::GetSaltedHashPass(sSalt);
+		CUtils::PrintMessage("Use this in the <User> section of your config:");
+		CUtils::PrintMessage("Pass = md5#" + sHash + "#" + sSalt + "#");
+
+		delete pZNC;
 		return 0;
 	}
 
-	CZNC* pZNC = &CZNC::Get();
-	pZNC->InitDirs(((argc) ? argv[0] : ""), sDataDir);
+	if (bMakeConf) {
+		if (!pZNC->WriteNewConfig(sConfig)) {
+			delete pZNC;
+			return 0;
+		}
+		/* Fall through to normal bootup */
+	}
 
 	if (!pZNC->ParseConfig(sConfig)) {
 		CUtils::PrintError("Unrecoverable config error.");
@@ -219,7 +185,8 @@ int main(int argc, char** argv) {
 		CUtils::PrintError("You are running ZNC as root! Don't do that! There are not many valid");
 		CUtils::PrintError("reasons for this and it can, in theory, cause great damage!");
 		if (!bAllowRoot) {
-			exit(1);
+			delete pZNC;
+			return 1;
 		}
 		CUtils::PrintError("You have been warned.");
 		CUtils::PrintError("Hit CTRL+C now if you don't want to run ZNC as root.");
@@ -241,7 +208,7 @@ int main(int argc, char** argv) {
 	if (iPid == -1) {
 		CUtils::PrintStatus(false, strerror(errno));
 		delete pZNC;
-		exit(1);
+		return 1;
 	}
 
 	if (iPid > 0) {
@@ -250,7 +217,8 @@ int main(int argc, char** argv) {
 
 		pZNC->WritePidFile(iPid);
 		CUtils::PrintMessage(CZNC::GetTag());
-		exit(0);
+		/* Don't destroy pZNC here or it will delete the pid file. */
+		return 0;
 	}
 
 	// Redirect std in/out/err to /dev/null
@@ -293,10 +261,22 @@ int main(int argc, char** argv) {
 	try {
 		iRet = pZNC->Loop();
 	} catch (CException e) {
-		// EX_Shutdown is thrown to exit
 		switch (e.GetType()) {
 			case CException::EX_Shutdown:
 				iRet = 0;
+				break;
+			case CException::EX_Restart: {
+				// strdup() because GCC is stupid
+				char *args[] = {
+					strdup(argv[0]),
+					strdup("--datadir"),
+					strdup(pZNC->GetZNCPath().c_str()),
+					strdup(pZNC->GetConfigFile().c_str()),
+					NULL
+				};
+				execvp(args[0], args);
+				CUtils::PrintError("Unable to restart znc [" + CString(strerror(errno)) + "]");
+			} /* Fall through */
 			default:
 				iRet = 1;
 		}
