@@ -10,7 +10,7 @@
 #define _MAIN_H
 
 // Keep the number in sync with configure.in (and also with configure)
-#define VERSION 0.058
+#define VERSION 0.062
 
 // You can add -DVERSION_EXTRA="stuff" to your CXXFLAGS!
 #ifndef VERSION_EXTRA
@@ -68,6 +68,6 @@ using std::endl;
 #warning PATH_MAX is undefined
 #endif
 
-#include "String.h"
+#include "ZNCString.h"
 
 #endif // !_MAIN_H
