@@ -58,7 +58,7 @@ public:
 	virtual void OnModCommand(const CString& sLine) {
 		CString sCommand = sLine.Token(0);
 
-		if (sCommand.CaseCmp("ADD") == 0) {
+		if (sCommand.Equals("ADD")) {
 			CString sChan = sLine.Token(1);
 
 			if (AlreadyAdded(sChan)) {
@@ -68,14 +68,14 @@ public:
 			} else {
 				PutModule("Usage: Add [!]<#chan>");
 			}
-		} else if (sCommand.CaseCmp("DEL") == 0) {
+		} else if (sCommand.Equals("DEL")) {
 			CString sChan = sLine.Token(1);
 
 			if (Del(sChan))
 				PutModule("Removed " + sChan + " from list");
 			else
 				PutModule("Usage: Del [!]<#chan>");
-		} else if (sCommand.CaseCmp("LIST") == 0) {
+		} else if (sCommand.Equals("LIST")) {
 			CTable Table;
 			Table.AddColumn("Chan");
 
@@ -90,16 +90,11 @@ public:
 			}
 
 			if (Table.size()) {
-				unsigned int uTableIdx = 0;
-				CString sTmp;
-
-				while (Table.GetLine(uTableIdx++, sTmp)) {
-					PutModule(sTmp);
-				}
+				PutModule(Table);
 			} else {
 				PutModule("You have no entries.");
 			}
-		} else if (sCommand.CaseCmp("HELP") == 0) {
+		} else if (sCommand.Equals("HELP")) {
 			CTable Table;
 			Table.AddColumn("Command");
 			Table.AddColumn("Description");
@@ -117,12 +112,7 @@ public:
 			Table.SetCell("Description", "List all entries");
 
 			if (Table.size()) {
-				unsigned int uTableIdx = 0;
-				CString sTmp;
-
-				while (Table.GetLine(uTableIdx++, sTmp)) {
-					PutModule(sTmp);
-				}
+				PutModule(Table);
 			} else {
 				PutModule("You have no entries.");
 			}
