@@ -62,14 +62,14 @@ public:
 		return true;
 	}
 
-	virtual void OnUserAttached()
+	virtual void OnClientLogin()
 	{
 		stringstream s;
 		s << "You have " << m_ssUidls.size() << " emails.";
 		PutModule(s.str());
 		StartTimer();
 	}
-	virtual void OnUserDetached()
+	virtual void OnClientDisconnect()
 	{
 		RemTimer("EMAIL::" + m_pUser->GetUserName());
 	}
@@ -122,11 +122,7 @@ public:
 			m_ssUidls = ssUidls;	// keep the list in synch
 
 			if (Table.size()) {
-				unsigned int uTableIdx = 0;
-				CString sLine;
-				while (Table.GetLine(uTableIdx++, sLine)) {
-					PutModule(sLine);
-				}
+				PutModule(Table);
 
 				stringstream s;
 				s << "You have " << vEmails.size() << " emails.";
@@ -186,9 +182,9 @@ public:
 			if (sLine.empty())
 				break;	// out of the headers
 
-			if (strncasecmp(sLine.c_str(), "From: ", 6) == 0)
+			if (sLine.Equals("From: ", false, 6))
 				tmp.sFrom = sLine.substr(6, CString::npos);
-			else if (strncasecmp(sLine.c_str(), "Subject: ", 9) == 0)
+			else if (sLine.Equals("Subject: ", false, 9))
 				tmp.sSubject = sLine.substr(9, CString::npos);
 
 			if ((!tmp.sFrom.empty()) && (!tmp.sSubject.empty()))
