@@ -15,7 +15,6 @@
  * PERL_DESTRUCT_LEVEL is a perl env flag to force the VM to cleanup, this also stops false alarms, but within perl
  */
 
-#ifdef HAVE_PERL
 #include "Chan.h"
 #include "User.h"
 #include "znc.h"
@@ -235,31 +234,33 @@ public:
 	{
 		perl_destruct(m_pPerl);
 		perl_free(m_pPerl);
+		PERL_SYS_TERM();
 		m_pPerl = NULL;
 	}
 
-	void SetupZNCScript()
+	bool SetupZNCScript()
 	{
 		CString sModule, sTmp;
 
-		if (CZNC::Get().FindModPath("modperl.pm", sModule, sTmp))
-		{
-			CString sBuffer, sScript;
-			CFile cFile(sModule);
-			if ((cFile.Exists()) && (cFile.Open(O_RDONLY)))
-			{
-				while (cFile.ReadLine(sBuffer))
-					sScript += sBuffer;
-				cFile.Close();
+		if (!CZNC::Get().FindModPath("modperl.pm", sModule, sTmp))
+			return false;
 
-				eval_pv(sScript.c_str(), FALSE);
-			}
-		}
+		CString sBuffer, sScript;
+		CFile cFile(sModule);
+		if (!cFile.Exists() || !cFile.Open(O_RDONLY))
+			return false;
+
+		while (cFile.ReadLine(sBuffer))
+			sScript += sBuffer;
+		cFile.Close();
+
+		eval_pv(sScript.c_str(), FALSE);
+		return true;
 	}
 
 	virtual EModRet OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan)
 	{
-		if ((sName.CaseCmp("loadperlmodule") == 0) && (pUser))
+		if (sName.Equals("loadperlmodule") && pUser)
 		{
 			m_pUser = pUser;
 			if (sValue.Right(3) == ".pm")
@@ -301,8 +302,8 @@ public:
 	void UnSetUser() { m_pUser = NULL; }
 
 	virtual bool OnLoad(const CString & sArgs, CString & sMessage);
-	virtual void OnUserAttached() {  CBNone("OnUserAttached"); }
-	virtual void OnUserDetached() {  CBNone("OnUserDetached"); }
+	virtual void OnClientLogin() { CBNone("OnClientLogin"); }
+	virtual void OnClientDisconnect() { CBNone("OnClientDisconnect"); }
 	virtual void OnIRCDisconnected() {  CBNone("OnIRCDisconnected"); }
 	virtual void OnIRCConnected() {  CBNone("OnIRCConnected"); }
 
@@ -332,6 +333,11 @@ public:
 	virtual EModRet OnUserRaw(CString& sLine) { return(CBSingle("OnUserRaw", sLine)); }
 	virtual EModRet OnRaw(CString& sLine) { return(CBSingle("OnRaw", sLine)); }
 
+	virtual EModRet OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName) {
+		return(CBFour("OnIRCRegistration", sPass, sNick, sIdent, sRealName));
+	}
+	virtual EModRet OnTimerAutoJoin(CChan& Channel) { return(CBSingle("OnTimerAutoJoin", CHAN(Channel))); }
+
 	virtual void OnModCommand(const CString& sCommand)
 	{
 		if (CBSingle("OnModCommand", sCommand) == 0)
@@ -470,18 +476,18 @@ public:
 
 	bool Eval(const CString & sScript, const CString & sFuncName = ZNCEvalCB);
 
-	virtual EModRet OnStatusCommand(const CString& sLine)
+	virtual EModRet OnStatusCommand(CString& sLine)
 	{
 		CString sCommand = sLine.Token(0);
 
-		if (sCommand.CaseCmp("loadperlmod", 11) == 0 || sCommand.CaseCmp("unloadperlmod", 13) == 0 || sCommand.CaseCmp("reloadperlmod", 13) == 0)
+		if (sCommand.Equals("loadperlmod", false, 11) || sCommand.Equals("unloadperlmod", false, 13) || sCommand.Equals("reloadperlmod", false, 13))
 		{
 			CString sModule = sLine.Token(1);
 			if (sModule.Right(3) != ".pm")
 				sModule += ".pm";
-			if (sCommand.CaseCmp("loadperlmod", 11) == 0)
+			if (sCommand.Equals("loadperlmod", false, 11))
 				LoadPerlMod(sModule);
-			else if (sCommand.CaseCmp("unloadperlmod", 13) == 0)
+			else if (sCommand.Equals("unloadperlmod", false, 13))
 				UnloadPerlMod(sModule);
 			else
 			{
@@ -982,13 +988,18 @@ EXTERN_C void boot_DynaLoader (pTHX_ CV* cv);
 
 bool CModPerl::OnLoad(const CString & sArgs, CString & sMessage)
 {
+	int iArgc = 5;
+	char *p[] = { "", "-e", "0", "-T", "-w", NULL };
+	char **pArgv = static_cast<char **>(p);
+	PERL_SYS_INIT3( &iArgc, &pArgv, &environ );
+
 	m_pPerl = perl_alloc();
 	perl_construct(m_pPerl);
-	const char *pArgv[] = { "", "-e", "0", "-T", "-w" };
 
-	if (perl_parse(m_pPerl, NULL, 2, (char **)pArgv, (char **)NULL) != 0)
+	if (perl_parse(m_pPerl, NULL, iArgc, (char **)pArgv, (char **)NULL) != 0)
 	{
 		perl_free(m_pPerl);
+		PERL_SYS_TERM();
 		m_pPerl = NULL;
 		return(false);
 	}
@@ -1016,7 +1027,10 @@ bool CModPerl::OnLoad(const CString & sArgs, CString & sMessage)
 	newXS("ZNC::SetSockValue", XS_ZNC_SetSockValue, "modperl");
 
 	// this sets up the eval CB that we call from here on out. this way we can grab the error produced
-	SetupZNCScript();
+	if (!SetupZNCScript()) {
+		sMessage = "Failed to load modperl.pm";
+		return false;
+	}
 
 	HV *pZNCSpace = get_hv("ZNC::", TRUE);
 
@@ -1215,13 +1229,3 @@ Csock *CPerlSock::GetSockObj(const CS_STRING & sHostname, u_short iPort)
 
 	return(p);
 }
-
-#endif /* HAVE_PERL */
-
-
-
-
-
-
-
-
