@@ -35,24 +35,17 @@ public:
 		if (sCmdName == "set") {
 			CString sPass = sCommand.Token(1, true);
 			m_sPass = sPass;
-			PutModule("Password set");
-		} else if (sCmdName == "show") {
-			if (m_sPass.empty())
-				PutModule("No password set");
-			else
-				PutModule("Current password: " + m_sPass);
-		} else if (sCmdName == "save") {
 			SetNV("Password", m_sPass);
-			PutModule("Saved!");
+			PutModule("Password set");
 		} else {
-			PutModule("Commands: set <password>, show, save");
+			PutModule("Commands: set <password>");
 		}
 	}
 
 	void HandleMessage(CNick& Nick, const CString& sMessage)
 	{
 		if (!m_sPass.empty()
-				&& Nick.GetNick().CaseCmp("NickServ") == 0
+				&& Nick.GetNick().Equals("NickServ")
 				&& sMessage.find("msg") != CString::npos
 				&& sMessage.AsUpper().find("IDENTIFY") != CString::npos
 				&& sMessage.find("help") == CString::npos) {
