@@ -61,7 +61,7 @@ public:
 			CUser* pUser = it->second;
 			if (pUser->GetIRCSock()) {
 				if (pUser->GetChanPrefixes().find("~") == CString::npos) {
-					pUser->PutUser(":" + pUser->GetIRCServer() + " 005 " + pUser->GetIRCNick().GetNick() + " CHANTYPES=" + pUser->GetChanPrefixes() + "~ :are supported by this server.");
+					pUser->PutUser(":" + GetIRCServer(pUser) + " 005 " + pUser->GetIRCNick().GetNick() + " CHANTYPES=" + pUser->GetChanPrefixes() + "~ :are supported by this server.");
 				}
 			}
 		}
@@ -122,24 +122,11 @@ public:
 	}
 
 	virtual EModRet OnDeleteUser(CUser& User) {
-		const CString& sNick = User.GetUserName();
-		CString sHost = User.GetVHost();
-
-		CUser* pTmp = m_pUser;
-		m_pUser = &User;
-
-		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {	// Loop through each chan
-			const CString& sChannel = (*it)->GetName();
-			const set<CString>& ssNicks = (*it)->GetNicks();
-
-			if (ssNicks.find(User.GetUserName()) != ssNicks.end()) {										// If the user is on this chan
-				User.PutUser(":*" + GetModName() + "!znc@rottenboy.com KICK " + sChannel + " " + sNick + " :User Deleted");
-				PutChan(ssNicks, ":*" + GetModName() + "!znc@rottenboy.com KICK " + sChannel + " ?" + sNick + " :User Deleted", false);
-			}
+		// Loop through each chan
+		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
+			RemoveUser(&User, *it, "KICK", true, "User deleted");
 		}
 
-		m_pUser = pTmp;
-
 		return CONTINUE;
 	}
 
@@ -161,66 +148,73 @@ public:
 		m_spInjectedPrefixes.erase(m_pUser);
 	}
 
-	virtual void OnUserAttached() {
+	virtual void OnClientLogin() {
 		if (m_spInjectedPrefixes.find(m_pUser) == m_spInjectedPrefixes.end()) {
-			m_pClient->PutClient(":" + m_pUser->GetIRCServer() + " 005 " + m_pUser->GetIRCNick().GetNick() + " CHANTYPES=" + m_pUser->GetChanPrefixes() + "~ :are supported by this server.");
+			m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 005 " + m_pUser->GetIRCNick().GetNick() + " CHANTYPES=" + m_pUser->GetChanPrefixes() + "~ :are supported by this server.");
 		}
 
 		// Make sure this user is in the default channels
 		for (set<CString>::iterator a = m_ssDefaultChans.begin(); a != m_ssDefaultChans.end(); a++) {
 			CPartylineChannel* pChannel = GetChannel(*a);
 			const CString& sNick = m_pUser->GetUserName();
+
+			if (pChannel->IsInChannel(sNick))
+				continue;
+
 			CString sHost = m_pUser->GetVHost();
 			const set<CString>& ssNicks = pChannel->GetNicks();
 
 			if (sHost.empty()) {
 				sHost = m_pUser->GetIRCNick().GetHost();
 			}
+			if (sHost.empty()) {
+				sHost = "znc.in";
+			}
 			PutChan(ssNicks, ":?" + sNick + "!" + m_pUser->GetIdent() + "@" + sHost + " JOIN " + *a, false);
 			pChannel->AddNick(sNick);
 		}
 
+		CString sNickMask = m_pClient->GetNickMask();
+
 		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
 			const set<CString>& ssNicks = (*it)->GetNicks();
 
 			if ((*it)->IsInChannel(m_pUser->GetUserName())) {
 
-				m_pClient->PutClient(":" + m_pUser->GetIRCNick().GetNickMask() + " JOIN " + (*it)->GetName());
+				m_pClient->PutClient(":" + sNickMask + " JOIN " + (*it)->GetName());
 
 				if (!(*it)->GetTopic().empty()) {
-					m_pClient->PutClient(":" + m_pUser->GetIRCServer() + " 332 " + m_pUser->GetIRCNick().GetNickMask() + " " + (*it)->GetName() + " :" + (*it)->GetTopic());
+					m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 332 " + m_pUser->GetIRCNick().GetNickMask() + " " + (*it)->GetName() + " :" + (*it)->GetTopic());
 				}
 
 				SendNickList(m_pUser, ssNicks, (*it)->GetName());
-				PutChan(ssNicks, ":*" + GetModName() + "!znc@rottenboy.com MODE " + (*it)->GetName() + " +" + CString(m_pUser->IsAdmin() ? "o" : "v") + " ?" + m_pUser->GetUserName(), true);
+				PutChan(ssNicks, ":*" + GetModName() + "!znc@znc.in MODE " + (*it)->GetName() + " +" + CString(m_pUser->IsAdmin() ? "o" : "v") + " ?" + m_pUser->GetUserName(), true);
 			}
 		}
 	}
 
-	virtual void OnUserDetached() {
+	virtual void OnClientDisconnect() {
 		if (!m_pUser->IsUserAttached() && !m_pUser->IsBeingDeleted()) {
 			for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
 				const set<CString>& ssNicks = (*it)->GetNicks();
 
 				if (ssNicks.find(m_pUser->GetUserName()) != ssNicks.end()) {
-					PutChan(ssNicks, ":*" + GetModName() + "!znc@rottenboy.com MODE " + (*it)->GetName() + " -ov ?" + m_pUser->GetUserName() + " ?" + m_pUser->GetUserName(), true);
+					PutChan(ssNicks, ":*" + GetModName() + "!znc@znc.in MODE " + (*it)->GetName() + " -ov ?" + m_pUser->GetUserName() + " ?" + m_pUser->GetUserName(), true);
 				}
 			}
 		}
 	}
 
 	virtual EModRet OnUserRaw(CString& sLine) {
-		if (sLine.Left(5).CaseCmp("WHO ~") == 0) {
+		if (sLine.Equals("WHO ~", false, 5)) {
 			return HALT;
-		} else if (sLine.Left(6).CaseCmp("MODE ~") == 0) {
+		} else if (sLine.Equals("MODE ~", false, 6)) {
 			return HALT;
-		} else if (sLine.Left(8).CaseCmp("TOPIC ~#") == 0) {
+		} else if (sLine.Equals("TOPIC ~#", false, 8)) {
 			CString sChannel = sLine.Token(1);
 			CString sTopic = sLine.Token(2, true);
 
-			if (sTopic.Left(1) == ":") {
-				sTopic.LeftChomp();
-			}
+			sTopic.TrimPrefix(":");
 
 			CPartylineChannel* pChannel = FindChannel(sChannel);
 
@@ -231,19 +225,19 @@ public:
 						PutChan(ssNicks, ":" + m_pUser->GetIRCNick().GetNickMask() + " TOPIC " + sChannel + " :" + sTopic);
 						pChannel->SetTopic(sTopic);
 					} else {
-						m_pUser->PutUser(":irc.znc.com 482 " +  m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :You're not channel operator");
+						m_pUser->PutUser(":irc.znc.in 482 " +  m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :You're not channel operator");
 					}
 				} else {
 					sTopic = pChannel->GetTopic();
 
 					if (sTopic.empty()) {
-						m_pUser->PutUser(":irc.znc.com 331 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :No topic is set.");
+						m_pUser->PutUser(":irc.znc.in 331 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :No topic is set.");
 					} else {
-						m_pUser->PutUser(":irc.znc.com 332 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :" + sTopic);
+						m_pUser->PutUser(":irc.znc.in 332 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :" + sTopic);
 					}
 				}
 			} else {
-			    m_pUser->PutUser(":irc.znc.com 442 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :You're not on that channel");
+				m_pUser->PutUser(":irc.znc.in 442 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :You're not on that channel");
 			}
 			return HALT;
 		}
@@ -257,7 +251,7 @@ public:
 		}
 
 		if (sChannel.Left(2) != "~#") {
-			m_pClient->PutClient(":" + m_pUser->GetIRCServer() + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :No such channel");
+			m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :No such channel");
 			return HALT;
 		}
 
@@ -268,11 +262,22 @@ public:
 		return HALT;
 	}
 
-	void PartUser(CUser* pUser, CPartylineChannel* pChannel, bool bForce = false) {
+	void PartUser(CUser* pUser, CPartylineChannel* pChannel, bool bForce = false,
+			const CString& sMessage = "") {
+		RemoveUser(pUser, pChannel, "PART", bForce, sMessage);
+	}
+
+	void RemoveUser(CUser* pUser, CPartylineChannel* pChannel, const CString& sCommand,
+			bool bForce = false, const CString& sMessage = "") {
 		if (!pChannel || !pChannel->IsInChannel(pUser->GetUserName())) {
 			return;
 		}
 
+		CString sCmd = " " + sCommand + " ";
+		CString sMsg = sMessage;
+		if (!sMsg.empty())
+			sMsg = " :" + sMsg;
+
 		if (!pChannel->IsFixedChan(pUser->GetUserName()) || bForce) {
 			pChannel->DelNick(pUser->GetUserName());
 			pChannel->DelFixedNick(pUser->GetUserName());
@@ -284,8 +289,10 @@ public:
 				sHost = pUser->GetIRCNick().GetHost();
 			}
 
-			pUser->PutUser(":" + pUser->GetIRCNick().GetNickMask() + " PART " + pChannel->GetName());
-			PutChan(ssNicks, ":?" + pUser->GetUserName() + "!" + pUser->GetIdent() + "@" + sHost + " PART " + pChannel->GetName(), false);
+			pUser->PutUser(":" + pUser->GetIRCNick().GetNickMask() + sCmd
+					+ pChannel->GetName() + sMsg);
+			PutChan(ssNicks, ":?" + pUser->GetUserName() + "!" + pUser->GetIdent() + "@" + sHost
+					+ sCmd + pChannel->GetName() + sMsg, false);
 
 			if (ssNicks.empty()) {
 				delete pChannel;
@@ -303,7 +310,7 @@ public:
 		}
 
 		if (sChannel.Left(2) != "~#") {
-			m_pClient->PutClient(":" + m_pUser->GetIRCServer() + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :Channels look like ~#znc");
+			m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :Channels look like ~#znc");
 			return HALT;
 		}
 
@@ -331,12 +338,12 @@ public:
 			PutChan(ssNicks, ":?" + sNick + "!" + pUser->GetIdent() + "@" + sHost + " JOIN " + pChannel->GetName(), false);
 
 			if (!pChannel->GetTopic().empty()) {
-				pUser->PutUser(":" + pUser->GetIRCServer() + " 332 " + pUser->GetIRCNick().GetNickMask() + " " + pChannel->GetName() + " :" + pChannel->GetTopic());
+				pUser->PutUser(":" + GetIRCServer(pUser) + " 332 " + pUser->GetIRCNick().GetNickMask() + " " + pChannel->GetName() + " :" + pChannel->GetTopic());
 			}
 			SendNickList(pUser, ssNicks, pChannel->GetName());
 
 			if (pUser->IsAdmin()) {
-				PutChan(ssNicks, ":*" + GetModName() + "!znc@rottenboy.com MODE " + pChannel->GetName() + " +o ?" + pUser->GetUserName(), (pUser == m_pUser) ? false : true, pUser);
+				PutChan(ssNicks, ":*" + GetModName() + "!znc@znc.in MODE " + pChannel->GetName() + " +o ?" + pUser->GetUserName(), (pUser == m_pUser) ? false : true, pUser);
 			}
 		}
 	}
@@ -360,7 +367,7 @@ public:
 
 		if (cPrefix == '~') {
 			if (FindChannel(sTarget) == NULL) {
-				m_pClient->PutClient(":" + m_pUser->GetIRCServer() + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sTarget + " :No such channel");
+				m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sTarget + " :No such channel");
 				return HALT;
 			}
 
@@ -372,7 +379,7 @@ public:
 			if (pUser) {
 				pUser->PutUser(":?" + m_pUser->GetUserName() + "!" + m_pUser->GetIdent() + "@" + sHost + " PRIVMSG " + pUser->GetIRCNick().GetNick() + " :" + sMessage);
 			} else {
-				m_pClient->PutClient(":" + m_pUser->GetIRCServer() + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sTarget + " :No such znc user: " + sNick + "");
+				m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sTarget + " :No such znc user: " + sNick + "");
 			}
 		}
 
@@ -382,7 +389,7 @@ public:
 	virtual void OnModCommand(const CString& sLine) {
 		CString sCommand = sLine.Token(0);
 
-		if (sCommand.CaseCmp("HELP") == 0) {
+		if (sCommand.Equals("HELP")) {
 			CTable Table;
 			Table.AddColumn("Command");
 			Table.AddColumn("Arguments");
@@ -418,13 +425,8 @@ public:
 			Table.SetCell("Arguments", "<channel>");
 			Table.SetCell("Description", "Show which users can not part this channel");
 
-			unsigned int uTableIdx = 0;
-			CString sTmp;
-
-			while (Table.GetLine(uTableIdx++, sTmp)) {
-				PutModule(sTmp);
-			}
-		} else if (sCommand.CaseCmp("LIST") == 0) {
+			PutModule(Table);
+		} else if (sCommand.Equals("LIST")) {
 			if (!m_ssChannels.size()) {
 				PutModule("There are no open channels.");
 				return;
@@ -442,13 +444,8 @@ public:
 				Table.SetCell("Users", CString((*a)->GetNicks().size()));
 			}
 
-			unsigned int uTableIdx = 0;
-			CString sTmp;
-
-			while (Table.GetLine(uTableIdx++, sTmp)) {
-				PutModule(sTmp);
-			}
-		} else if (sCommand.CaseCmp("ADDFIXCHAN") == 0) {
+			PutModule(Table);
+		} else if (sCommand.Equals("ADDFIXCHAN")) {
 			if (!m_pUser->IsAdmin()) {
 				PutModule("Access denied");
 				return;
@@ -475,7 +472,7 @@ public:
 			SaveFixedChans(pUser);
 
 			PutModule("Fixed " + sUser + " to channel " + sChan);
-		} else if (sCommand.CaseCmp("DELFIXCHAN") == 0) {
+		} else if (sCommand.Equals("DELFIXCHAN")) {
 			if (!m_pUser->IsAdmin()) {
 				PutModule("Access denied");
 				return;
@@ -500,7 +497,7 @@ public:
 			SaveFixedChans(pUser);
 
 			PutModule("Removed " + sUser + " from " + sChan);
-		} else if (sCommand.CaseCmp("LISTFIXCHANS") == 0) {
+		} else if (sCommand.Equals("LISTFIXCHANS")) {
 			if (!m_pUser->IsAdmin()) {
 				PutModule("Access denied");
 				return;
@@ -518,7 +515,7 @@ public:
 				}
 			}
 			PutModule("--- End of list");
-		} else if (sCommand.CaseCmp("LISTFIXUSERS") == 0) {
+		} else if (sCommand.Equals("LISTFIXUSERS")) {
 			if (!m_pUser->IsAdmin()) {
 				PutModule("Access denied");
 				return;
@@ -542,6 +539,14 @@ public:
 		}
 	}
 
+	const CString GetIRCServer(CUser *pUser) {
+		const CString& sServer = pUser->GetIRCServer();
+
+		if (!sServer.empty())
+			return sServer;
+		return "irc.znc.in";
+	}
+
 	bool PutChan(const CString& sChan, const CString& sLine, bool bIncludeCurUser = true, bool bIncludeClient = true, CUser* pUser = NULL, CClient* pClient = NULL) {
 		CPartylineChannel* pChannel = FindChannel(sChan);
 
@@ -574,8 +579,23 @@ public:
 		}
 	}
 
+	void PutUserIRCNick(CUser *pUser, const CString& sPre, const CString& sPost) {
+		const CString& sIRCNick = pUser->GetIRCNick().GetNick();
+		if (!sIRCNick.empty()) {
+			pUser->PutUser(sPre + sIRCNick + sPost);
+			return;
+		}
+
+		const vector<CClient*>& vClients = pUser->GetClients();
+		vector<CClient*>::const_iterator it;
+		for (it = vClients.begin(); it != vClients.end(); it++) {
+			(*it)->PutClient(sPre + (*it)->GetNick() + sPost);
+		}
+	}
+
 	void SendNickList(CUser* pUser, const set<CString>& ssNicks, const CString& sChan) {
 		CString sNickList;
+
 		for (set<CString>::iterator it = ssNicks.begin(); it != ssNicks.end(); it++) {
 			CUser* pChanUser = CZNC::Get().FindUser(*it);
 
@@ -586,16 +606,16 @@ public:
 			sNickList += "?" + (*it) + " ";
 
 			if (sNickList.size() >= 500) {
-				pUser->PutUser(":" + pUser->GetIRCServer() + " 353 " + pUser->GetIRCNick().GetNick() + " @ " + sChan + " :" + sNickList);
+				PutUserIRCNick(pUser, ":" + GetIRCServer(pUser) + " 353 ", " @ " + sChan + " :" + sNickList);
 				sNickList.clear();
 			}
 		}
 
 		if (sNickList.size()) {
-			pUser->PutUser(":" + pUser->GetIRCServer() + " 353 " + pUser->GetIRCNick().GetNick() + " @ " + sChan + " :" + sNickList);
+			PutUserIRCNick(pUser, ":" + GetIRCServer(pUser) + " 353 ", " @ " + sChan + " :" + sNickList);
 		}
 
-		pUser->PutUser(":" + pUser->GetIRCServer() + " 366 " + pUser->GetIRCNick().GetNick() + " " + sChan + " :End of /NAMES list.");
+		PutUserIRCNick(pUser, ":" + GetIRCServer(pUser) + " 366 ", " " + sChan + " :End of /NAMES list.");
 	}
 
 	CPartylineChannel* FindChannel(const CString& sChan) {
