@@ -52,6 +52,11 @@ public:
 		PutModule("You got disconnected BoyOh.");
 	}
 
+	virtual EModRet OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName) {
+		sRealName += " - ZNC";
+		return CONTINUE;
+	}
+
 	virtual EModRet OnBroadcast(CString& sMessage) {
 		PutModule("------ [" + sMessage + "]");
 		sMessage = "======== [" + sMessage + "] ========";
@@ -100,6 +105,11 @@ public:
 		PutModule("* Quits: " + Nick.GetNick() + " (" + Nick.GetIdent() + "!" + Nick.GetHost() + ") (" + sMessage + ")");
 	}
 
+	virtual EModRet OnTimerAutoJoin(CChan& Channel) {
+		PutModule("Attempting to join " + Channel.GetName());
+		return CONTINUE;
+	}
+
 	virtual void OnJoin(const CNick& Nick, CChan& Channel) {
 		PutModule("* Joins: " + Nick.GetNick() + " (" + Nick.GetIdent() + "!" + Nick.GetHost() + ")");
 	}
@@ -205,13 +215,13 @@ public:
 	}
 
 	virtual void OnModCommand(const CString& sCommand) {
-		if (strcasecmp(sCommand.c_str(), "TIMERS") == 0) {
+		if (sCommand.Equals("TIMERS")) {
 			ListTimers();
 		}
 	}
 
-	virtual EModRet OnStatusCommand(const CString& sCommand) {
-		if (strcasecmp(sCommand.c_str(), "SAMPLE") == 0) {
+	virtual EModRet OnStatusCommand(CString& sCommand) {
+		if (sCommand.Equals("SAMPLE")) {
 			PutModule("Hi, I'm your friendly sample module.");
 			return HALT;
 		}
