@@ -57,6 +57,8 @@ public:
 		EnableReadLine();
 	}
 
+	~CSChatSock();
+
 	virtual Csock *GetSockObj(const CS_STRING & sHostname, u_short iPort)
 	{
 		CSChatSock *p = new CSChatSock(sHostname, iPort);
@@ -133,7 +135,7 @@ public:
 		return true;
 	}
 
-	virtual void OnUserAttached()
+	virtual void OnClientLogin()
 	{
 		CString sName = "SCHAT::" + m_pUser->GetUserName();
 		for (u_int a = 0; a < m_pManager->size(); a++)
@@ -159,12 +161,12 @@ public:
 
 	virtual EModRet OnUserRaw(CString & sLine)
 	{
-		if (sLine.CaseCmp("schat ", 6) == 0)
+		if (sLine.Equals("schat ", false, 6))
 		{
 			OnModCommand("chat " + sLine.substr(6));
 			return(HALT);
 
-		} else if (sLine.CaseCmp("schat") == 0)
+		} else if (sLine.Equals("schat"))
 		{
 			PutModule("SChat User Area ...");
 			OnModCommand("help");
@@ -182,7 +184,7 @@ public:
 		sCom = sCommand.Token(0);
 		sArgs = sCommand.Token(1, true);
 
-		if ((sCom.CaseCmp("chat") == 0) && (!sArgs.empty())) {
+		if (sCom.Equals("chat") && !sArgs.empty()) {
 			CString sSockName = "SCHAT::" + m_pUser->GetUserName();
 			CString sNick = "(s)" + sArgs;
 			for (u_int a= 0; a < m_pManager->size(); a++)
@@ -191,7 +193,7 @@ public:
 					continue;
 
 				CSChatSock *pSock = (CSChatSock *)(*m_pManager)[a];
-				if (pSock->GetChatNick().CaseCmp(sNick.c_str()) == 0)
+				if (pSock->GetChatNick().Equals(sNick))
 				{
 					PutModule("Already Connected to [" + sArgs + "]");
 					return;
@@ -221,7 +223,7 @@ public:
 
 			PutIRC(s.str());
 
-		} else if (sCom.CaseCmp("list") == 0)
+		} else if (sCom.Equals("list"))
 		{
 			CString sName = "SCHAT::" + m_pUser->GetUserName();
 			CTable Table;
@@ -265,16 +267,12 @@ public:
 					Table.SetCell("Port", CString(pSock->GetLocalPort()));
 				}
 			}
-			if (Table.size())
-			{
-				unsigned int uTableIdx = 0;
-				CString sLine;
-				while (Table.GetLine(uTableIdx++, sLine))
-					PutModule(sLine);
+			if (Table.size()) {
+				PutModule(Table);
 			} else
 				PutModule("No SDCCs currently in session");
 
-		} else if (sCom.CaseCmp("close") == 0)
+		} else if (sCom.Equals("close"))
 		{
 			CString sName = "SCHAT::" + m_pUser->GetUserName();
 			for (u_int a = 0; a < m_pManager->size(); a++)
@@ -283,17 +281,17 @@ public:
 					continue;
 
 				CSChatSock *pSock = (CSChatSock *)(*m_pManager)[a];
-				if (sArgs.CaseCmp("(s)", 3) != 0)
+				if (!sArgs.Equals("(s)", false, 3))
 					sArgs = "(s)" + sArgs;
 
-				if (sArgs.CaseCmp(pSock->GetChatNick()) == 0)
+				if (sArgs.Equals(pSock->GetChatNick()))
 				{
 					pSock->Close();
 					return;
 				}
 			}
 			PutModule("No Such Chat [" + sArgs + "]");
-		} else if (sCom.CaseCmp("showsocks") == 0)
+		} else if (sCom.Equals("showsocks") && m_pUser->IsAdmin())
 		{
 			CTable Table;
 			Table.AddColumn("SockName");
@@ -342,15 +340,11 @@ public:
 				}
 			}
 			if (Table.size())
-			{
-				unsigned int uTableIdx = 0;
-				CString sLine;
-				while (Table.GetLine(uTableIdx++, sLine))
-					PutModule(sLine);
-			} else
+				PutModule(Table);
+			else
 				PutModule("Error Finding Sockets");
 
-		} else if (sCom.CaseCmp("help") == 0)
+		} else if (sCom.Equals("help"))
 		{
 			PutModule("Commands are: ");
 			PutModule("    help           - This text.");
@@ -358,8 +352,10 @@ public:
 			PutModule("    list           - List current chats.");
 			PutModule("    close <nick>   - Close a chat to a nick.");
 			PutModule("    timers         - Shows related timers.");
-			PutModule("    showsocks      - Shows all socket connections.");
-		} else if (sCom.CaseCmp("timers") == 0)
+			if (m_pUser->IsAdmin()) {
+				PutModule("    showsocks      - Shows all socket connections.");
+			}
+		} else if (sCom.Equals("timers"))
 			ListTimers();
 		else
 			PutModule("Unknown command [" + sCom + "] [" + sArgs + "]");
@@ -367,7 +363,7 @@ public:
 
 	virtual EModRet OnPrivCTCP(CNick& Nick, CString& sMessage)
 	{
-		if (sMessage.CaseCmp("DCC SCHAT ", 10) == 0)
+		if (sMessage.Equals("DCC SCHAT ", false, 10))
 		{
 			// chat ip port
 			unsigned long iIP = sMessage.Token(3).ToULong();
@@ -419,7 +415,7 @@ public:
 
 				if (it != m_siiWaitingChats.end())
 				{
-					if (sMessage.CaseCmp("yes") != 0)
+					if (!sMessage.Equals("yes"))
 						SendToUser(sTarget + "!" + sTarget + "@" +
 								CUtils::GetIP(it->second.first),
 								"Refusing to accept DCC SCHAT!");
@@ -447,7 +443,7 @@ public:
 
 	void SendToUser(const CString & sFrom, const CString & sText)
 	{
-		//:*schat!znc@znc.com PRIVMSG Jim :
+		//:*schat!znc@znc.in PRIVMSG Jim :
 		CString sSend = ":" + sFrom + " PRIVMSG " + m_pUser->GetCurNick() + " :" + sText;
 		PutUser(sSend);
 	}
@@ -465,6 +461,12 @@ private:
 
 //////////////////// methods ////////////////
 
+CSChatSock::~CSChatSock()
+{
+	m_pModule->GetUser()->AddBytesRead(GetBytesRead());
+	m_pModule->GetUser()->AddBytesWritten(GetBytesWritten());
+}
+
 void CSChatSock::ReadLine(const CS_STRING & sLine)
 {
 	if (m_pModule)
