@@ -18,6 +18,11 @@ public:
 		EnableReadLine();
 		m_pParent = pShellMod;
 		m_pClient = pClient;
+
+		// Get rid of that write fd, we aren't going to use it
+		// (And clients expecting input will fail this way).
+		close(GetWSock());
+		SetWSock(open("/dev/null", O_WRONLY));
 	}
 	// These next two function's bodies are at the bottom of the file since they reference CShellMod
 	virtual void ReadLine(const CString& sData);
@@ -55,9 +60,11 @@ public:
 		return true;
 	}
 
-	virtual void OnModCommand(const CString& sCommand) {
-		if ((strcasecmp(sCommand.c_str(), "cd") == 0) || (strncasecmp(sCommand.c_str(), "cd ", 3) == 0)) {
-			CString sPath = CDir::ChangeDir(m_sPath, ((sCommand.length() == 2) ? CString(CZNC::Get().GetHomePath()) : CString(sCommand.substr(3))), CZNC::Get().GetHomePath());
+	virtual void OnModCommand(const CString& sLine) {
+		CString sCommand = sLine.Token(0);
+		if (sCommand.Equals("cd")) {
+			CString sArg = sCommand.Token(1, true);
+			CString sPath = CDir::ChangeDir(m_sPath, (sArg.empty() ? CString(CZNC::Get().GetHomePath()) : sArg), CZNC::Get().GetHomePath());
 			CFile Dir(sPath);
 
 			if (Dir.IsDir()) {
@@ -69,9 +76,9 @@ public:
 			}
 
 			PutShell("znc$");
-		} else if (strcasecmp(sCommand.Token(0).c_str(), "SEND") == 0) {
-			CString sToNick = sCommand.Token(1);
-			CString sFile = sCommand.Token(2);
+		} else if (sCommand.Equals("SEND")) {
+			CString sToNick = sLine.Token(1);
+			CString sFile = sLine.Token(2);
 
 			if ((sToNick.empty()) || (sFile.empty())) {
 				PutShell("usage: Send <nick> <file>");
@@ -86,8 +93,8 @@ public:
 					m_pUser->SendFile(sToNick, sFile, GetModName());
 				}
 			}
-		} else if (strcasecmp(sCommand.Token(0).c_str(), "GET") == 0) {
-			CString sFile = sCommand.Token(1);
+		} else if (sCommand.Equals("GET")) {
+			CString sFile = sLine.Token(1);
 
 			if (sFile.empty()) {
 				PutShell("usage: Get <file>");
@@ -103,12 +110,12 @@ public:
 				}
 			}
 		} else {
-			RunCommand(sCommand);
+			RunCommand(sLine);
 		}
 	}
 
-	virtual EModRet OnStatusCommand(const CString& sCommand) {
-		if (strcasecmp(sCommand.c_str(), "SHELL") == 0) {
+	virtual EModRet OnStatusCommand(CString& sCommand) {
+		if (sCommand.Equals("SHELL")) {
 			PutShell("-- ZNC Shell Service --");
 			return HALT;
 		}
@@ -117,7 +124,7 @@ public:
 	}
 
 	virtual EModRet OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize) {
-		if (strcasecmp(RemoteNick.GetNick().c_str(), CString(GetModNick()).c_str()) == 0) {
+		if (RemoteNick.GetNick().Equals(GetModNick())) {
 			CString sLocalFile = CDir::ChangeDir(m_sPath, sFile, CZNC::Get().GetHomePath());
 
 			m_pUser->GetFile(m_pUser->GetCurNick(), CUtils::GetIP(uLongIP), uPort, sLocalFile, uFileSize, GetModName());
@@ -150,9 +157,7 @@ private:
 void CShellSock::ReadLine(const CString& sData) {
 	CString sLine = sData;
 
-	while (sLine.length() && (sLine[sLine.length() -1] == '\r' || sLine[sLine.length() -1] == '\n')) {
-		sLine = sLine.substr(0, sLine.length() -1);
-	}
+	sLine.TrimRight("\r\n");
 
 	CString::size_type a = sLine.find('\t');
 	while (a != CString::npos) {
