@@ -43,6 +43,7 @@ public:
 	virtual bool OnPageRequest(const CString& sURI, CString& sPageRet);
 	virtual bool OnLogin(const CString& sUser, const CString& sPass);
 
+	CString GetAvailSkinsDir();
 	CString GetSkinDir();
 	void PrintPage(CString& sPageRet, const CString& sTmplName);
 
@@ -96,9 +97,10 @@ public:
 	virtual Csock* GetSockObj(const CString& sHost, unsigned short uPort);
 	bool IsAdmin(bool bAllowUserAdmin = true) const { return m_bAdmin; }
 
+	CWebAdminMod* GetModule() const { return (CWebAdminMod*) m_pModule; }
+
 private:
 protected:
-	CWebAdminMod*			m_pModule;
 	CUser*					m_pUser;
 	CUser*					m_pSessionUser;
 	bool					m_bAdmin;
@@ -109,36 +111,28 @@ protected:
 class CWebAdminMod : public CGlobalModule {
 public:
 	GLOBALMODCONSTRUCTOR(CWebAdminMod) {
-		m_uPort = 8080;
 		m_sSkinName = GetNV("SkinName");
 	}
 
 	virtual ~CWebAdminMod() {
-		while (m_spSocks.size()) {							// Loop through the sockets that we have created
-			m_pManager->DelSockByAddr(*m_spSocks.begin());	// Delete each one which will call SockDestroyed() and erase it from the set
-		}													// This way we don't want to erase it ourselves, that's why we're using the funky while loop
-
-		m_spSocks.clear();
-	}
-
-	virtual bool OnBoot() {
-		return true;
 	}
 
 	virtual bool OnLoad(const CString& sArgStr, CString& sMessage) {
 		bool bSSL = false;
 		bool bIPv6 = false;
+		unsigned short uPort = 8080;
 		CString sArgs(sArgStr);
 		CString sOpt;
 		CString sPort;
+		CString sListenHost;
 
 		if (sArgs.Left(1) == "-") {
 			sOpt = sArgs.Token(0);
 			sArgs = sArgs.Token(1, true);
 
-			if (sOpt.CaseCmp("-IPV6") == 0) {
+			if (sOpt.Equals("-IPV6")) {
 				bIPv6 = true;
-			} else if (sOpt.CaseCmp("-IPV4") == 0) {
+			} else if (sOpt.Equals("-IPV4")) {
 				bIPv6 = false;
 			} else {
 				sMessage = "Unknown option [" + sOpt + "] valid options are -ipv4 or -ipv6";
@@ -147,7 +141,7 @@ public:
 		}
 
 		if (sArgs.find(" ") != CString::npos) {
-			m_sListenHost = sArgs.Token(0);
+			sListenHost = sArgs.Token(0);
 			sPort = sArgs.Token(1);
 		} else {
 			sPort = sArgs.Token(0);
@@ -163,7 +157,7 @@ public:
 		}
 
 		if (!sPort.empty()) {
-			m_uPort = sPort.ToUShort();
+			uPort = sPort.ToUShort();
 		}
 
 		CWebAdminSock* pListenSock = new CWebAdminSock(this);
@@ -173,21 +167,18 @@ public:
 		}
 #endif
 
-		bool b = m_pManager->ListenHost(m_uPort, "WebAdmin::Listener", m_sListenHost, bSSL, SOMAXCONN, pListenSock, 0, bIPv6);
+		errno = 0;
+		bool b = m_pManager->ListenHost(uPort, "WebAdmin::Listener", sListenHost, bSSL, SOMAXCONN, pListenSock, 0, bIPv6);
 		if (!b) {
-			sMessage = "Could not bind to port " + CString(m_uPort) + ": " + CString(strerror(errno));
+			sMessage = "Could not bind to port " + CString(uPort);
+			if (!sListenHost.empty())
+				sMessage += " on vhost [" + sListenHost + "]";
+			if (errno != 0)
+				sMessage += ": " + CString(strerror(errno));
 		}
 		return b;
 	}
 
-	void AddSock(CWebAdminSock* pSock) {
-		m_spSocks.insert(pSock);
-	}
-
-	void SockDestroyed(CWebAdminSock* pSock) {
-		m_spSocks.erase(pSock);
-	}
-
 	void SetSkinName(const CString& s) {
 		m_sSkinName = s;
 		SetNV("SkinName", m_sSkinName);
@@ -217,16 +208,16 @@ public:
 	}
 
 private:
-	unsigned short				m_uPort;
 	CString						m_sSkinName;
-	set<CWebAdminSock*>			m_spSocks;
-	CString						m_sListenHost;
 	map<CString, unsigned int>	m_suSwitchCounters;
 };
 
+CString CWebAdminSock::GetAvailSkinsDir() {
+	return m_pModule->GetModDataDir() + "/skins/";
+}
+
 CString CWebAdminSock::GetSkinDir() {
-	CString sSkinDir = m_pModule->GetModDataDir() + "/skins/"
-		+ m_pModule->GetSkinName() + "/";
+	CString sSkinDir = GetAvailSkinsDir() + GetModule()->GetSkinName() + "/";
 
 	if (CFile::IsDir(sSkinDir)) {
 		return sSkinDir;
@@ -302,39 +293,38 @@ void CWebAdminSock::ListUsersPage(CString& sPageRet) {
 }
 
 Csock* CWebAdminSock::GetSockObj(const CString& sHost, unsigned short uPort) {
-	CWebAdminSock* pSock = new CWebAdminSock(m_pModule, sHost, uPort);
+	CWebAdminSock* pSock = new CWebAdminSock(GetModule(), sHost, uPort);
 	pSock->SetSockName("WebAdmin::Client");
 	pSock->SetTimeout(120);
-	m_pModule->AddSock(pSock);
 
 	return pSock;
 }
 
-CWebAdminSock::CWebAdminSock(CWebAdminMod* pModule) : CHTTPSock() {
+CWebAdminSock::CWebAdminSock(CWebAdminMod* pModule) : CHTTPSock(pModule) {
 	m_pModule = pModule;
 	m_pUser = NULL;
 	m_pSessionUser = NULL;
 	m_bAdmin = false;
-	m_pModule->AddSock(this);
 	SetDocRoot(GetSkinDir());
 }
 
-CWebAdminSock::CWebAdminSock(CWebAdminMod* pModule, const CString& sHostname, unsigned short uPort, int iTimeout) : CHTTPSock(sHostname, uPort, iTimeout) {
+CWebAdminSock::CWebAdminSock(CWebAdminMod* pModule, const CString& sHostname, unsigned short uPort, int iTimeout)
+		: CHTTPSock(pModule, sHostname, uPort, iTimeout) {
 	m_pModule = pModule;
 	m_pUser = NULL;
 	m_pSessionUser = NULL;
 	m_bAdmin = false;
-	m_pModule->AddSock(this);
 	SetDocRoot(GetSkinDir());
 }
 
 CWebAdminSock::~CWebAdminSock() {
-	m_pModule->SockDestroyed(this);
-
 	if (!m_spAuth.IsNull()) {
 		CWebAdminAuth* pAuth = (CWebAdminAuth*) &(*m_spAuth);
 		pAuth->SetWebAdminSock(NULL);
 	}
+
+	CZNC::Get().AddBytesRead(GetBytesRead());
+	CZNC::Get().AddBytesWritten(GetBytesWritten());
 }
 
 bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
@@ -359,15 +349,15 @@ bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
 		m_Template["Title"] = "Main Page";
 		m_Template["Action"] = "home";
 		PrintPage(sPageRet, "Main.tmpl");
-	} else if (sURI.Left(5).CaseCmp("/css/") == 0) {
+	} else if (sURI.Equals("/css/", false, 5)) {
 		SetDocRoot(GetSkinDir() + "/css");
 		PrintFile(sURI.substr(5), "text/css");
 		return false;
-	} else if (sURI.Left(5).CaseCmp("/img/") == 0) {
+	} else if (sURI.Equals("/img/", false, 5)) {
 		SetDocRoot(GetSkinDir() + "/img");
 		PrintFile(sURI.substr(5));
 		return false;
-	} else if (sURI.Left(4).CaseCmp("/js/") == 0) {
+	} else if (sURI.Equals("/js/", false, 4)) {
 		SetDocRoot(GetSkinDir() + "/js");
 		PrintFile(sURI.substr(4), "application/x-javascript");
 		return false;
@@ -385,7 +375,7 @@ bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
 		}
 	} else if (sURI == "/switchuser") {
 		unsigned int uCurCnt = GetParam("cnt").ToUInt();
-		unsigned int uCounter = m_pModule->GetSwitchCounter(GetRemoteIP());
+		unsigned int uCounter = GetModule()->GetSwitchCounter(GetRemoteIP());
 
 		if (!uCurCnt) {
 			Redirect("/switchuser?cnt=" + CString(uCounter));
@@ -394,7 +384,7 @@ bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
 
 		if (uCurCnt >= uCounter) {
 			m_bLoggedIn = false;
-			m_pModule->IncSwitchCounter(GetRemoteIP());
+			GetModule()->IncSwitchCounter(GetRemoteIP());
 			ForceLogin();
 		} else {
 			Redirect("/home");
@@ -538,7 +528,7 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 #endif
 		}
 
-		CString sDir(GetSkinDir() + "/..");
+		CString sDir(GetAvailSkinsDir());
 
 		if (CFile::IsDir(sDir)) {
 			CDir Dir(sDir);
@@ -554,7 +544,7 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 					CTemplate& l = m_Template.AddRow("SkinLoop");
 					l["Name"] = SubDir.GetShortName();
 
-					if (SubDir.GetShortName() == m_pModule->GetSkinName()) {
+					if (SubDir.GetShortName() == GetModule()->GetSkinName()) {
 						l["Checked"] = "true";
 					}
 				}
@@ -607,7 +597,7 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 		CZNC::Get().AddVHost(vsArgs[a].Trim_n());
 	}
 
-	m_pModule->SetSkinName(GetParam("skin"));
+	GetModule()->SetSkinName(GetParam("skin"));
 
 	set<CString> ssArgs;
 	GetParamValues("loadmod", ssArgs);
@@ -619,16 +609,12 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 		if (!sModName.empty()) {
 			CString sArgs = GetParam("modargs_" + sModName);
 
-			try {
-				if (!CZNC::Get().GetModules().FindModule(sModName)) {
-					if (!CZNC::Get().GetModules().LoadModule(sModName, sArgs, NULL, sModRet)) {
-						DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] [" << sModRet << "]" << endl);
-					}
-				} else {
-					DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] because it is already loaded" << endl);
+			if (!CZNC::Get().GetModules().FindModule(sModName)) {
+				if (!CZNC::Get().GetModules().LoadModule(sModName, sArgs, NULL, sModRet)) {
+					DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] [" << sModRet << "]" << endl);
 				}
-			} catch(...) {
-				DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] [" << sArgs << "]" << endl);
+			} else {
+				DEBUG_ONLY(cerr << "Unable to load module [" << sModName << "] because it is already loaded" << endl);
 			}
 		}
 	}
@@ -685,10 +671,7 @@ bool CWebAdminSock::ChanPage(CString& sPageRet, CChan* pChan) {
 			m_Template["InConfig"] = "true";
 		}
 
-		CTemplate& o1 = m_Template.AddRow("OptionLoop");
-		o1["Name"] = "autocycle";
-		o1["DisplayName"] = "Auto Cycle";
-		if ((pChan && pChan->AutoCycle()) || (!pChan && m_pUser->AutoCycle())) { o1["Checked"] = "true"; }
+		/* o1 used to be AutoCycle which was removed */
 
 		CTemplate& o2 = m_Template.AddRow("OptionLoop");
 		o2["Name"] = "keepbuffer";
@@ -724,7 +707,6 @@ bool CWebAdminSock::ChanPage(CString& sPageRet, CChan* pChan) {
 	pChan->SetDefaultModes(GetParam("defmodes"));
 	pChan->SetBufferCount(GetParam("buffercount").ToUInt());
 	pChan->SetInConfig(GetParam("save").ToBool());
-	pChan->SetAutoCycle(GetParam("autocycle").ToBool());
 	pChan->SetKeepBuffer(GetParam("keepbuffer").ToBool());
 
 	bool bDetached = GetParam("detached").ToBool();
@@ -737,8 +719,6 @@ bool CWebAdminSock::ChanPage(CString& sPageRet, CChan* pChan) {
 		}
 	}
 
-	m_pUser->JoinChans();
-
 	if (!CZNC::Get().WriteConfig()) {
 		GetErrorPage(sPageRet, "Channel added/modified, but config was not written");
 		return true;
@@ -792,6 +772,8 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 			m_Template["BufferCount"] = CString(pUser->GetBufferCount());
 			m_Template["TimestampFormat"] = pUser->GetTimestampFormat();
 			m_Template["TimezoneOffset"] = CString(pUser->GetTimezoneOffset());
+			m_Template["JoinTries"] = CString(pUser->JoinTries());
+			m_Template["MaxJoins"] = CString(pUser->MaxJoins());
 
 			const set<CString>& ssAllowedHosts = pUser->GetAllowedHosts();
 			for (set<CString>::const_iterator it = ssAllowedHosts.begin(); it != ssAllowedHosts.end(); it++) {
@@ -892,15 +874,7 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 		o1["DisplayName"] = "Keep Buffer";
 		if (!pUser || pUser->KeepBuffer()) { o1["Checked"] = "true"; }
 
-		CTemplate& o2 = m_Template.AddRow("OptionLoop");
-		o2["Name"] = "autocycle";
-		o2["DisplayName"] = "Auto Cycle";
-		if (!pUser || pUser->AutoCycle()) { o2["Checked"] = "true"; }
-
-		CTemplate& o3 = m_Template.AddRow("OptionLoop");
-		o3["Name"] = "keepnick";
-		o3["DisplayName"] = "Keep Nick";
-		if (!pUser || pUser->GetKeepNick()) { o3["Checked"] = "true"; }
+		/* o2 used to be auto cycle which was removed */
 
 		CTemplate& o4 = m_Template.AddRow("OptionLoop");
 		o4["Name"] = "multiclients";
@@ -1027,7 +1001,9 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 	}
 
 	if (!sArg.empty()) {
-		pNewUser->SetPass(sArg.MD5(), true);
+		CString sSalt = CUtils::GetSalt();
+		CString sSaltedPass = sArg + sSalt;
+		pNewUser->SetPass(sSaltedPass.MD5(), true, sSalt);
 	}
 
 	VCString vsArgs;
@@ -1116,12 +1092,12 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 	pNewUser->SetKeepBuffer(GetParam("keepbuffer").ToBool());
 	pNewUser->SetMultiClients(GetParam("multiclients").ToBool());
 	pNewUser->SetBounceDCCs(GetParam("bouncedccs").ToBool());
-	pNewUser->SetAutoCycle(GetParam("autocycle").ToBool());
-	pNewUser->SetKeepNick(GetParam("keepnick").ToBool());
 	pNewUser->SetUseClientIP(GetParam("useclientip").ToBool());
 	pNewUser->SetTimestampAppend(GetParam("appendtimestamp").ToBool());
 	pNewUser->SetTimestampPrepend(GetParam("prependtimestamp").ToBool());
 	pNewUser->SetTimezoneOffset(GetParam("timezoneoffset").ToDouble());
+	pNewUser->SetJoinTries(GetParam("jointries").ToUInt());
+	pNewUser->SetMaxJoins(GetParam("maxjoins").ToUInt());
 
 	if (IsAdmin()) {
 		pNewUser->SetDenyLoadMod(GetParam("denyloadmod").ToBool());
