@@ -94,7 +94,7 @@ CString CZNC::GetTag(bool bIncludeVersion) {
 	return szBuf;
 }
 
-CString CZNC::GetUptime() {
+CString CZNC::GetUptime() const {
 	time_t now = time(NULL);
 	return CString::ToTimeStr(now - TimeStarted());
 }
@@ -203,7 +203,6 @@ bool CZNC::HandleUserDeletion()
 	}
 
 	m_msDelUsers.clear();
-	RestartConnectUser();
 
 	return true;
 }
@@ -211,8 +210,6 @@ bool CZNC::HandleUserDeletion()
 int CZNC::Loop() {
 	while (true) {
 		CString sError;
-		map<CString, CUser*>::iterator it;
-		map<CString, CUser*>::iterator end;
 
 		if (GetNeedRehash()) {
 			SetNeedRehash(false);
@@ -390,8 +387,8 @@ Csock* CZNC::FindSockByName(const CString& sSockName) {
 	return m_Manager.FindSockByName(sSockName);
 }
 
-bool CZNC::IsHostAllowed(const CString& sHostMask) {
-	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
+bool CZNC::IsHostAllowed(const CString& sHostMask) const {
+	for (map<CString,CUser*>::const_iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
 		if (a->second->IsHostAllowed(sHostMask)) {
 			return true;
 		}
@@ -434,9 +431,9 @@ void CZNC::InitDirs(const CString& sArgvPath, const CString& sDataDir) {
 	}
 
 	if (sDataDir.empty()) {
-	    m_sZNCPath = m_sHomePath + "/.znc";
+		m_sZNCPath = m_sHomePath + "/.znc";
 	} else {
-	    m_sZNCPath = sDataDir;
+		m_sZNCPath = sDataDir;
 	}
 
 	// Other dirs that we use
@@ -555,24 +552,12 @@ bool CZNC::WriteConfig() {
 	return true;
 }
 
-bool CZNC::WriteNewConfig(const CString& sConfig) {
-	CString sConfigFile = ExpandConfigPath((sConfig.empty()) ? "znc.conf" : sConfig);
+bool CZNC::WriteNewConfig(CString& sConfigFile) {
 	CString sAnswer, sUser;
-	vector<CString> vsLines;
+	VCString vsLines;
 
-	if (CFile::Exists(sConfigFile)) {
-		if (!m_LockFile.TryExLock(sConfigFile)) {
-			CUtils::PrintError("ZNC is currently running on this config.");
-			return false;
-		}
-
-		if (!CUtils::GetBoolInput("This config already exists.  Would you like to overwrite it?", false)) {
-			m_LockFile.UnLock();
-			return false;
-		}
-	}
-
-	CUtils::PrintMessage("Writing new config [" + sConfigFile + "]");
+	sConfigFile = ExpandConfigPath((sConfigFile.empty()) ? "znc.conf" : sConfigFile);
+	CUtils::PrintMessage("Building new config");
 
 	CUtils::PrintMessage("");
 	CUtils::PrintMessage("First lets start with some global settings...");
@@ -586,6 +571,15 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 #ifdef HAVE_LIBSSL
 	if (CUtils::GetBoolInput("Would you like ZNC to listen using SSL?", false)) {
 		sSSL = "+";
+
+		CString sPemFile = GetPemLocation();
+		if (!CFile::Exists(sPemFile)) {
+			CUtils::PrintError("Unable to locate pem file: [" + sPemFile + "]");
+			if (CUtils::GetBoolInput("Would you like to create a new pem file now?",
+						true)) {
+				WritePemFile();
+			}
+		}
 	}
 #endif
 
@@ -639,7 +633,7 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 				const CModInfo& Info = *it;
 				CString sName = Info.GetName();
 
-				if (sName.Right(3).CaseCmp(".so") == 0) {
+				if (sName.Right(3).Equals(".so")) {
 					sName.RightChomp(3);
 				}
 
@@ -666,8 +660,9 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 		} while (!CUser::IsValidUserName(sUser));
 
 		vsLines.push_back("<User " + sUser + ">");
-		sAnswer = CUtils::GetHashPass();
-		vsLines.push_back("\tPass       = " + sAnswer + " -");
+		CString sSalt;
+		sAnswer = CUtils::GetSaltedHashPass(sSalt);
+		vsLines.push_back("\tPass       = md5#" + sAnswer + "#" + sSalt + "#");
 
 		if (CUtils::GetBoolInput("Would you like this user to be an admin?", bFirstUser)) {
 			vsLines.push_back("\tAdmin      = true");
@@ -691,12 +686,6 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 		}
 		// todo: Possibly add motd
 
-		if (CUtils::GetBoolInput("Would you like ZNC to keep trying for your primary nick?", false)) {
-			vsLines.push_back("\tKeepNick   = true");
-		} else {
-			vsLines.push_back("\tKeepNick   = false");
-		}
-
 		unsigned int uBufferCount = 0;
 
 		CUtils::GetNumInput("Number of lines to buffer per channel", uBufferCount, 0, ~0, 50);
@@ -775,6 +764,8 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 #endif
 
 			vsLines.push_back("\tServer     = " + sHost + ((bSSL) ? " +" : " ") + CString(uServerPort) + " " + sPass);
+
+			CUtils::PrintMessage("");
 		} while (CUtils::GetBoolInput("Would you like to add another server?", false));
 
 		vsLines.push_back("");
@@ -802,32 +793,47 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 	} while (CUtils::GetBoolInput("Would you like to setup another user?", false));
 	// !User
 
-	CUtils::PrintAction("Writing config [" + sConfigFile + "]");
-	CFile File(sConfigFile);
+	CFile File;
+	bool bFileOK, bFileOpen = false;
+	do {
+		CUtils::PrintAction("Writing config [" + sConfigFile + "]");
 
-	bool bFileOpen = false;
+		bFileOK = true;
+		if (CFile::Exists(sConfigFile)) {
+			if (!m_LockFile.TryExLock(sConfigFile)) {
+				CUtils::PrintStatus(false, "ZNC is currently running on this config.");
+				bFileOK = false;
+			} else {
+				m_LockFile.Close();
+				CUtils::PrintStatus(false, "This config already exists.");
+				if (CUtils::GetBoolInput("Would you like to overwrite it?", false))
+					CUtils::PrintAction("Overwriting config [" + sConfigFile + "]");
+				else
+					bFileOK = false;
+			}
+		}
 
-	if (File.Open(O_WRONLY | O_CREAT | O_TRUNC, 0600)) {
-		bFileOpen = true;
-	} else {
-		CUtils::PrintStatus(false, "Unable to open file");
-		CUtils::GetInput("Alternate location", sConfigFile, "/tmp/" + sConfig);
-
-		if (!CFile::Exists(sConfigFile) || CUtils::GetBoolInput("Would you like to overwrite the existing alt file", false)) {
-			CUtils::PrintAction("Writing to alt location [" + sConfigFile + "]");
+		if (bFileOK) {
 			File.SetFileName(sConfigFile);
-
 			if (File.Open(O_WRONLY | O_CREAT | O_TRUNC, 0600)) {
 				bFileOpen = true;
 			} else {
-				CUtils::PrintStatus(false, "Unable to open alt file");
+				CUtils::PrintStatus(false, "Unable to open file");
+				bFileOK = false;
 			}
 		}
-	}
+		if (!bFileOK) {
+			CUtils::GetInput("Please specify an alternate location (or \"stdout\" for displaying the config)", sConfigFile, sConfigFile);
+			if (sConfigFile.Equals("stdout"))
+				bFileOK = true;
+			else
+				sConfigFile = ExpandConfigPath(sConfigFile);
+		}
+	} while (!bFileOK);
 
 	if (!bFileOpen) {
 		CUtils::PrintMessage("");
-		CUtils::PrintMessage("Printing new config to stdout since we were unable to open a file");
+		CUtils::PrintMessage("Printing the new config to stdout:");
 		CUtils::PrintMessage("");
 		cout << endl << "----------------------------------------------------------------------------" << endl << endl;
 	}
@@ -842,7 +848,6 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 
 	if (bFileOpen) {
 		File.Close();
-
 		CUtils::PrintStatus(true);
 	} else {
 		cout << endl << "----------------------------------------------------------------------------" << endl << endl;
@@ -851,14 +856,14 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 	CUtils::PrintMessage("");
 	CUtils::PrintMessage("To connect to this znc you need to connect to it as your irc server", true);
 	CUtils::PrintMessage("using the port that you supplied.  You have to supply your login info", true);
-	CUtils::PrintMessage("as the irc server password like so.. user:pass.", true);
+	CUtils::PrintMessage("as the irc server password like so... user:pass.", true);
 	CUtils::PrintMessage("");
 	CUtils::PrintMessage("Try something like this in your IRC client...", true);
 	CUtils::PrintMessage("/server <znc_server_ip> " + CString(uListenPort) + " " + sUser + ":<pass>", true);
 	CUtils::PrintMessage("");
 
 	m_LockFile.UnLock();
-	return CUtils::GetBoolInput("Launch znc now?", true);
+	return bFileOpen && CUtils::GetBoolInput("Launch znc now?", true);
 }
 
 bool CZNC::ParseConfig(const CString& sConfig)
@@ -887,9 +892,6 @@ bool CZNC::RehashConfig(CString& sError)
 	m_msDelUsers = m_msUsers;
 	m_msUsers.clear();
 
-	// Make sure that timer doesn't have a stale iterator
-	DisableConnectUser();
-
 	if (DoRehash(sError)) {
 #ifdef _MODULES
 		GetModules().OnPostRehash();
@@ -909,9 +911,6 @@ bool CZNC::RehashConfig(CString& sError)
 		m_msDelUsers.erase(m_msDelUsers.begin());
 	}
 
-	// Make sure that users that want to connect do so
-	RestartConnectUser();
-
 	return false;
 }
 
@@ -966,9 +965,6 @@ bool CZNC::DoRehash(CString& sError)
 		m_vpListeners.erase(m_vpListeners.begin());
 	}
 
-	// Make sure that timer doesn't have a stale iterator
-	DisableConnectUser();
-
 	CString sLine;
 	bool bCommented = false;	// support for /**/ style comments
 	CUser* pUser = NULL;	// Used to keep track of which user block we are in
@@ -984,9 +980,8 @@ bool CZNC::DoRehash(CString& sError)
 	while (File.ReadLine(sLine)) {
 		uLineNum++;
 
-		while ((sLine.Right(1) == "\r") || (sLine.Right(1) == "\n")) {
-			sLine.Trim();
-		}
+		// Remove all leading / trailing spaces and line endings
+		sLine.Trim();
 
 		if ((sLine.empty()) || (sLine[0] == '#') || (sLine.Left(2) == "//")) {
 			continue;
@@ -1024,7 +1019,7 @@ bool CZNC::DoRehash(CString& sError)
 
 				if (pUser) {
 					if (pChan) {
-						if (sTag.CaseCmp("Chan") == 0) {
+						if (sTag.Equals("Chan")) {
 							// Save the channel name, because AddChan
 							// deletes the CChannel*, if adding fails
 							sError = pChan->GetName();
@@ -1037,7 +1032,7 @@ bool CZNC::DoRehash(CString& sError)
 							pChan = NULL;
 							continue;
 						}
-					} else if (sTag.CaseCmp("User") == 0) {
+					} else if (sTag.Equals("User")) {
 						CString sErr;
 
 						if (pRealUser) {
@@ -1068,7 +1063,7 @@ bool CZNC::DoRehash(CString& sError)
 						continue;
 					}
 				}
-			} else if (sTag.CaseCmp("User") == 0) {
+			} else if (sTag.Equals("User")) {
 				if (pUser) {
 					sError = "You may not nest <User> tags inside of other <User> tags.";
 					CUtils::PrintError(sError);
@@ -1109,7 +1104,7 @@ bool CZNC::DoRehash(CString& sError)
 				}
 
 				continue;
-			} else if (sTag.CaseCmp("Chan") == 0) {
+			} else if (sTag.Equals("Chan")) {
 				if (!pUser) {
 					sError = "<Chan> tags must be nested inside of a <User> tag.";
 					CUtils::PrintError(sError);
@@ -1136,128 +1131,151 @@ bool CZNC::DoRehash(CString& sError)
 		if ((!sName.empty()) && (!sValue.empty())) {
 			if (pUser) {
 				if (pChan) {
-					if (sName.CaseCmp("Buffer") == 0) {
+					if (sName.Equals("Buffer")) {
 						pChan->SetBufferCount(strtoul(sValue.c_str(), NULL, 10));
 						continue;
-					} else if (sName.CaseCmp("KeepBuffer") == 0) {
-						pChan->SetKeepBuffer((sValue.CaseCmp("true") == 0));
+					} else if (sName.Equals("KeepBuffer")) {
+						pChan->SetKeepBuffer(sValue.Equals("true"));
 						continue;
-					} else if (sName.CaseCmp("Detached") == 0) {
-						pChan->SetDetached((sValue.CaseCmp("true") == 0));
+					} else if (sName.Equals("Detached")) {
+						pChan->SetDetached(sValue.Equals("true"));
 						continue;
-					} else if (sName.CaseCmp("AutoCycle") == 0) {
-						pChan->SetAutoCycle((sValue.CaseCmp("true") == 0));
+					} else if (sName.Equals("AutoCycle")) {
+						if (sValue.Equals("true")) {
+							CUtils::PrintError("WARNING: AutoCycle has been removed, instead try -> LoadModule = autocycle " + pChan->GetName());
+						}
 						continue;
-					} else if (sName.CaseCmp("Key") == 0) {
+					} else if (sName.Equals("Key")) {
 						pChan->SetKey(sValue);
 						continue;
-					} else if (sName.CaseCmp("Modes") == 0) {
+					} else if (sName.Equals("Modes")) {
 						pChan->SetDefaultModes(sValue);
 						continue;
 					}
 				} else {
-					if (sName.CaseCmp("Buffer") == 0) {
+					if (sName.Equals("Buffer")) {
 						pUser->SetBufferCount(strtoul(sValue.c_str(), NULL, 10));
 						continue;
-					} else if (sName.CaseCmp("KeepBuffer") == 0) {
-						pUser->SetKeepBuffer((sValue.CaseCmp("true") == 0));
+					} else if (sName.Equals("KeepBuffer")) {
+						pUser->SetKeepBuffer(sValue.Equals("true"));
 						continue;
-					} else if (sName.CaseCmp("AutoCycle") == 0) {
-						pUser->SetAutoCycle((sValue.CaseCmp("true") == 0));
-						continue;
-					} else if (sName.CaseCmp("Nick") == 0) {
+					} else if (sName.Equals("Nick")) {
 						pUser->SetNick(sValue);
 						continue;
-					} else if (sName.CaseCmp("CTCPReply") == 0) {
+					} else if (sName.Equals("CTCPReply")) {
 						pUser->AddCTCPReply(sValue.Token(0), sValue.Token(1, true));
 						continue;
-					} else if (sName.CaseCmp("QuitMsg") == 0) {
+					} else if (sName.Equals("QuitMsg")) {
 						pUser->SetQuitMsg(sValue);
 						continue;
-					} else if (sName.CaseCmp("AltNick") == 0) {
+					} else if (sName.Equals("AltNick")) {
 						pUser->SetAltNick(sValue);
 						continue;
-					} else if (sName.CaseCmp("AwaySuffix") == 0) {
+					} else if (sName.Equals("AwaySuffix")) {
 						CUtils::PrintMessage("WARNING: AwaySuffix has been depricated, instead try -> LoadModule = awaynick %nick%_" + sValue);
 						continue;
-					} else if (sName.CaseCmp("Pass") == 0) {
+					} else if (sName.Equals("AutoCycle")) {
+						if (sValue.Equals("true")) {
+							CUtils::PrintError("WARNING: AutoCycle has been removed, instead try -> LoadModule = autocycle");
+						}
+						continue;
+					} else if (sName.Equals("Pass")) {
+						// There are different formats for this available:
+						// Pass = <plain text>
+						// Pass = <md5 hash> -
+						// Pass = plain#<plain text>
+						// Pass = md5#<md5 hash>
+						// Pass = md5#<salted md5 hash>#<salt>#
+						// The last one is the md5 hash of 'password' + 'salt'
 						if (sValue.Right(1) == "-") {
 							sValue.RightChomp();
 							sValue.Trim();
 							pUser->SetPass(sValue, true);
 						} else {
-							pUser->SetPass(sValue, false);
+							CString sMethod = sValue.Token(0, false, "#");
+							CString sPass = sValue.Token(1, true, "#");
+							if (sMethod == "md5") {
+								CString sSalt = sPass.Token(1, false, "#");
+								sPass = sPass.Token(0, false, "#");
+								pUser->SetPass(sPass, true, sSalt);
+							} else if (sMethod == "plain") {
+								pUser->SetPass(sPass, false);
+							} else {
+								pUser->SetPass(sValue, false);
+							}
 						}
 
 						continue;
-					} else if (sName.CaseCmp("MultiClients") == 0) {
-						pUser->SetMultiClients(sValue.CaseCmp("true") == 0);
+					} else if (sName.Equals("MultiClients")) {
+						pUser->SetMultiClients(sValue.Equals("true"));
 						continue;
-					} else if (sName.CaseCmp("BounceDCCs") == 0) {
-						pUser->SetBounceDCCs(sValue.CaseCmp("true") == 0);
+					} else if (sName.Equals("BounceDCCs")) {
+						pUser->SetBounceDCCs(sValue.Equals("true"));
 						continue;
-					} else if (sName.CaseCmp("Ident") == 0) {
+					} else if (sName.Equals("Ident")) {
 						pUser->SetIdent(sValue);
 						continue;
-					} else if (sName.CaseCmp("DenyLoadMod") == 0) {
-						pUser->SetDenyLoadMod((sValue.CaseCmp("TRUE") == 0));
+					} else if (sName.Equals("DenyLoadMod")) {
+						pUser->SetDenyLoadMod(sValue.Equals("true"));
 						continue;
-					} else if (sName.CaseCmp("Admin") == 0) {
-						pUser->SetAdmin((sValue.CaseCmp("TRUE") == 0));
+					} else if (sName.Equals("Admin")) {
+						pUser->SetAdmin(sValue.Equals("true"));
 						continue;
-					} else if (sName.CaseCmp("DenySetVHost") == 0) {
-						pUser->SetDenySetVHost((sValue.CaseCmp("TRUE") == 0));
+					} else if (sName.Equals("DenySetVHost")) {
+						pUser->SetDenySetVHost(sValue.Equals("true"));
 						continue;
-					} else if (sName.CaseCmp("StatusPrefix") == 0) {
+					} else if (sName.Equals("StatusPrefix")) {
 						if (!pUser->SetStatusPrefix(sValue)) {
 							sError = "Invalid StatusPrefix [" + sValue + "] Must be 1-5 chars, no spaces.";
 							CUtils::PrintError(sError);
 							return false;
 						}
 						continue;
-					} else if (sName.CaseCmp("DCCLookupMethod") == 0) {
-						pUser->SetUseClientIP((sValue.CaseCmp("Client") == 0));
+					} else if (sName.Equals("DCCLookupMethod")) {
+						pUser->SetUseClientIP(sValue.Equals("Client"));
 						continue;
-					} else if (sName.CaseCmp("RealName") == 0) {
+					} else if (sName.Equals("RealName")) {
 						pUser->SetRealName(sValue);
 						continue;
-					} else if (sName.CaseCmp("KeepNick") == 0) {
-						pUser->SetKeepNick((sValue.CaseCmp("true") == 0));
+					} else if (sName.Equals("KeepNick")) {
+						if (sValue.Equals("true")) {
+							CUtils::PrintError("WARNING: KeepNick has been deprecated, instead try -> LoadModule = keepnick");
+						}
 						continue;
-					} else if (sName.CaseCmp("ChanModes") == 0) {
+					} else if (sName.Equals("ChanModes")) {
 						pUser->SetDefaultChanModes(sValue);
 						continue;
-					} else if (sName.CaseCmp("VHost") == 0) {
+					} else if (sName.Equals("VHost")) {
 						pUser->SetVHost(sValue);
 						continue;
-					} else if (sName.CaseCmp("Allow") == 0) {
+					} else if (sName.Equals("Allow")) {
 						pUser->AddAllowedHost(sValue);
 						continue;
-					} else if (sName.CaseCmp("Server") == 0) {
+					} else if (sName.Equals("Server")) {
 						CUtils::PrintAction("Adding Server [" + sValue + "]");
 						CUtils::PrintStatus(pUser->AddServer(sValue));
 						continue;
-					} else if (sName.CaseCmp("Chan") == 0) {
+					} else if (sName.Equals("Chan")) {
 						pUser->AddChan(sValue, true);
 						continue;
-					} else if (sName.CaseCmp("TimestampFormat") == 0) {
+					} else if (sName.Equals("TimestampFormat")) {
 						pUser->SetTimestampFormat(sValue);
 						continue;
-					} else if (sName.CaseCmp("AppendTimestamp") == 0) {
+					} else if (sName.Equals("AppendTimestamp")) {
 						pUser->SetTimestampAppend(sValue.ToBool());
 						continue;
-					} else if (sName.CaseCmp("PrependTimestamp") == 0) {
+					} else if (sName.Equals("PrependTimestamp")) {
 						pUser->SetTimestampPrepend(sValue.ToBool());
 						continue;
-					} else if (sName.CaseCmp("Timestamp") == 0) {
-						if (sValue.Trim_n().CaseCmp("true") != 0) {
-							if (sValue.Trim_n().CaseCmp("append") == 0) {
+					} else if (sName.Equals("Timestamp")) {
+						if (!sValue.Trim_n().Equals("true")) {
+							if (sValue.Trim_n().Equals("append")) {
 								pUser->SetTimestampAppend(true);
 								pUser->SetTimestampPrepend(false);
-							} else if (sValue.Trim_n().CaseCmp("prepend") == 0) {
+							} else if (sValue.Trim_n().Equals("prepend")) {
 								pUser->SetTimestampAppend(false);
 								pUser->SetTimestampPrepend(true);
-							} else if (sValue.Trim_n().CaseCmp("false") == 0) {
+							} else if (sValue.Trim_n().Equals("false")) {
 								pUser->SetTimestampAppend(false);
 								pUser->SetTimestampPrepend(false);
 							} else {
@@ -1265,35 +1283,32 @@ bool CZNC::DoRehash(CString& sError)
 							}
 						}
 						continue;
-					} else if (sName.CaseCmp("TimezoneOffset") == 0) {
+					} else if (sName.Equals("TimezoneOffset")) {
 						pUser->SetTimezoneOffset(sValue.ToDouble()); // there is no ToFloat()
 						continue;
-					} else if (sName.CaseCmp("JoinTries") == 0) {
+					} else if (sName.Equals("JoinTries")) {
 						pUser->SetJoinTries(sValue.ToUInt());
 						continue;
-					} else if (sName.CaseCmp("LoadModule") == 0) {
+					} else if (sName.Equals("MaxJoins")) {
+						pUser->SetMaxJoins(sValue.ToUInt());
+						continue;
+					} else if (sName.Equals("LoadModule")) {
 						CString sModName = sValue.Token(0);
 						CUtils::PrintAction("Loading Module [" + sModName + "]");
 #ifdef _MODULES
 						CString sModRet;
 						CString sArgs = sValue.Token(1, true);
 
-						try {
-							bool bModRet = pUser->GetModules().LoadModule(sModName, sArgs, pUser, sModRet);
+						bool bModRet = pUser->GetModules().LoadModule(sModName, sArgs, pUser, sModRet);
 
-							// If the module was loaded, sModRet contains
-							// "Loaded Module [name] ..." and we strip away this beginning.
-							if (bModRet)
-								sModRet = sModRet.Token(1, true, sModName + "] ");
+						// If the module was loaded, sModRet contains
+						// "Loaded Module [name] ..." and we strip away this beginning.
+						if (bModRet)
+							sModRet = sModRet.Token(1, true, sModName + "] ");
 
-							CUtils::PrintStatus(bModRet, sModRet);
-							if (!bModRet) {
-								sError = sModRet;
-								return false;
-							}
-						} catch (CException e) {
-							sError = "Caught exception while loading [" + sModName + "]";
-							CUtils::PrintStatus(false, sModRet);
+						CUtils::PrintStatus(bModRet, sModRet);
+						if (!bModRet) {
+							sError = sModRet;
 							return false;
 						}
 #else
@@ -1304,9 +1319,9 @@ bool CZNC::DoRehash(CString& sError)
 					}
 				}
 			} else {
-				if (sName.CaseCmp("Listen") == 0 || sName.CaseCmp("ListenPort") == 0 || sName.CaseCmp("Listen6") == 0) {
+				if (sName.Equals("Listen") || sName.Equals("ListenPort") || sName.Equals("Listen6")) {
 					bool bSSL = false;
-					bool bIPV6 = (sName.CaseCmp("Listen6") == 0);
+					bool bIPV6 = sName.Equals("Listen6");
 					CString sPort;
 
 					CString sBindHost;
@@ -1394,7 +1409,7 @@ bool CZNC::DoRehash(CString& sError)
 					CUtils::PrintStatus(true);
 
 					continue;
-				} else if (sName.CaseCmp("LoadModule") == 0) {
+				} else if (sName.Equals("LoadModule")) {
 #ifdef _MODULES
 					CString sModName = sValue.Token(0);
 					CString sArgs = sValue.Token(1, true);
@@ -1410,29 +1425,29 @@ bool CZNC::DoRehash(CString& sError)
 					CUtils::PrintError("Modules are not enabled.");
 #endif
 					continue;
-				} else if (sName.CaseCmp("ISpoofFormat") == 0) {
+				} else if (sName.Equals("ISpoofFormat")) {
 					m_sISpoofFormat = sValue;
 					continue;
-				} else if (sName.CaseCmp("ISpoofFile") == 0) {
+				} else if (sName.Equals("ISpoofFile")) {
 					if (sValue.Left(2) == "~/") {
 						sValue.LeftChomp(2);
 						sValue = GetHomePath() + "/" + sValue;
 					}
 					m_sISpoofFile = sValue;
 					continue;
-				} else if (sName.CaseCmp("MOTD") == 0) {
+				} else if (sName.Equals("MOTD")) {
 					AddMotd(sValue);
 					continue;
-				} else if (sName.CaseCmp("VHost") == 0) {
+				} else if (sName.Equals("VHost")) {
 					AddVHost(sValue);
 					continue;
-				} else if (sName.CaseCmp("PidFile") == 0) {
+				} else if (sName.Equals("PidFile")) {
 					m_sPidFile = sValue;
 					continue;
-				} else if (sName.CaseCmp("StatusPrefix") == 0) {
+				} else if (sName.Equals("StatusPrefix")) {
 					m_sStatusPrefix = sValue;
 					continue;
-				} else if (sName.CaseCmp("ConnectDelay") == 0) {
+				} else if (sName.Equals("ConnectDelay")) {
 					m_uiConnectDelay = sValue.ToUInt();
 					continue;
 				}
@@ -1440,7 +1455,7 @@ bool CZNC::DoRehash(CString& sError)
 
 		}
 
-		if (sName.CaseCmp("GM:", 3) == 0)
+		if (sName.Equals("GM:", false, 3))
 		{ // GM: prefix is a pass through to config lines for global modules
 			CGlobalModuleConfigLine cTmp;
 			cTmp.m_sName = sName.substr(3, CString::npos);
@@ -1469,43 +1484,31 @@ bool CZNC::DoRehash(CString& sError)
 		if (!pOldMod) {
 			CUtils::PrintAction("Loading Global Module [" + sModName + "]");
 
-			try {
-				bool bModRet = GetModules().LoadModule(sModName, sArgs, NULL, sModRet);
+			bool bModRet = GetModules().LoadModule(sModName, sArgs, NULL, sModRet);
 
-				// If the module was loaded, sModRet contains
-				// "Loaded Module [name] ..." and we strip away this beginning.
-				if (bModRet)
-					sModRet = sModRet.Token(1, true, sModName + "] ");
+			// If the module was loaded, sModRet contains
+			// "Loaded Module [name] ..." and we strip away this beginning.
+			if (bModRet)
+				sModRet = sModRet.Token(1, true, sModName + "] ");
 
-				CUtils::PrintStatus(bModRet, sModRet);
-				if (!bModRet) {
-					sError = sModRet;
-					return false;
-				}
-			} catch (CException e) {
-				sError = "Caught an exception while loading [" + sModName + "]";
-				CUtils::PrintStatus(false, sModRet);
+			CUtils::PrintStatus(bModRet, sModRet);
+			if (!bModRet) {
+				sError = sModRet;
 				return false;
 			}
 		} else if (pOldMod->GetArgs() != sArgs) {
 			CUtils::PrintAction("Reloading Global Module [" + sModName + "]");
 
-			try {
-				bool bModRet = GetModules().ReloadModule(sModName, sArgs, NULL, sModRet);
+			bool bModRet = GetModules().ReloadModule(sModName, sArgs, NULL, sModRet);
 
-				// If the module was loaded, sModRet contains
-				// "Loaded Module [name] ..." and we strip away this beginning.
-				if (bModRet)
-					sModRet = sModRet.Token(1, true, sModName + "] ");
+			// If the module was loaded, sModRet contains
+			// "Loaded Module [name] ..." and we strip away this beginning.
+			if (bModRet)
+				sModRet = sModRet.Token(1, true, sModName + "] ");
 
-				CUtils::PrintStatus(bModRet, sModRet);
-				if (!bModRet) {
-					sError = sModRet;
-					return false;
-				}
-			} catch (CException e) {
-				sError = "Caught an exception while reloading [" + sModName + "]";
-				CUtils::PrintStatus(false, sModRet);
+			CUtils::PrintStatus(bModRet, sModRet);
+			if (!bModRet) {
+				sError = sModRet;
 				return false;
 			}
 		} else
@@ -1565,7 +1568,7 @@ bool CZNC::DoRehash(CString& sError)
 	}
 
 	// Make sure that users that want to connect do so
-	RestartConnectUser();
+	EnableConnectUser();
 
 	return true;
 }
@@ -1580,7 +1583,7 @@ bool CZNC::AddVHost(const CString& sHost) {
 	}
 
 	for (unsigned int a = 0; a < m_vsVHosts.size(); a++) {
-		if (m_vsVHosts[a].CaseCmp(sHost) == 0) {
+		if (m_vsVHosts[a].Equals(sHost)) {
 			return false;
 		}
 	}
@@ -1590,8 +1593,15 @@ bool CZNC::AddVHost(const CString& sHost) {
 }
 
 bool CZNC::RemVHost(const CString& sHost) {
-	// @todo
-	return true;
+	VCString::iterator it;
+	for (it = m_vsVHosts.begin(); it != m_vsVHosts.end(); it++) {
+		if (sHost.Equals(*it)) {
+			m_vsVHosts.erase(it);
+			return true;
+		}
+	}
+
+	return false;
 }
 
 void CZNC::Broadcast(const CString& sMessage, bool bAdminOnly,
@@ -1727,44 +1737,69 @@ public:
 	CConnectUserTimer(int iSecs) : CCron() {
 		SetName("Connect users");
 		Start(iSecs);
-		m_itUserIter = CZNC::Get().GetUserMap().begin();
+		m_uiPosNextUser = 0;
+		// Don't wait iSecs seconds for first timer run
+		m_bRunOnNextCall = true;
 	}
 	virtual ~CConnectUserTimer() {}
 
 protected:
 	virtual void RunJob() {
 		unsigned int uiUserCount;
-		map<CString,CUser*>::const_iterator end;
 		bool bUsersLeft = false;
+		const map<CString,CUser*>& mUsers = CZNC::Get().GetUserMap();
+		map<CString,CUser*>::const_iterator it = mUsers.begin();
 
 		uiUserCount = CZNC::Get().GetUserMap().size();
-		end = CZNC::Get().GetUserMap().end();
+
+		if (m_uiPosNextUser >= uiUserCount) {
+			m_uiPosNextUser = 0;
+		}
+
+		for (unsigned int i = 0; i < m_uiPosNextUser; i++) {
+			it++;
+		}
 
 		// Try to connect each user, if this doesnt work, abort
 		for (unsigned int i = 0; i < uiUserCount; i++) {
-			if (m_itUserIter == end) {
-				m_itUserIter = CZNC::Get().GetUserMap().begin();
-			}
+			if (it == mUsers.end())
+				it = mUsers.begin();
 
-			CUser* pUser = m_itUserIter->second;
+			CUser* pUser = it->second;
+			it++;
+			m_uiPosNextUser = (m_uiPosNextUser + 1) % uiUserCount;
 
-			m_itUserIter++;
+			// Is this user disconnected?
+			if (pUser->GetIRCSock() != NULL)
+				continue;
 
-			// Is this user disconnected and does he want to connect?
-			if (pUser->GetIRCSock() == NULL && pUser->GetIRCConnectEnabled())
-				bUsersLeft = true;
+			// Does this user want to connect?
+			if (!pUser->GetIRCConnectEnabled())
+				continue;
+
+			// Does this user have any servers?
+			if (!pUser->HasServers())
+				continue;
+
+			// The timer runs until it once didn't find any users to connect
+			bUsersLeft = true;
+
+			DEBUG_ONLY(cout << "Connecting user [" << pUser->GetUserName()
+					<< "]" << endl);
 
 			if (CZNC::Get().ConnectUser(pUser))
-				// Wait until next time timer fires
+				// User connecting, wait until next time timer fires
 				return;
 		}
 
-		if (bUsersLeft == false)
+		if (bUsersLeft == false) {
+			DEBUG_ONLY(cout << "ConnectUserTimer done" << endl);
 			CZNC::Get().DisableConnectUser();
+		}
 	}
 
 private:
-	map<CString,CUser*>::const_iterator	m_itUserIter;
+	size_t	m_uiPosNextUser;
 };
 
 void CZNC::EnableConnectUser() {
@@ -1783,16 +1818,3 @@ void CZNC::DisableConnectUser() {
 	m_pConnectUserTimer->Stop();
 	m_pConnectUserTimer = NULL;
 }
-
-void CZNC::RestartConnectUser() {
-	DisableConnectUser();
-
-	map<CString, CUser*>::iterator end = m_msUsers.end();
-	for (map<CString,CUser*>::iterator it = m_msUsers.begin(); it != end; it++) {
-		// If there is a user without irc socket we need the timer
-		if (it->second->GetIRCSock() == NULL) {
-			EnableConnectUser();
-			return;
-		}
-	}
-}
