@@ -100,17 +100,17 @@ public:
 	bool DeletePidFile();
 	CUser* GetUser(const CString& sUser);
 	Csock* FindSockByName(const CString& sSockName);
-	bool IsHostAllowed(const CString& sHostMask);
+	bool IsHostAllowed(const CString& sHostMask) const;
 	void InitDirs(const CString& sArgvPath, const CString& sDataDir);
 	bool OnBoot();
 	CString ExpandConfigPath(const CString& sConfigFile);
-	bool WriteNewConfig(const CString& sConfig);
+	bool WriteNewConfig(CString& sConfigFile);
 	bool WriteConfig();
 	bool ParseConfig(const CString& sConfig);
 	bool RehashConfig(CString& sError);
 	static CString GetVersion();
 	static CString GetTag(bool bIncludeVersion = true);
-	CString GetUptime();
+	CString GetUptime() const;
 	// This returns the path to the .so and to the data dir
 	// which is where static data (webadmin skins) are saved
 	bool FindModPath(const CString& sModule, CString& sModPath,
@@ -139,7 +139,7 @@ public:
 	// !Setters
 
 	// Getters
-	bool GetNeedRehash() { return m_bNeedRehash; }
+	bool GetNeedRehash() const { return m_bNeedRehash; }
 	CSockManager& GetManager() { return m_Manager; }
 #ifdef _MODULES
 	CGlobalModules& GetModules() { return *m_pModules; }
@@ -180,8 +180,6 @@ public:
 	// This creates a CConnectUserTimer if we haven't got one yet
 	void EnableConnectUser();
 	void DisableConnectUser();
-	// This needs to be called if anything was added / removed to m_msUsers
-	void RestartConnectUser();
 
 private:
 	bool DoRehash(CString& sError);
@@ -225,6 +223,34 @@ protected:
 	CConnectUserTimer		*m_pConnectUserTimer;
 };
 
+class CRealListener : public Csock {
+public:
+	CRealListener() : Csock() {}
+	~CRealListener() {}
+
+	virtual bool ConnectionFrom(const CString& sHost, unsigned short uPort) {
+		DEBUG_ONLY(cout << GetSockName() << " == ConnectionFrom(" << sHost << ", " << uPort << ")" << endl);
+		return CZNC::Get().IsHostAllowed(sHost);
+	}
+
+	virtual Csock* GetSockObj(const CString& sHost, unsigned short uPort) {
+		CClient *pClient = new CClient(sHost, uPort);
+#ifdef _MODULES
+		CZNC::Get().GetModules().OnClientConnect(pClient, sHost, uPort);
+#endif
+		return pClient;
+	}
+
+	virtual void SockError(int iErrno) {
+		DEBUG_ONLY(cout << GetSockName() << " == SockError(" << strerror(iErrno) << ")" << endl);
+		if (iErrno == EMFILE) {
+			// We have too many open fds, let's close this listening port to be able to continue
+			// to work, next rehash will (try to) reopen it.
+			Close();
+		}
+	}
+};
+
 class CListener {
 public:
 	CListener(unsigned short uPort, const CString& sBindHost, bool bSSL, bool bIPV6) {
@@ -232,12 +258,12 @@ public:
 		m_sBindHost = sBindHost;
 		m_bSSL = bSSL;
 		m_bIPV6 = bIPV6;
-		m_pClient = NULL;
+		m_pListener = NULL;
 	}
 
 	virtual ~CListener() {
-		if (m_pClient)
-			CZNC::Get().GetManager().DelSockByAddr(m_pClient);
+		if (m_pListener)
+			CZNC::Get().GetManager().DelSockByAddr(m_pListener);
 	}
 
 	// Setters
@@ -255,21 +281,22 @@ public:
 	// !Getters
 
 	bool Listen() {
-		if (!m_uPort || m_pClient) {
+		if (!m_uPort || m_pListener) {
 			return false;
 		}
 
-		m_pClient = new CClient;
+		m_pListener = new CRealListener;
 
 		bool bSSL = false;
 #ifdef HAVE_LIBSSL
 		if (IsSSL()) {
 			bSSL = true;
-			m_pClient->SetPemLocation(CZNC::Get().GetPemLocation());
+			m_pListener->SetPemLocation(CZNC::Get().GetPemLocation());
 		}
 #endif
 
-		return CZNC::Get().GetManager().ListenHost(m_uPort, "_LISTENER", m_sBindHost, bSSL, SOMAXCONN, m_pClient, 0, m_bIPV6);
+		return CZNC::Get().GetManager().ListenHost(m_uPort, "_LISTENER", m_sBindHost, bSSL, SOMAXCONN,
+				m_pListener, 0, m_bIPV6);
 	}
 private:
 protected:
@@ -277,7 +304,7 @@ protected:
 	bool			m_bIPV6;
 	unsigned short	m_uPort;
 	CString			m_sBindHost;
-	CClient*		m_pClient;
+	CRealListener*		m_pListener;
 };
 
 #endif // !_ZNC_H
